@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,18 +39,14 @@
 #include <string>
 #include <type_traits>
 
-#include "frequency_monitor.hpp"
+#include "efficiency_monitor.hpp"
 
 #include "testing_matmul.hpp"
 
-#include "utility.hpp"
-#include <algorithm>
-#undef I
-
 using namespace roc; // For emulated program_options
 using namespace std::literals; // For std::string literals of form "str"s
 
-struct perf_matmul: hipblaslt_test_valid
+struct perf_matmul : hipblaslt_test_valid
 {
     void operator()(const Arguments& arg)
     {
@@ -62,7 +58,11 @@ struct perf_matmul: hipblaslt_test_valid
     }
 };
 
-int run_bench_test(Arguments& arg, const std::string& filter, bool any_stride, bool yaml = false)
+int run_bench_test(Arguments&         arg,
+                   const std::string& filter,
+                   bool               any_stride,
+                   hipDeviceProp_t&   props,
+                   bool               yaml = false)
 {
     hipblaslt_cout << std::setiosflags(std::ios::fixed)
                    << std::setprecision(7); // Set precision to 7 digits
@@ -133,7 +133,7 @@ int run_bench_test(Arguments& arg, const std::string& filter, bool any_stride, b
         int64_t min_stride_c = arg.ldc[i] * arg.N[i];
         int64_t min_stride_d = arg.ldd[i] * arg.N[i];
         int64_t min_stride_e = arg.lde[i] * arg.N[i];
-        if(!any_stride && arg.stride_a[i] < min_stride_a)
+        if(!any_stride && arg.stride_a[i] < min_stride_a && !arg.swizzle_a)
         {
             //hipblaslt_cout << "hipblaslt-bench INFO: stride_a < min_stride_a, set stride_a = "
             //               << min_stride_a << std::endl;
@@ -165,15 +165,49 @@ int run_bench_test(Arguments& arg, const std::string& filter, bool any_stride, b
         }
     }
 
+    // Check for F8 OCP data types and convert to NANOO
+    {
+        std::string deviceFullString(props.gcnArchName);
+        std::string deviceString = deviceFullString.substr(0, deviceFullString.find(":"));
+
+        bool isGFX942 = deviceString.find("gfx942") != std::string::npos;
+
+        if(isGFX942)
+        {
+            auto convertF8Type = [](hipDataType type) {
+                if(type == HIP_R_8F_E4M3)
+                {
+                    hipblaslt_cerr << "hipblaslt-bench INFO: Using f8_fnuz_r instead of f8_r"
+                                   << std::endl;
+                    return HIP_R_8F_E4M3_FNUZ;
+                }
+                if(type == HIP_R_8F_E5M2)
+                {
+                    hipblaslt_cerr << "hipblaslt-bench INFO: Using b8_fnuz_r instead of b8_r"
+                                   << std::endl;
+                    return HIP_R_8F_E5M2_FNUZ;
+                }
+                else
+                    return type;
+            };
+
+            arg.a_type   = convertF8Type(arg.a_type);
+            arg.b_type   = convertF8Type(arg.b_type);
+            arg.c_type   = convertF8Type(arg.c_type);
+            arg.d_type   = convertF8Type(arg.d_type);
+            arg.aux_type = convertF8Type(arg.aux_type);
+        }
+    }
+
     perf_matmul{}(arg);
     return 0;
 }
 
-int hipblaslt_bench_datafile(const std::string& filter, bool any_stride)
+int hipblaslt_bench_datafile(const std::string& filter, bool any_stride, hipDeviceProp_t& props)
 {
     int ret = 0;
     for(Arguments arg : HipBlasLt_TestData())
-        ret |= run_bench_test(arg, filter, any_stride, true);
+        ret |= run_bench_test(arg, filter, any_stride, props, true);
     test_cleanup::cleanup();
     return ret;
 }
@@ -232,17 +266,6 @@ bool tuning_path_compare_git_version(const char* tuningEnv)
     return false;
 }
 
-void hipblaslt_print_version(void)
-{
-    int                    version;
-    char                   git_version[128];
-    hipblaslt_local_handle handle;
-    hipblasLtGetVersion(handle, &version);
-    hipblasLtGetGitRevision(handle, &git_version[0]);
-    hipblaslt_cout << "hipBLASLt version: " << version << std::endl;
-    hipblaslt_cout << "hipBLASLt git version: " << git_version << std::endl;
-}
-
 int main(int argc, char* argv[])
 try
 {
@@ -263,8 +286,11 @@ try
     std::string initialization;
     std::string filter;
     std::string activation_type;
+    std::string aux_type;
     int         scaleAFormat;
     int         scaleBFormat;
+    int         scaleCFormat;
+    int         scaleDFormat;
     int         device_id;
     int         flags             = 0;
     bool        datafile          = hipblaslt_parse_data(argc, argv);
@@ -283,7 +309,8 @@ try
     std::vector<int64_t>  stride_a, stride_b, stride_c, stride_d, stride_e;
     std::vector<uint32_t> gsu_vector, wgm_vector;
     arg.init(); // set all defaults
-    const char* tuningEnv = getenv("HIPBLASLT_TUNING_FILE");
+    const char* tuningEnv          = getenv("HIPBLASLT_TUNING_FILE");
+    const char* tuningMaxWorkSpace = getenv("HIPBLASLT_TUNING_USER_MAX_WORKSPACE");
     if(tuningEnv)
     {
         bool tuning_success = tuning_path_compare_git_version(tuningEnv);
@@ -390,11 +417,11 @@ try
 
         ("compute_input_typeA",
          value<std::string>(&compute_input_typeA), "Precision of computation input A. "
-         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, The default value indicates that the compute_input_typeA has no effect.")
+         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, f8_fnuz_r, bf8_fnuz_r, The default value indicates that the compute_input_typeA has no effect.")
 
         ("compute_input_typeB",
          value<std::string>(&compute_input_typeB), "Precision of computation input B. "
-         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, The default value indicates that the compute_input_typeA has no effect.")
+         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, f8_fnuz_r, bf8_fnuz_r, The default value indicates that the compute_input_typeA has no effect.")
 
         ("scale_type",
          value<std::string>(&scale_type), "Precision of scalar. "
@@ -403,7 +430,7 @@ try
         ("initialization",
          value<std::string>(&initialization)->default_value("hpl"),
          "Initialize matrix data."
-         "Options: rand_int, trig_float, hpl(floating), special, zero")
+         "Options: rand_int, trig_float, hpl(floating), special, zero, norm_dist")
 
         ("transA",
          value<char>(&arg.transA)->default_value('N'),
@@ -413,6 +440,10 @@ try
          value<char>(&arg.transB)->default_value('N'),
          "N = no transpose, T = transpose")
 
+        ("swizzleA",
+         value<bool>(&arg.swizzle_a)->default_value(false),
+         "Enable tensor swizzling for A")
+
         ("batch_count",
          value<int32_t>(&arg.batch_count)->default_value(1),
          "Number of matrices. Only applicable to batched and strided_batched routines")
@@ -434,7 +465,7 @@ try
          "Cold Iterations to run before entering the timing loop")
 
         ("algo_method",
-         value<std::string>(&algo_method_str)->default_value(tuningEnv? "all" : "heuristic"),
+         value<std::string>(&algo_method_str)->default_value("heuristic"),
          "Use different algorithm search API. Options: heuristic, all, index.")
 
         ("solution_index",
@@ -447,7 +478,7 @@ try
 
         ("activation_type",
          value<std::string>(&activation_type)->default_value("none"),
-         "Options: none, gelu, relu")
+         "Options: none, gelu, relu, swish")
 
         ("activation_arg1",
          value<float>(&arg.activation_arg1)->default_value(0),
@@ -471,16 +502,40 @@ try
 
         ("scaleA",
          value<int>(&scaleAFormat)->default_value(0),
-         "Apply scale for A buffer. 0 = None, 1 = scalar, 2 = vector.")
+         "Apply scale for A buffer. 0 = None, 1 = scalar, 2 = vector, 3 = block.")
 
         ("scaleB",
          value<int>(&scaleBFormat)->default_value(0),
-         "Apply scale for B buffer. 0 = None, 1 = scalar, 2 = vector.")
+         "Apply scale for B buffer. 0 = None, 1 = scalar, 2 = vector, 3 = block.")
+
+        ("scaleC",
+         value<int>(&scaleCFormat)->default_value(0),
+         "Apply scale for C buffer. 0 = None, 1 = scalar")
+
+        ("scaleD",
+         value<int>(&scaleDFormat)->default_value(0),
+         "Apply scale for D buffer. 0 = None, 1 = scalar")
 
         ("scaleAlpha_vector",
          bool_switch(&arg.scaleAlpha_vector)->default_value(false),
          "Apply scaleAlpha vector")
 
+        ("scaleABlockRowSize",
+         value<uint32_t>(&arg.scaleABlockRowSize)->default_value(32u),
+         "Set the row size of scale block for A")
+
+        ("scaleABlockColSize",
+         value<uint32_t>(&arg.scaleABlockColSize)->default_value(1u),
+         "Set the column size of scale block for A")
+
+        ("scaleBBlockRowSize",
+         value<uint32_t>(&arg.scaleBBlockRowSize)->default_value(1u),
+         "Set the row size of scale block for B")
+
+        ("scaleBBlockColSize",
+         value<uint32_t>(&arg.scaleBBlockColSize)->default_value(32u),
+         "Set the column size of scale block for B")
+
         ("amaxScaleA",
          bool_switch(&arg.amaxScaleA)->default_value(false),
          "Apply scale for A buffer by abs max of A buffer")
@@ -497,6 +552,10 @@ try
          bool_switch(&arg.use_e)->default_value(false),
          "Apply AUX output/ gradient input")
 
+        ("aux_type",
+         value<std::string>(&aux_type), "Used with --use_e. Precision of AUX output (matrix E)."
+	 "Options: f16_r, default (same with D type)")
+
         ("gradient",
          bool_switch(&arg.gradient)->default_value(false),
          "Enable gradient")
@@ -518,7 +577,7 @@ try
          "C and D are stored in same memory")
 
         ("workspace",
-         value<size_t>(&arg.user_allocated_workspace)->default_value(128 * 1024 * 1024),
+         value<size_t>(&arg.user_allocated_workspace)->default_value(tuningEnv && tuningMaxWorkSpace ? atoi(tuningMaxWorkSpace) : 128 * 1024 * 1024),
          "Set fixed workspace memory size (bytes) instead of using hipblaslt managed memory")
 
         ("log_function_name",
@@ -611,11 +670,11 @@ try
     }
     else if(algo_method_str.compare("all") == 0)
     {
-        arg.algo_method = 1;
+        arg.algo_method = tuningEnv ? 0 : 1;
     }
     else if(algo_method_str.compare("index") == 0)
     {
-        arg.algo_method = tuningEnv ? 1 : 2;
+        arg.algo_method = tuningEnv ? 0 : 2;
     }
     else
     {
@@ -734,18 +793,19 @@ try
     }
 
     // Device Query
-    int64_t device_count = query_device_property();
+    hipDeviceProp_t props;
+    int64_t         device_count = query_device_property(device_id, props);
 
     hipblaslt_cout << std::endl;
     if(device_count <= device_id)
         throw std::invalid_argument("Invalid Device ID");
     set_device(device_id);
 
-    FrequencyMonitor& freq_monitor = getFrequencyMonitor();
-    freq_monitor.set_device_id(device_id);
+    EfficiencyMonitor& perf_monitor = getEfficiencyMonitor();
+    perf_monitor.set_device_id(device_id);
 
     if(datafile)
-        return hipblaslt_bench_datafile(filter, any_stride);
+        return hipblaslt_bench_datafile(filter, any_stride, props);
 
     // single bench run
 
@@ -780,6 +840,11 @@ try
     if(arg.d_type == HIPBLASLT_DATATYPE_INVALID)
         throw std::invalid_argument("Invalid value for --d_type " + d_type);
 
+    if(arg.c_type != arg.d_type)
+        throw std::invalid_argument(
+            "Invalid: --c_type " + std::string(hip_datatype_to_string(arg.c_type))
+            + " is not equal to --d_type " + std::string(hip_datatype_to_string(arg.d_type)));
+
     bool is_f16 = arg.a_type == HIP_R_16F || arg.a_type == HIP_R_16BF;
     bool is_f32 = arg.a_type == HIP_R_32F;
     arg.compute_type
@@ -818,6 +883,24 @@ try
 
     arg.bias_source = string_to_hipblaslt_bias_source(bias_source);
 
+    if(!(aux_type == "" || aux_type == "default" || arg.use_e))
+        hipblaslt_cerr << "warning: --use_e not set but --aux_type is provided" << std::endl;
+
+    arg.aux_type
+        = (aux_type == "" || aux_type == "default") ? arg.d_type : string_to_hip_datatype(aux_type);
+    if(arg.aux_type == HIPBLASLT_DATATYPE_INVALID)
+        throw std::invalid_argument("Invalid value for --aux_type " + aux_type);
+
+    if(arg.swizzle_a
+       && (arg.transA != 'T' || arg.transB != 'N'
+           || (arg.a_type != string_to_hip_datatype("f16_r")
+               && arg.a_type != string_to_hip_datatype("f8_fnuz_r")
+               && arg.a_type != string_to_hip_datatype("bf16_r"))))
+    {
+        hipblaslt_cerr << "For swizzle-A, problem type must be FP16 or BF16 or FP8 TN" << std::endl;
+        return 1;
+    }
+
     auto scaleInt2Enum = [](int s) {
         if(s == 0)
             return hipblaslt_scaling_format::none;
@@ -825,11 +908,62 @@ try
             return hipblaslt_scaling_format::Scalar;
         if(s == 2)
             return hipblaslt_scaling_format::Vector;
+        if(s == 3)
+            return hipblaslt_scaling_format::Block;
 
         return hipblaslt_scaling_format::none;
     };
     arg.scaleA = scaleInt2Enum(scaleAFormat);
     arg.scaleB = scaleInt2Enum(scaleBFormat);
+    arg.scaleC = scaleCFormat;
+    arg.scaleD = scaleDFormat;
+
+    // Validation for F4 and F6
+    if(arg.a_type == HIP_R_4F_E2M1_EXT || arg.a_type == HIP_R_6F_E2M3_EXT
+       || arg.a_type == HIP_R_6F_E3M2_EXT)
+    {
+        if(arg.scaleA != hipblaslt_scaling_format::Block)
+            throw std::invalid_argument("scaleA must be block format for F4 and F6 types");
+    }
+    if(arg.b_type == HIP_R_4F_E2M1_EXT || arg.b_type == HIP_R_6F_E2M3_EXT
+       || arg.b_type == HIP_R_6F_E3M2_EXT)
+    {
+        if(arg.scaleB != hipblaslt_scaling_format::Block)
+            throw std::invalid_argument("scaleB must be block format for F4 and F6 types");
+    }
+
+    // Block scaling only allows F8/F6/F4
+    if(arg.scaleA == hipblaslt_scaling_format::Block)
+    {
+        if(arg.a_type != HIP_R_8F_E4M3 && arg.a_type != HIP_R_8F_E5M2
+           && arg.a_type != HIP_R_4F_E2M1_EXT && arg.a_type != HIP_R_6F_E2M3_EXT
+           && arg.a_type != HIP_R_6F_E3M2_EXT)
+            throw std::invalid_argument("Invalid a_type for block scaling format: "s
+                                        + hip_datatype_to_string(arg.a_type));
+    }
+    if(arg.scaleB == hipblaslt_scaling_format::Block)
+    {
+        if(arg.b_type != HIP_R_8F_E4M3 && arg.b_type != HIP_R_8F_E5M2
+           && arg.b_type != HIP_R_4F_E2M1_EXT && arg.b_type != HIP_R_6F_E2M3_EXT
+           && arg.b_type != HIP_R_6F_E3M2_EXT)
+            throw std::invalid_argument("Invalid b_type for block scaling format: "s
+                                        + hip_datatype_to_string(arg.b_type));
+    }
+    if(arg.scaleA == hipblaslt_scaling_format::Block
+       || arg.scaleB == hipblaslt_scaling_format::Block)
+    {
+        if(arg.compute_type != HIPBLAS_COMPUTE_32F)
+            throw std::invalid_argument("Block scaling only supports f32 as compute type not "s
+                                        + compute_type);
+
+        // For C and D, only F32/BF16/F16 allowed when A or B is block scaling format
+        if(arg.c_type != HIP_R_32F && arg.c_type != HIP_R_16F && arg.c_type != HIP_R_16BF)
+            throw std::invalid_argument("Invalid c_type for block scaling format: "s
+                                        + hip_datatype_to_string(arg.c_type));
+        if(arg.d_type != HIP_R_32F && arg.d_type != HIP_R_16F && arg.d_type != HIP_R_16BF)
+            throw std::invalid_argument("Invalid d_type for block scaling format: "s
+                                        + hip_datatype_to_string(arg.d_type));
+    }
 
     if(arg.M[0] < 0)
         throw std::invalid_argument("Invalid value for -m " + std::to_string(arg.M[0]));
@@ -872,8 +1006,8 @@ try
     }
 
     arg.norm_check_assert = false;
-    int status            = run_bench_test(arg, filter, any_stride);
-    freeFrequencyMonitor();
+    int status            = run_bench_test(arg, filter, any_stride, props);
+    freeEfficiencyMonitor();
     return status;
 }
 catch(const std::invalid_argument& exp)
