@@ -37,6 +37,7 @@
 #include <limits>
 #include <string>
 #include <vector>
+#include "utility.hpp"
 
 #ifndef CHECK_HIP_ERROR
 #define CHECK_HIP_ERROR(error)                    \
@@ -72,9 +73,10 @@
 
 typedef enum _ActivationType
 {
-    NONE = 0,
-    RELU = 1,
-    GELU = 2,
+    NONE  = 0,
+    RELU  = 1,
+    GELU  = 2,
+    SWISH = 3,
 } ActivationType;
 
 inline const char* ToString(ActivationType act)
@@ -87,6 +89,8 @@ inline const char* ToString(ActivationType act)
         return "relu";
     case GELU:
         return "gelu";
+    case SWISH:
+        return "swish";
     default:
         return "[Unknown Activation Type]";
     }
@@ -107,6 +111,13 @@ auto _gelu = [](auto in) -> decltype(in) {
         0.5f * (in_Tc * (1.f + std::tanh(k0 * (in_Tc * (1.f + k1 * (in_Tc * in_Tc)))))));
 };
 
+// swish with beta=1
+auto _silu = [](auto in) -> decltype(in) {
+    using Tc = float;
+    Tc in_Tc = static_cast<Tc>(in);
+    return static_cast<decltype(in)>(in_Tc / (1.f + exp(-in_Tc)));
+};
+
 template <typename T>
 inline bool AlmostEqual(T a, T b)
 {
@@ -159,6 +170,8 @@ void mat_mul_bias_activation(Tc             alpha,
         actFunc = _relu;
     else if(actType == ActivationType::GELU)
         actFunc = _gelu;
+    else if(actType == ActivationType::SWISH)
+        actFunc = _silu;
 
     for(int batch = 0; batch < batch_count; batch++)
     {
@@ -182,31 +195,6 @@ void mat_mul_bias_activation(Tc             alpha,
     }
 }
 
-double get_time_us_no_sync()
-{
-    auto now = std::chrono::steady_clock::now();
-    // now.time_since_epoch() is the duration since epoch
-    // which is converted to microseconds
-    auto duration
-        = std::chrono::duration_cast<std::chrono::microseconds>(now.time_since_epoch()).count();
-    return (static_cast<double>(duration));
-};
-
-double get_time_us_sync()
-{
-    if(hipDeviceSynchronize() != hipSuccess)
-    {
-        std::cout << "Synchronizing device failed" << std::endl;
-    }
-
-    auto now = std::chrono::steady_clock::now();
-    // now.time_since_epoch() is the duration since epoch
-    // which is converted to microseconds
-    auto duration
-        = std::chrono::duration_cast<std::chrono::microseconds>(now.time_since_epoch()).count();
-    return (static_cast<double>(duration));
-};
-
 // cppcheck-suppress constParameter
 static void show_usage(char* argv[])
 {
@@ -242,7 +230,8 @@ static void show_usage(char* argv[])
         << "\t--stride_d \t\tstride_d \tGEMM_STRIDED argument stride_d\n"
         << "\t--alpha \t\talpha \t\tGEMM_STRIDED argument alpha (default is 1)\n"
         << "\t--beta \t\t\tbeta \t\tGEMM_STRIDED argument beta (default is 0)\n"
-        << "\t--act \t\t\tact \t\tGEMM_STRIDED set activation type: relu, gelu, none (default is "
+        << "\t--act \t\t\tact \t\tGEMM_STRIDED set activation type: relu, gelu, swish, none "
+           "(default is "
            "none)\n"
         << "\t--bias \t\t\tbias \t\tGEMM_STRIDED set bias: 0 or 1 (default is 0)\n"
         << std::endl;
@@ -382,6 +371,8 @@ static int parse_arguments(int                          argc,
                         actType.push_back(ActivationType::RELU);
                     else if(strncmp(argv[i], "gelu", 4) == 0)
                         actType.push_back(ActivationType::GELU);
+                    else if(strncmp(argv[i], "swish", 5) == 0)
+                        actType.push_back(ActivationType::SWISH);
                     else if(strncmp(argv[i], "none", 4) == 0)
                         actType.push_back(ActivationType::NONE);
                     else
@@ -827,8 +818,8 @@ int test_hipblaslt(hipDataType                 in_datatype,
                   << ", " << ToString(actType[i]) << std::endl;
     }
 
-    std::vector<hipblaslt_ext::GemmEpilogueV2> gemmEpilogue(gemm_count);
-    std::vector<hipblaslt_ext::GemmInputsV2>   gemmInputs(gemm_count);
+    std::vector<hipblaslt_ext::GemmEpilogue> gemmEpilogue(gemm_count);
+    std::vector<hipblaslt_ext::GemmInputs>   gemmInputs(gemm_count);
     for(size_t i = 0; i < gemm_count; i++)
     {
         if(enable_bias[i] && actType[i] == ActivationType::NONE)
@@ -837,12 +828,16 @@ int test_hipblaslt(hipDataType                 in_datatype,
             epilogue[i] = HIPBLASLT_EPILOGUE_RELU_BIAS;
         else if(enable_bias[i] && actType[i] == ActivationType::GELU)
             epilogue[i] = HIPBLASLT_EPILOGUE_GELU_BIAS;
+        else if(enable_bias[i] && actType[i] == ActivationType::SWISH)
+            epilogue[i] = HIPBLASLT_EPILOGUE_SWISH_BIAS_EXT;
         else if(!enable_bias[i] && actType[i] == ActivationType::NONE)
             epilogue[i] = HIPBLASLT_EPILOGUE_DEFAULT;
         else if(!enable_bias[i] && actType[i] == ActivationType::RELU)
             epilogue[i] = HIPBLASLT_EPILOGUE_RELU;
         else if(!enable_bias[i] && actType[i] == ActivationType::GELU)
             epilogue[i] = HIPBLASLT_EPILOGUE_GELU;
+        else if(!enable_bias[i] && actType[i] == ActivationType::SWISH)
+            epilogue[i] = HIPBLASLT_EPILOGUE_SWISH_EXT;
         gemmEpilogue[i].setMode(epilogue[i]);
         gemmEpilogue[i].setBiasDataType(static_cast<hipDataType>(HIP_R_32F));
         gemmInputs[i].setA(da[i]);
@@ -854,13 +849,13 @@ int test_hipblaslt(hipDataType                 in_datatype,
         gemmInputs[i].setBias(d_bias[i]);
     }
 
-    auto gemmProblemType = hipblaslt_ext::GemmProblemTypeV2(trans_a,
-                                                            trans_b,
-                                                            in_datatype,
-                                                            in_datatype,
-                                                            out_datatype,
-                                                            out_datatype,
-                                                            HIPBLAS_COMPUTE_32F);
+    auto gemmProblemType = hipblaslt_ext::GemmProblemType(trans_a,
+                                                          trans_b,
+                                                          in_datatype,
+                                                          in_datatype,
+                                                          out_datatype,
+                                                          out_datatype,
+                                                          HIPBLAS_COMPUTE_32F);
 
     // step 1: set problem to {Ms, {sum of N, 1, 1, 1, ...}, Ks}
     CHECK_HIPBLASLT_ERROR(groupedGemm.setProblem(m,
