@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -123,16 +123,12 @@ void sat_cast_mul(void* dst, hipDataType typeD, customVector<TcCast>& src, Tc sc
         sat_cast_mul<hipblaslt_bf8_fnuz, TcCast, Tc>(
             static_cast<hipblaslt_bf8_fnuz*>(dst), src, scale, size);
         break;
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
-        sat_cast_mul<hipblaslt_f8, TcCast, Tc>(
-            static_cast<hipblaslt_f8*>(dst), src, scale, size);
+        sat_cast_mul<hipblaslt_f8, TcCast, Tc>(static_cast<hipblaslt_f8*>(dst), src, scale, size);
         break;
     case HIP_R_8F_E5M2:
-        sat_cast_mul<hipblaslt_bf8, TcCast, Tc>(
-            static_cast<hipblaslt_bf8*>(dst), src, scale, size);
+        sat_cast_mul<hipblaslt_bf8, TcCast, Tc>(static_cast<hipblaslt_bf8*>(dst), src, scale, size);
         break;
-#endif
     case HIP_R_32I:
         sat_cast_mul<int32_t, TcCast, Tc>(static_cast<int32_t*>(dst), src, scale, size);
         break;
@@ -152,11 +148,9 @@ void cast_mul(customVector<TcCast>& dst, const TiA* src, size_t size)
                  || (!std::is_same<TiA, hipblaslt_bf8_fnuz>::value
                      && !std::is_same<TiA, hipblaslt_f8_fnuz>::value))
     {
-#ifdef ROCM_USE_FLOAT8
         if constexpr(std::is_same<TcCast, float>::value
                      || !(std::is_same<TiA, hipblaslt_bf8>::value
                           || std::is_same<TiA, hipblaslt_f8>::value))
-#endif
             for(size_t i = 0; i < size; i++)
             {
                 dst[i] = static_cast<TcCast>(src[i]);
@@ -188,14 +182,12 @@ void cast_mul(customVector<TcCast>& dst, const void* src, hipDataType TiA, size_
         cast_mul<TcCast, hipblaslt_bf8_fnuz>(
             dst, static_cast<const hipblaslt_bf8_fnuz*>(src), size);
         break;
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
         cast_mul<TcCast, hipblaslt_f8>(dst, static_cast<const hipblaslt_f8*>(src), size);
         break;
     case HIP_R_8F_E5M2:
         cast_mul<TcCast, hipblaslt_bf8>(dst, static_cast<const hipblaslt_bf8*>(src), size);
         break;
-#endif
     case HIP_R_32I:
         cast_mul<TcCast, int32_t>(dst, static_cast<const int32_t*>(src), size);
         break;
@@ -217,18 +209,17 @@ void cast_mul(customVector<TcCast>& dst,
               bool                  transA,
               int64_t               m,
               int64_t               k,
-              size_t                size)
+              size_t                size,
+              bool                  isMXFormat = false)
 {
     if constexpr((std::is_same<TcCast, float>::value)
                  || (!std::is_same<TiA, hipblaslt_bf8_fnuz>::value
                      && !std::is_same<TiA, hipblaslt_f8_fnuz>::value))
     {
-#ifdef ROCM_USE_FLOAT8
         if constexpr(std::is_same<TcCast, float>::value
                      || !(std::is_same<TiA, hipblaslt_bf8>::value
                           || std::is_same<TiA, hipblaslt_f8>::value))
         {
-#endif
             if(AlphaVec != nullptr)
             {
                 if(transA)
@@ -236,20 +227,34 @@ void cast_mul(customVector<TcCast>& dst,
 #pragma omp for
                     for(size_t i = 0; i < size; i++)
                     {
-                        auto scaleA = isScaleAVec ? scaleAVec[i % m] : scaleAVec[0];
-                        dst[i]      = static_cast<TcCast>(A[i]) * scaleA * AlphaVec[i % m];
+                        if(isMXFormat)
+                        {
+                            dst[i] = static_cast<TcCast>(A[i]) * AlphaVec[i % m];
+                        }
+                        else
+                        {
+                            auto scaleA = isScaleAVec ? scaleAVec[i % m] : scaleAVec[0];
+                            dst[i]      = static_cast<TcCast>(A[i]) * scaleA * AlphaVec[i % m];
+                        }
                     }
-                }
+                } // transA
                 else
                 {
 #pragma omp for
                     for(size_t i = 0; i < size; i++)
                     {
-                        auto scaleA = isScaleAVec ? scaleAVec[i / k] : scaleAVec[0];
-                        dst[i]      = static_cast<TcCast>(A[i]) * scaleA * AlphaVec[i / k];
+                        if(isMXFormat)
+                        {
+                            dst[i] = static_cast<TcCast>(A[i]) * AlphaVec[i / k];
+                        }
+                        else
+                        {
+                            auto scaleA = isScaleAVec ? scaleAVec[i / k] : scaleAVec[0];
+                            dst[i]      = static_cast<TcCast>(A[i]) * scaleA * AlphaVec[i / k];
+                        }
                     }
                 }
-            }
+            } // AlphaVec != nullptr
             else
             {
                 if(transA)
@@ -257,23 +262,35 @@ void cast_mul(customVector<TcCast>& dst,
 #pragma omp for
                     for(size_t i = 0; i < size; i++)
                     {
-                        auto scaleA = isScaleAVec ? scaleAVec[i % m] : scaleAVec[0];
-                        dst[i]      = static_cast<TcCast>(A[i] * scaleA);
+                        if(isMXFormat)
+                        {
+                            dst[i] = static_cast<TcCast>(A[i]);
+                        }
+                        else
+                        {
+                            auto scaleA = isScaleAVec ? scaleAVec[i % m] : scaleAVec[0];
+                            dst[i]      = static_cast<TcCast>(A[i] * scaleA);
+                        }
                     }
                 }
-                else
+                else // not transA
                 {
 #pragma omp for
                     for(size_t i = 0; i < size; i++)
                     {
-                        auto scaleA = isScaleAVec ? scaleAVec[i / k] : scaleAVec[0];
-                        dst[i]      = static_cast<TcCast>(A[i] * scaleA);
+                        if(isMXFormat)
+                        {
+                            dst[i] = static_cast<TcCast>(A[i]);
+                        }
+                        else
+                        {
+                            auto scaleA = isScaleAVec ? scaleAVec[i / k] : scaleAVec[0];
+                            dst[i]      = static_cast<TcCast>(A[i] * scaleA);
+                        }
                     }
                 }
             }
-#ifdef ROCM_USE_FLOAT8
         }
-#endif
     }
 }
 
@@ -287,7 +304,8 @@ void cast_mul(customVector<TcCast>& dst,
               bool                  transA,
               int64_t               m,
               int64_t               k,
-              size_t                size)
+              size_t                size,
+              bool                  isMXFormat = false)
 {
     switch(TiA)
     {
@@ -300,7 +318,8 @@ void cast_mul(customVector<TcCast>& dst,
                                     transA,
                                     m,
                                     k,
-                                    size);
+                                    size,
+                                    isMXFormat);
         break;
     case HIP_R_64F:
         cast_mul<TcCast, Tc, double>(dst,
@@ -311,7 +330,8 @@ void cast_mul(customVector<TcCast>& dst,
                                      transA,
                                      m,
                                      k,
-                                     size);
+                                     size,
+                                     isMXFormat);
         break;
     case HIP_R_16F:
         cast_mul<TcCast, Tc, hipblasLtHalf>(dst,
@@ -322,7 +342,8 @@ void cast_mul(customVector<TcCast>& dst,
                                             transA,
                                             m,
                                             k,
-                                            size);
+                                            size,
+                                            isMXFormat);
         break;
     case HIP_R_16BF:
         cast_mul<TcCast, Tc, hip_bfloat16>(dst,
@@ -333,7 +354,8 @@ void cast_mul(customVector<TcCast>& dst,
                                            transA,
                                            m,
                                            k,
-                                           size);
+                                           size,
+                                           isMXFormat);
         break;
     case HIP_R_8F_E4M3_FNUZ:
         cast_mul<TcCast, Tc, hipblaslt_f8_fnuz>(dst,
@@ -344,7 +366,8 @@ void cast_mul(customVector<TcCast>& dst,
                                                 transA,
                                                 m,
                                                 k,
-                                                size);
+                                                size,
+                                                isMXFormat);
         break;
     case HIP_R_8F_E5M2_FNUZ:
         cast_mul<TcCast, Tc, hipblaslt_bf8_fnuz>(dst,
@@ -355,32 +378,33 @@ void cast_mul(customVector<TcCast>& dst,
                                                  transA,
                                                  m,
                                                  k,
-                                                 size);
+                                                 size,
+                                                 isMXFormat);
         break;
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
         cast_mul<TcCast, Tc, hipblaslt_f8>(dst,
-                                               static_cast<const hipblaslt_f8*>(src),
-                                               isScaleAVec,
-                                               scaleAVec,
-                                               AlphaVec,
-                                               transA,
-                                               m,
-                                               k,
-                                               size);
+                                           static_cast<const hipblaslt_f8*>(src),
+                                           isScaleAVec,
+                                           scaleAVec,
+                                           AlphaVec,
+                                           transA,
+                                           m,
+                                           k,
+                                           size,
+                                           isMXFormat);
         break;
     case HIP_R_8F_E5M2:
         cast_mul<TcCast, Tc, hipblaslt_bf8>(dst,
-                                                static_cast<const hipblaslt_bf8*>(src),
-                                                isScaleAVec,
-                                                scaleAVec,
-                                                AlphaVec,
-                                                transA,
-                                                m,
-                                                k,
-                                                size);
+                                            static_cast<const hipblaslt_bf8*>(src),
+                                            isScaleAVec,
+                                            scaleAVec,
+                                            AlphaVec,
+                                            transA,
+                                            m,
+                                            k,
+                                            size,
+                                            isMXFormat);
         break;
-#endif
     case HIP_R_32I:
         cast_mul<TcCast, Tc, int32_t>(dst,
                                       static_cast<const int32_t*>(src),
@@ -426,14 +450,12 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
                         && (!std::is_same<TiA, hipblaslt_bf8_fnuz>::value
                             && !std::is_same<TiA, hipblaslt_f8_fnuz>::value))
     {
-#ifdef ROCM_USE_FLOAT8
         if constexpr(std::is_same<TcCast, float>::value
                      || (!std::is_same<TciACast, hipblaslt_bf8>::value
                          && !std::is_same<TciACast, hipblaslt_f8>::value)
                             && (!std::is_same<TiA, hipblaslt_bf8>::value
                                 && !std::is_same<TiA, hipblaslt_f8>::value))
         {
-#endif
             if(AlphaVec != nullptr)
             {
                 if(transA)
@@ -478,9 +500,7 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
                     }
                 }
             }
-#ifdef ROCM_USE_FLOAT8
         }
-#endif
     }
 }
 
@@ -567,18 +587,16 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
             k,
             size);
         break;
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
-        cast_mul_with_Tci<TcCast, Tc, TciACast, hipblaslt_f8>(
-            dst,
-            static_cast<const hipblaslt_f8*>(src),
-            isScaleAVec,
-            scaleAVec,
-            AlphaVec,
-            transA,
-            m,
-            k,
-            size);
+        cast_mul_with_Tci<TcCast, Tc, TciACast, hipblaslt_f8>(dst,
+                                                              static_cast<const hipblaslt_f8*>(src),
+                                                              isScaleAVec,
+                                                              scaleAVec,
+                                                              AlphaVec,
+                                                              transA,
+                                                              m,
+                                                              k,
+                                                              size);
         break;
     case HIP_R_8F_E5M2:
         cast_mul_with_Tci<TcCast, Tc, TciACast, hipblaslt_bf8>(
@@ -592,7 +610,6 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
             k,
             size);
         break;
-#endif
     case HIP_R_32I:
         cast_mul_with_Tci<TcCast, Tc, TciACast, int32_t>(dst,
                                                          static_cast<const int32_t*>(src),
@@ -661,7 +678,6 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
         cast_mul_with_Tci<TcCast, Tc, hipblaslt_bf8_fnuz>(
             dst, src, TiA, isScaleAVec, scaleAVec, AlphaVec, transA, m, k, size);
         break;
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
         cast_mul_with_Tci<TcCast, Tc, hipblaslt_f8>(
             dst, src, TiA, isScaleAVec, scaleAVec, AlphaVec, transA, m, k, size);
@@ -670,7 +686,6 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
         cast_mul_with_Tci<TcCast, Tc, hipblaslt_bf8>(
             dst, src, TiA, isScaleAVec, scaleAVec, AlphaVec, transA, m, k, size);
         break;
-#endif
     case HIP_R_32I:
         cast_mul_with_Tci<TcCast, Tc, int32_t>(
             dst, src, TiA, isScaleAVec, scaleAVec, AlphaVec, transA, m, k, size);
@@ -690,17 +705,17 @@ void cast_mul_with_Tci(customVector<TcCast>& dst,
 template <typename T>
 void small_gemm(hipblasOperation_t transA,
                 hipblasOperation_t transB,
-                int               m,
-                int               n,
-                int               k,
-                T                 alpha,
-                const T*          A,
-                int               lda,
-                const T*          B,
-                int               ldb,
-                T                 beta,
-                T*                C,
-                int               ldc)
+                int                m,
+                int                n,
+                int                k,
+                T                  alpha,
+                const T*           A,
+                int                lda,
+                const T*           B,
+                int                ldb,
+                T                  beta,
+                T*                 C,
+                int                ldc)
 {
     bool notTA = (transA == HIPBLAS_OP_N);
     bool notTB = (transB == HIPBLAS_OP_N);
@@ -892,26 +907,31 @@ void cblas_gemm(hipblasOperation_t       transA,
                 hipDataType              Tc_enum,
                 hipDataType              TciA,
                 hipDataType              TciB,
-                bool                     alt)
+                bool                     alt,
+                bool                     isScaleAMXFormat,
+                bool                     isScaleBMXFormat)
 {
     using IntTcCast = std::conditional_t<std::is_same<Tc, int32_t>::value, double, Tc>;
     // cblas does not support hipblasLtHalf, so convert to higher precision float
     // This will give more precise result which is acceptable for testing
     using HalfTcCast = std::conditional_t<std::is_same<Tc, hipblasLtHalf>::value, float, Tc>;
-    using TcCast = std::conditional_t<std::is_same<Tc, int32_t>::value, IntTcCast, HalfTcCast>;
+    using TcCast     = std::conditional_t<std::is_same<Tc, int32_t>::value, IntTcCast, HalfTcCast>;
 
-    if (Tc_enum == HIP_R_32I) {
+    if(Tc_enum == HIP_R_32I)
+    {
         Tc_enum = HIP_R_64F;
-    } else if (Tc_enum == HIP_R_16F) {
+    }
+    else if(Tc_enum == HIP_R_16F)
+    {
         Tc_enum = HIP_R_32F;
     }
 
     hipDataType TciACast = (TciA == HIP_R_32I) ? HIP_R_64F : TciA;
     hipDataType TciBCast = (TciB == HIP_R_32I) ? HIP_R_64F : TciB;
 
-    size_t sizeA = (transA == HIPBLAS_OP_N ? k : m) * size_t(lda);
-    size_t sizeB = (transB == HIPBLAS_OP_N ? n : k) * size_t(ldb);
-    size_t sizeC = n * size_t(ldc);
+    size_t sizeA          = (transA == HIPBLAS_OP_N ? k : m) * size_t(lda);
+    size_t sizeB          = (transB == HIPBLAS_OP_N ? n : k) * size_t(ldb);
+    size_t sizeC          = n * size_t(ldc);
     size_t scaleAVec_size = isScaleAVec ? m : 1;
     size_t scaleBVec_size = isScaleBVec ? n : 1;
 
@@ -952,8 +972,17 @@ void cblas_gemm(hipblasOperation_t       transA,
     }
     else
     {
-        cast_mul<TcCast, Tc>(
-            A_Tc, A, TiA, isScaleAVec, scaleA_Tc, AlphaVec_Tc, transA == HIPBLAS_OP_N, m, k, sizeA);
+        cast_mul<TcCast, Tc>(A_Tc,
+                             A,
+                             TiA,
+                             isScaleAVec,
+                             scaleA_Tc,
+                             AlphaVec_Tc,
+                             transA == HIPBLAS_OP_N,
+                             m,
+                             k,
+                             sizeA,
+                             isScaleAMXFormat);
     }
 
     B_Tc.initialize(sizeB);
@@ -973,8 +1002,17 @@ void cblas_gemm(hipblasOperation_t       transA,
     }
     else
     {
-        cast_mul<TcCast, Tc>(
-            B_Tc, B, TiB, isScaleBVec, scaleB_Tc, nullptr, transB != HIPBLAS_OP_N, n, k, sizeB);
+        cast_mul<TcCast, Tc>(B_Tc,
+                             B,
+                             TiB,
+                             isScaleBVec,
+                             scaleB_Tc,
+                             nullptr,
+                             transB != HIPBLAS_OP_N,
+                             n,
+                             k,
+                             sizeB,
+                             isScaleBMXFormat);
     }
 
     if(To == Tc_enum)
@@ -998,23 +1036,24 @@ void cblas_gemm(hipblasOperation_t       transA,
         if(m > small || n > small || k > small || lda > small || ldb > small || ldc > small)
         {
             cblas_sgemm(CblasColMajor,
-                    HIPOperationToCBLASTanspose(transA),
-                    HIPOperationToCBLASTanspose(transB),
-                    m,
-                    n,
-                    k,
-                    alphaCast,
-                    A_Tc,
-                    lda,
-                    B_Tc,
-                    ldb,
-                    betaCast,
-                    C_Tc,
-                    ldc);
+                        HIPOperationToCBLASTanspose(transA),
+                        HIPOperationToCBLASTanspose(transB),
+                        m,
+                        n,
+                        k,
+                        alphaCast,
+                        A_Tc,
+                        lda,
+                        B_Tc,
+                        ldb,
+                        betaCast,
+                        C_Tc,
+                        ldc);
         }
         else
         {
-            small_gemm<float>(transA, transB, m, n, k, alphaCast, A_Tc, lda, B_Tc, ldb, betaCast, C_Tc, ldc);
+            small_gemm<float>(
+                transA, transB, m, n, k, alphaCast, A_Tc, lda, B_Tc, ldb, betaCast, C_Tc, ldc);
         }
     }
     else if constexpr(std::is_same<TcCast, double>::value)
@@ -1023,23 +1062,24 @@ void cblas_gemm(hipblasOperation_t       transA,
         if(m > small || n > small || k > small || lda > small || ldb > small || ldc > small)
         {
             cblas_dgemm(CblasColMajor,
-                    HIPOperationToCBLASTanspose(transA),
-                    HIPOperationToCBLASTanspose(transB),
-                    m,
-                    n,
-                    k,
-                    alphaCast,
-                    A_Tc,
-                    lda,
-                    B_Tc,
-                    ldb,
-                    betaCast,
-                    C_Tc,
-                    ldc);
+                        HIPOperationToCBLASTanspose(transA),
+                        HIPOperationToCBLASTanspose(transB),
+                        m,
+                        n,
+                        k,
+                        alphaCast,
+                        A_Tc,
+                        lda,
+                        B_Tc,
+                        ldb,
+                        betaCast,
+                        C_Tc,
+                        ldc);
         }
         else
         {
-            small_gemm<double>(transA, transB, m, n, k, alphaCast, A_Tc, lda, B_Tc, ldb, betaCast, C_Tc, ldc);
+            small_gemm<double>(
+                transA, transB, m, n, k, alphaCast, A_Tc, lda, B_Tc, ldb, betaCast, C_Tc, ldc);
         }
     }
 
@@ -1056,33 +1096,35 @@ void cblas_gemm(hipblasOperation_t       transA,
     }
 }
 
-#define CREATEFUNCTION(Tc)                                             \
-    template void cblas_gemm<Tc>(hipblasOperation_t       transA,      \
-                                 hipblasOperation_t       transB,      \
-                                 int64_t                  m,           \
-                                 int64_t                  n,           \
-                                 int64_t                  k,           \
-                                 Tc                       alpha,       \
-                                 const void*              A,           \
-                                 int64_t                  lda,         \
-                                 const void*              B,           \
-                                 int64_t                  ldb,         \
-                                 Tc                       beta,        \
-                                 std::add_pointer_t<void> C,           \
-                                 int64_t                  ldc,         \
-                                 const void*              AlphaVec,    \
-                                 const void*              scaleAVec,   \
-                                 const void*              scaleBVec,   \
-                                 Tc                       scaleD,      \
-                                 bool                     isScaleAVec, \
-                                 bool                     isScaleBVec, \
-                                 hipDataType              TiA,         \
-                                 hipDataType              TiB,         \
-                                 hipDataType              To,          \
-                                 hipDataType              Tc_enum,     \
-                                 hipDataType              TciA,        \
-                                 hipDataType              TciB,        \
-                                 bool                     alt);
+#define CREATEFUNCTION(Tc)                                                  \
+    template void cblas_gemm<Tc>(hipblasOperation_t       transA,           \
+                                 hipblasOperation_t       transB,           \
+                                 int64_t                  m,                \
+                                 int64_t                  n,                \
+                                 int64_t                  k,                \
+                                 Tc                       alpha,            \
+                                 const void*              A,                \
+                                 int64_t                  lda,              \
+                                 const void*              B,                \
+                                 int64_t                  ldb,              \
+                                 Tc                       beta,             \
+                                 std::add_pointer_t<void> C,                \
+                                 int64_t                  ldc,              \
+                                 const void*              AlphaVec,         \
+                                 const void*              scaleAVec,        \
+                                 const void*              scaleBVec,        \
+                                 Tc                       scaleD,           \
+                                 bool                     isScaleAVec,      \
+                                 bool                     isScaleBVec,      \
+                                 hipDataType              TiA,              \
+                                 hipDataType              TiB,              \
+                                 hipDataType              To,               \
+                                 hipDataType              Tc_enum,          \
+                                 hipDataType              TciA,             \
+                                 hipDataType              TciB,             \
+                                 bool                     alt,              \
+                                 bool                     isScaleAMXFormat, \
+                                 bool                     isScaleBMXFormat);
 
 CREATEFUNCTION(hipblasLtHalf)
 CREATEFUNCTION(float)
