@@ -104,3 +104,27 @@ constexpr double gelu_gflop_count(int64_t m, int64_t n, hipDataType type)
         return gelu_gflop_count<float>(m, n);
     }
 }
+
+template <typename T>
+constexpr double silu_gflop_count(int64_t m, int64_t n)
+{
+    return (5.0 * m * n) / 1e9;
+}
+
+constexpr double silu_gflop_count(int64_t m, int64_t n, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        return silu_gflop_count<float>(m, n);
+    case HIP_R_64F:
+        return silu_gflop_count<double>(m, n);
+    case HIP_R_16F:
+        return silu_gflop_count<hipblasLtHalf>(m, n);
+    case HIP_R_32I:
+        return silu_gflop_count<int32_t>(m, n);
+    default:
+        hipblaslt_cerr << "Error type in silu_gflop_count()" << std::endl;
+        return silu_gflop_count<float>(m, n);
+    }
+}
\ No newline at end of file
