@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -50,6 +50,14 @@ enum hipblaslt_argument : int;
 constexpr std::size_t MAX_SUPPORTED_NUM_PROBLEMS{32};
 struct Arguments
 {
+    enum ScalingFormat
+    {
+        None   = 0,
+        Scalar = 1,
+        Vector = 2,
+        Block  = 3
+    };
+
     /*************************************************************************
      *                    Beginning Of Arguments                             *
      *************************************************************************/
@@ -129,6 +137,7 @@ struct Arguments
     float                     activation_arg2; // upperbound when activation type is relu
 
     hipDataType              bias_type;
+    hipDataType              aux_type;
     hipblaslt_bias_source    bias_source;
     bool                     bias_vector;
     hipblaslt_scaling_format scaleA;
@@ -145,6 +154,12 @@ struct Arguments
     bool                     use_e;
     bool                     gradient;
     bool                     norm_check_assert;
+    bool                     swizzle_a;
+
+    uint32_t scaleABlockRowSize;
+    uint32_t scaleABlockColSize;
+    uint32_t scaleBBlockRowSize;
+    uint32_t scaleBBlockColSize;
 
     // API related
     bool    use_ext;
@@ -164,6 +179,7 @@ struct Arguments
     bool print_kernel_info;
 
     bool flush;
+    int tensile_solution_selection_method;
 
     /*************************************************************************
      *                     End Of Arguments                                  *
@@ -228,6 +244,7 @@ struct Arguments
     OPER(activation_arg1) SEP        \
     OPER(activation_arg2) SEP        \
     OPER(bias_type) SEP              \
+    OPER(aux_type) SEP               \
     OPER(bias_source) SEP            \
     OPER(bias_vector) SEP            \
     OPER(scaleA) SEP                 \
@@ -244,6 +261,11 @@ struct Arguments
     OPER(use_e) SEP                  \
     OPER(gradient) SEP               \
     OPER(norm_check_assert) SEP      \
+    OPER(swizzle_a) SEP              \
+    OPER(scaleABlockRowSize) SEP     \
+    OPER(scaleABlockColSize) SEP     \
+    OPER(scaleBBlockRowSize) SEP     \
+    OPER(scaleBBlockColSize) SEP     \
     OPER(use_ext) SEP                \
     OPER(use_ext_setproblem) SEP     \
     OPER(algo_method) SEP            \
@@ -256,7 +278,8 @@ struct Arguments
     OPER(wgm_vector) SEP             \
     OPER(print_solution_found) SEP   \
     OPER(print_kernel_info) SEP      \
-    OPER(flush) SEP
+    OPER(flush) SEP                  \
+    OPER(tensile_solution_selection_method) SEP
 
     // clang-format on
 
@@ -273,7 +296,7 @@ struct Arguments
     // Function to read Arguments data from stream
     friend std::istream& operator>>(std::istream& str, Arguments& arg);
 
-#ifdef WIN32
+#ifdef _WIN32
     // Clang specific code
     template <typename T>
     friend hipblaslt_internal_ostream& operator<<(hipblaslt_internal_ostream& os,
@@ -848,7 +871,7 @@ namespace ArgumentsHelper
                 func("rotating_buffer", arg.rotating);
         };
 };
-// clang-format on
+    // clang-format on
 
 #else
 #error "Unsupported C++ version"
