@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -106,9 +106,7 @@ void m_axpy(size_t* N, T* alpha, T* x, int* incx, T* y, int* incy)
 template <
     typename T,
     std::enable_if_t<!(std::is_same<T, hipblaslt_f8_fnuz>{} || std::is_same<T, hipblaslt_bf8_fnuz>{}
-#ifdef ROCM_USE_FLOAT8
                        || std::is_same<T, hipblaslt_f8>{} || std::is_same<T, hipblaslt_bf8>{}
-#endif
                        ),
                      int>
     = 0>
@@ -145,7 +143,6 @@ double norm_check_general(char norm_type, int64_t M, int64_t N, int64_t lda, T*
     double cpu_norm = xlange(&norm_type, &m, &n, hCPU_double.data(), &l, work);
     m_axpy(&size, &alpha, hCPU_double.data(), &incx, hGPU_double.data(), &incx);
     double error = xlange(&norm_type, &m, &n, hGPU_double.data(), &l, work) / cpu_norm;
-
     return error;
 }
 
@@ -187,7 +184,6 @@ double norm_check_general(char norm_type, int64_t M, int64_t N, int64_t lda, T*
     return error;
 }
 
-#ifdef ROCM_USE_FLOAT8
 template <
     typename T,
     std::enable_if_t<(std::is_same<T, hipblaslt_f8>{} || std::is_same<T, hipblaslt_bf8>{}), int>
@@ -224,7 +220,6 @@ double norm_check_general(char norm_type, int64_t M, int64_t N, int64_t lda, T*
 
     return error;
 }
-#endif
 
 /* ============== Norm Check for strided_batched case ============= */
 template <typename T, template <typename> class VEC, typename T_hpa>
@@ -385,22 +380,20 @@ double norm_check_general(
                                                       lda,
                                                       static_cast<hipblaslt_bf8_fnuz*>(hCPU),
                                                       static_cast<hipblaslt_bf8_fnuz*>(hGPU));
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
         return norm_check_general<hipblaslt_f8>(norm_type,
-                                                    M,
-                                                    N,
-                                                    lda,
-                                                    static_cast<hipblaslt_f8*>(hCPU),
-                                                    static_cast<hipblaslt_f8*>(hGPU));
+                                                M,
+                                                N,
+                                                lda,
+                                                static_cast<hipblaslt_f8*>(hCPU),
+                                                static_cast<hipblaslt_f8*>(hGPU));
     case HIP_R_8F_E5M2:
         return norm_check_general<hipblaslt_bf8>(norm_type,
-                                                     M,
-                                                     N,
-                                                     lda,
-                                                     static_cast<hipblaslt_bf8*>(hCPU),
-                                                     static_cast<hipblaslt_bf8*>(hGPU));
-#endif
+                                                 M,
+                                                 N,
+                                                 lda,
+                                                 static_cast<hipblaslt_bf8*>(hCPU),
+                                                 static_cast<hipblaslt_bf8*>(hGPU));
     case HIP_R_32I:
         return norm_check_general<int32_t>(
             norm_type, M, N, lda, static_cast<int32_t*>(hCPU), static_cast<int32_t*>(hGPU));
@@ -483,26 +476,24 @@ double norm_check_general(char        norm_type,
                                                       static_cast<hipblaslt_bf8_fnuz*>(hCPU),
                                                       static_cast<hipblaslt_bf8_fnuz*>(hGPU),
                                                       batch_count);
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
         return norm_check_general<hipblaslt_f8>(norm_type,
-                                                    M,
-                                                    N,
-                                                    lda,
-                                                    stride_a,
-                                                    static_cast<hipblaslt_f8*>(hCPU),
-                                                    static_cast<hipblaslt_f8*>(hGPU),
-                                                    batch_count);
+                                                M,
+                                                N,
+                                                lda,
+                                                stride_a,
+                                                static_cast<hipblaslt_f8*>(hCPU),
+                                                static_cast<hipblaslt_f8*>(hGPU),
+                                                batch_count);
     case HIP_R_8F_E5M2:
         return norm_check_general<hipblaslt_bf8>(norm_type,
-                                                     M,
-                                                     N,
-                                                     lda,
-                                                     stride_a,
-                                                     static_cast<hipblaslt_bf8*>(hCPU),
-                                                     static_cast<hipblaslt_bf8*>(hGPU),
-                                                     batch_count);
-#endif
+                                                 M,
+                                                 N,
+                                                 lda,
+                                                 stride_a,
+                                                 static_cast<hipblaslt_bf8*>(hCPU),
+                                                 static_cast<hipblaslt_bf8*>(hGPU),
+                                                 batch_count);
     case HIP_R_32I:
         return norm_check_general<int32_t>(norm_type,
                                            M,
@@ -546,15 +537,17 @@ bool norm_check(double norm_error)
         return norm_error < 0.125;
     if(std::is_same<T, hipblaslt_bf8_fnuz>{})
         return norm_error < 0.25;
-#ifdef ROCM_USE_FLOAT8
     if(std::is_same<T, hipblaslt_f8>{})
         return norm_error < 0.125;
     if(std::is_same<T, hipblaslt_bf8>{})
         return norm_error < 0.25;
-#endif
     return false;
 }
 
+// TODO: norm_check determines the required norm solely based on
+//       To (type). This might cause tests to fail when the input
+//       matrices are MX types (F4/F8/F6). A better way is
+//       needed to determine the required norm for MX types.
 bool norm_check(double norm_error, hipDataType type)
 {
     switch(type)
@@ -568,19 +561,20 @@ bool norm_check(double norm_error, hipDataType type)
     case HIP_R_16BF:
         return norm_error < 0.1;
     case HIP_R_8F_E4M3_FNUZ:
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E4M3:
-#endif
         return norm_error < 0.125;
     case HIP_R_8F_E5M2_FNUZ:
-#ifdef ROCM_USE_FLOAT8
     case HIP_R_8F_E5M2:
-#endif
         return norm_error < 0.25;
     case HIP_R_32I:
         return norm_error < 0.0001;
     case HIP_R_8I:
         return norm_error < 0.01;
+    // TODO: find a suitable rnom value for f6 and f4
+    case HIP_R_6F_E2M3_EXT:
+    case HIP_R_6F_E3M2_EXT:
+    case HIP_R_4F_E2M1_EXT:
+        return norm_error < 0.5;
     default:
         return false;
     }
