@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -102,6 +102,7 @@ inline bool is_bias_enabled(hipblasLtEpilogue_t value_)
     case HIPBLASLT_EPILOGUE_DGELU_BGRAD:
     case HIPBLASLT_EPILOGUE_BGRADA:
     case HIPBLASLT_EPILOGUE_BGRADB:
+    case HIPBLASLT_EPILOGUE_SWISH_BIAS_EXT:
         return true;
     default:
         return false;
@@ -119,6 +120,8 @@ enum class hipblaslt_batch_type
 class hipblaslt_local_handle
 {
     hipblasLtHandle_t m_handle;
+    std::string    m_sol_selec_saved_status = "";
+    bool           m_sol_selec_env_set{false};
 
 public:
     hipblaslt_local_handle();
@@ -317,7 +320,7 @@ public:
 
 /* ============================================================================================ */
 /*  device query and print out their ID and name */
-int64_t query_device_property();
+int64_t query_device_property(int device_id, hipDeviceProp_t &props);
 
 /*  set current device to device_id */
 void set_device(int64_t device_id);
@@ -524,3 +527,4 @@ typename std::enable_if<!std::is_same<int8_t, T>::value, T>::type saturate_cast(
 
 std::vector<void*> benchmark_allocation();
 int32_t            hipblaslt_get_arch_major();
+void hipblaslt_print_version();
