@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
  *
  *******************************************************************************/
 
-#include "hipblaslt-ext-op.h"
+#include "hipblaslt/hipblaslt-ext-op.h"
 #include "hipblaslt-ext-op-internal.hpp"
 #include <Tensile/hip/HipHardware.hpp>
 #include <Tensile/hip/HipSolutionAdapter.hpp>
@@ -32,11 +32,12 @@
 #include <cmath>
 #include <cstdlib>
 #include <cstring>
+#include <filesystem>
 #include <fstream>
 #include <hip/hip_ext.h>
 #include <hip/hip_runtime_api.h>
-#include <libgen.h>
 #include <memory>
+#include <optional>
 #include <rocblaslt-auxiliary.h>
 #include <sstream>
 #include <string>
@@ -99,17 +100,6 @@ hipblasStatus_t hipblasltAMaxRun(const hipDataType datatype,
                                  uint32_t          n,
                                  hipStream_t       stream);
 
-hipblasStatus_t hipblasltAMaxWithScaleRun(const hipDataType datatype,
-                                          const hipDataType outDatatype,
-                                          const hipDataType scaleDatatype,
-                                          void*             output,
-                                          void*             outputD,
-                                          void*             input,
-                                          void*             inputScale,
-                                          uint32_t          m,
-                                          uint32_t          n,
-                                          hipStream_t       stream);
-
 hipblasStatus_t hipblasltExtAMax(const hipDataType datatype,
                                  const hipDataType outDatatype,
                                  void*             output,
@@ -121,21 +111,6 @@ hipblasStatus_t hipblasltExtAMax(const hipDataType datatype,
     return hipblasltAMaxRun(datatype, outDatatype, output, input, m, n, stream);
 }
 
-hipblasStatus_t hipblasltExtAMaxWithScale(const hipDataType datatype,
-                                          const hipDataType outDatatype,
-                                          const hipDataType scaleDatatype,
-                                          void*             output,
-                                          void*             outputD,
-                                          void*             input,
-                                          void*             inputScale,
-                                          uint32_t          m,
-                                          uint32_t          n,
-                                          hipStream_t       stream)
-{
-    return hipblasltAMaxWithScaleRun(
-        datatype, outDatatype, scaleDatatype, output, outputD, input, inputScale, m, n, stream);
-}
-
 namespace
 {
     constexpr char DEFAULT_EXT_OP_LIBRARY_PATH[]
@@ -162,22 +137,10 @@ namespace
             return libPath;
         }
 
-        auto        soPath = rocblaslt_internal_get_so_path("hipblaslt");
-        std::string libPath(dirname(&soPath[0]));
-
-        if(rocblaslt_internal_test_path(libPath + "/../Tensile/library"))
-            libPath += "/../Tensile/library";
-        else if(rocblaslt_internal_test_path(libPath + "library"))
-            libPath += "/library";
-        else
-            libPath += "/hipblaslt/library";
-
-        libPath += "/hipblasltExtOpLibrary.dat";
-
-        if(rocblaslt_internal_test_path(libPath))
-        {
-            return libPath;
-        }
+        auto path = rocblaslt_find_library_relative_path(
+            std::filesystem::path("hipblasltExtOpLibrary.dat"));
+        if(path)
+            return path->string();
 
         return DEFAULT_EXT_OP_LIBRARY_PATH;
     }
@@ -221,15 +184,15 @@ namespace
         return elementNumBytes(datatype) * tileM * tileN;
     }
 
-    static const ExtOpMasterLibrary& getExtOpMasterLibrary()
+    static const hipblaslt_ext::ExtOpMasterLibrary& getExtOpMasterLibrary()
     {
-        static ExtOpMasterLibrary lib(getExtOpLibraryPath());
+        static hipblaslt_ext::ExtOpMasterLibrary lib(getExtOpLibraryPath());
         return lib;
     }
 
-    std::vector<std::unique_ptr<Tensile::hip::SolutionAdapter>>& extOpLibraries()
+    std::vector<std::unique_ptr<TensileLite::hip::SolutionAdapter>>& extOpLibraries()
     {
-        static std::vector<std::unique_ptr<Tensile::hip::SolutionAdapter>> adapters;
+        static std::vector<std::unique_ptr<TensileLite::hip::SolutionAdapter>> adapters;
 
         if(adapters.size())
         {
@@ -241,7 +204,7 @@ namespace
 
         for(std::size_t i = 0; i < numDevices; ++i)
         {
-            adapters.emplace_back(std::make_unique<Tensile::hip::SolutionAdapter>());
+            adapters.emplace_back(std::make_unique<TensileLite::hip::SolutionAdapter>());
         }
 
         int currentDevice{};
@@ -295,30 +258,32 @@ hipblasStatus_t hipblasltSoftmaxRun(hipDataType datatype,
     int         currentDeviceId{};
     auto        err       = hipGetDevice(&currentDeviceId);
     auto&       adapter   = extOpLibraries().at(currentDeviceId);
-    auto        gpu       = Tensile::hip::GetCurrentDevice();
+    auto        gpu       = TensileLite::hip::GetCurrentDevice();
     const auto  archName  = trimArchName(gpu->archName());
     auto&       masterLib = getExtOpMasterLibrary();
-    const auto& lib
-        = masterLib
-              .getLibrary(archName, SoftmaxSolutionLibrary::opName, hipDataTypeo_char(datatype))
-              ->as<SoftmaxSolutionLibrary>();
-    auto sol
-        = lib.findBestSolution(SoftmaxProblem(m, n, hipDataType_to_tensile_type(datatype)), *gpu);
+    const auto& lib       = masterLib
+                          .getLibrary(archName,
+                                      hipblaslt_ext::SoftmaxSolutionLibrary::opName,
+                                      hipDataTypeo_char(datatype))
+                          ->as<hipblaslt_ext::SoftmaxSolutionLibrary>();
+    auto sol = lib.findBestSolution(
+        hipblaslt_ext::SoftmaxProblem(m, n, hipDataType_to_tensile_type(datatype)), *gpu);
     const auto kernelName = sol->name();
     err                   = adapter->initKernel(kernelName);
-    Tensile::KernelArguments kArgs(false);
+    TensileLite::KernelArguments kArgs(false);
     kArgs.append("input", input);
     kArgs.append("output", output);
     kArgs.append("m", m);
     kArgs.append("n", n);
-    const auto                numWorkgroups = getSoftmaxNumWorkgroups(m, tileM);
-    Tensile::KernelInvocation invocation{kernelName,
-                                         sol->getCodeObjectPath(),
-                                         {WORKGROUP_SIZE, 1, 1},
-                                         {numWorkgroups, 1, 1},
-                                         {numWorkgroups * WORKGROUP_SIZE, 1, 1},
-                                         getLdsUsageByte(datatype, tileM, tileN),
-                                         kArgs};
+    const auto                    numWorkgroups = getSoftmaxNumWorkgroups(m, tileM);
+    TensileLite::KernelInvocation invocation{kernelName,
+                                             sol->getCodeObjectPath(),
+                                             false,
+                                             {WORKGROUP_SIZE, 1, 1},
+                                             {numWorkgroups, 1, 1},
+                                             {numWorkgroups * WORKGROUP_SIZE, 1, 1},
+                                             getLdsUsageByte(datatype, tileM, tileN),
+                                             kArgs};
 
     err = adapter->launchKernel(invocation, stream, nullptr, nullptr);
 
@@ -354,20 +319,21 @@ hipblasStatus_t hipblasltLayerNormRun(hipDataType datatype,
     int         currentDeviceId{};
     auto        err       = hipGetDevice(&currentDeviceId);
     auto&       adapter   = extOpLibraries().at(currentDeviceId);
-    auto        gpu       = Tensile::hip::GetCurrentDevice();
+    auto        gpu       = TensileLite::hip::GetCurrentDevice();
     const auto  archName  = trimArchName(gpu->archName());
     auto&       masterLib = getExtOpMasterLibrary();
-    const auto& lib
-        = masterLib
-              .getLibrary(archName, LayerNormSolutionLibrary::opName, hipDataTypeo_char(datatype))
-              ->as<LayerNormSolutionLibrary>();
-    auto sol
-        = lib.findBestSolution(LayerNormProblem(m, n, hipDataType_to_tensile_type(datatype)), *gpu);
+    const auto& lib       = masterLib
+                          .getLibrary(archName,
+                                      hipblaslt_ext::LayerNormSolutionLibrary::opName,
+                                      hipDataTypeo_char(datatype))
+                          ->as<hipblaslt_ext::LayerNormSolutionLibrary>();
+    auto sol = lib.findBestSolution(
+        hipblaslt_ext::LayerNormProblem(m, n, hipDataType_to_tensile_type(datatype)), *gpu);
     const auto kernelName    = sol->name();
     err                      = adapter->initKernel(kernelName);
     const auto numWorkgroups = m;
 
-    Tensile::KernelInvocation invocation;
+    TensileLite::KernelInvocation invocation;
     invocation.kernelName      = kernelName;
     invocation.codeObjectFile  = sol->getCodeObjectPath();
     invocation.workGroupSize.x = sol->getNumWorkitems();
@@ -380,7 +346,7 @@ hipblasStatus_t hipblasltLayerNormRun(hipDataType datatype,
     invocation.numWorkItems.y  = numWorkgroups;
     invocation.numWorkItems.z  = 1;
     invocation.sharedMemBytes  = 32 * sizeof(float);
-    invocation.args            = Tensile::KernelArguments(false);
+    invocation.args            = TensileLite::KernelArguments(false);
     invocation.args.reserve(60, 9);
     invocation.args.append("output", output);
     invocation.args.append("mean", mean);
@@ -422,20 +388,22 @@ hipblasStatus_t hipblasltAMaxRun(const hipDataType datatype,
     int         currentDeviceId{};
     auto        err       = hipGetDevice(&currentDeviceId);
     auto&       adapter   = extOpLibraries().at(currentDeviceId);
-    auto        gpu       = Tensile::hip::GetCurrentDevice();
+    auto        gpu       = TensileLite::hip::GetCurrentDevice();
     const auto  archName  = trimArchName(gpu->archName());
     auto&       masterLib = getExtOpMasterLibrary();
-    const auto& lib
-        = masterLib.getLibrary(archName, AMaxSolutionLibrary::opName, hipDataTypeo_char(datatype))
-              ->as<AMaxSolutionLibrary>();
-    auto       sol        = lib.findBestSolution(AMaxProblem(len,
-                                                hipDataType_to_tensile_type(datatype),
-                                                hipDataType_to_tensile_type(outDatatype)),
-                                    *gpu);
+    const auto& lib       = masterLib
+                          .getLibrary(archName,
+                                      hipblaslt_ext::AMaxSolutionLibrary::opName,
+                                      hipDataTypeo_char(datatype))
+                          ->as<hipblaslt_ext::AMaxSolutionLibrary>();
+    auto sol = lib.findBestSolution(
+        hipblaslt_ext::AMaxProblem(
+            len, hipDataType_to_tensile_type(datatype), hipDataType_to_tensile_type(outDatatype)),
+        *gpu);
     const auto kernelName = sol->name();
     err                   = adapter->initKernel(kernelName);
 
-    Tensile::KernelInvocation invocation;
+    TensileLite::KernelInvocation invocation;
     invocation.kernelName      = kernelName;
     invocation.codeObjectFile  = sol->getCodeObjectPath();
     invocation.workGroupSize.x = sol->getNumWorkitems();
@@ -448,7 +416,7 @@ hipblasStatus_t hipblasltAMaxRun(const hipDataType datatype,
     invocation.numWorkItems.y  = 1;
     invocation.numWorkItems.z  = 1;
     invocation.sharedMemBytes  = 32 * sizeof(float);
-    invocation.args            = Tensile::KernelArguments(false);
+    invocation.args            = TensileLite::KernelArguments(false);
     invocation.args.reserve(20, 3);
     invocation.args.append("output", output);
     invocation.args.append("input", input);
@@ -464,81 +432,67 @@ hipblasStatus_t hipblasltAMaxRun(const hipDataType datatype,
     return HIPBLAS_STATUS_SUCCESS;
 }
 
-hipblasStatus_t hipblasltAMaxWithScaleRun(const hipDataType datatype,
-                                          const hipDataType outDatatype,
-                                          const hipDataType scaleDatatype,
-                                          void*             output,
-                                          void*             outputD,
-                                          void*             input,
-                                          void*             inputScale,
-                                          uint32_t          m,
-                                          uint32_t          n,
-                                          hipStream_t       stream)
+// Setter for the members of class UserClientArguments, only for internal use.
+// The class UserClientArguments is declared and defined in utility.hpp and utility.cpp respectively
+void hipblasltSetFlushValue(bool newFlush)
 {
-    if(datatype != HIP_R_32F
-       || scaleDatatype != HIP_R_8F_E4M3_FNUZ && scaleDatatype != HIP_R_8F_E5M2_FNUZ)
-    {
-        return HIPBLAS_STATUS_NOT_SUPPORTED;
-    }
-
-    if(!output || !outputD || !input || !inputScale || !m || !n)
-    {
-        return HIPBLAS_STATUS_INVALID_VALUE;
-    }
-
-    uint32_t    len = m * n;
-    int         currentDeviceId{};
-    auto        err       = hipGetDevice(&currentDeviceId);
-    auto&       adapter   = extOpLibraries().at(currentDeviceId);
-    auto        gpu       = Tensile::hip::GetCurrentDevice();
-    const auto  archName  = trimArchName(gpu->archName());
-    auto&       masterLib = getExtOpMasterLibrary();
-    const auto& lib
-        = masterLib.getLibrary(archName, AMaxSolutionLibrary::opName, hipDataTypeo_char(datatype))
-              ->as<AMaxSolutionLibrary>();
-    auto sol = lib.findBestSolution(AMaxProblem(len,
-                                                hipDataType_to_tensile_type(datatype),
-                                                hipDataType_to_tensile_type(outDatatype),
-                                                hipDataType_to_tensile_type(scaleDatatype),
-                                                true),
-                                    *gpu);
-
-    if(!sol)
-    {
-        std::cerr << "AMaxWithScale: No valid solution found!" << std::endl;
-        return HIPBLAS_STATUS_SUCCESS;
-    }
-
-    const auto kernelName = sol->name();
-    err                   = adapter->initKernel(kernelName);
-
-    Tensile::KernelInvocation invocation;
-    invocation.kernelName      = kernelName;
-    invocation.codeObjectFile  = sol->getCodeObjectPath();
-    invocation.workGroupSize.x = sol->getNumWorkitems();
-    invocation.workGroupSize.y = 1;
-    invocation.workGroupSize.z = 1;
-    invocation.numWorkGroups.x = 1;
-    invocation.numWorkGroups.y = 1;
-    invocation.numWorkGroups.z = 1;
-    invocation.numWorkItems.x  = sol->getNumWorkitems();
-    invocation.numWorkItems.y  = 1;
-    invocation.numWorkItems.z  = 1;
-    invocation.sharedMemBytes  = 32 * sizeof(float);
-    invocation.args            = Tensile::KernelArguments(false);
-    invocation.args.reserve(20, 3);
-    invocation.args.append("output", output);
-    invocation.args.append("outputD", outputD);
-    invocation.args.append("input", input);
-    invocation.args.append("inputScae", inputScale);
-    invocation.args.append("length", len);
-
-    err = adapter->launchKernel(invocation, stream, nullptr, nullptr);
-
-    if(err)
-    {
-        return HIPBLAS_STATUS_INTERNAL_ERROR;
-    }
-
-    return HIPBLAS_STATUS_SUCCESS;
+    UserClientArguments clientArguments;
+    clientArguments.SetFlushValue(newFlush);
+}
+void hipblasltSetRotatingBufferSizeValue(int newrotatingBufferSize)
+{
+    UserClientArguments clientArguments;
+    clientArguments.SetRotatingBufferSizeValue(newrotatingBufferSize);
+}
+
+void hipblasltSetColdIterationsValue(int newColdIterations)
+{
+    UserClientArguments clientArguments;
+    clientArguments.SetColdIterationsValue(newColdIterations);
+}
+
+void hipblasltSetHotIterationsValue(int newHotIterations)
+{
+    UserClientArguments clientArguments;
+    clientArguments.SetHotIterationsValue(newHotIterations);
+}
+
+// Get hipblaslt client performance args, For internal use only.
+double hipblasltGetTotalGranularityValue()
+{
+    return hipblasltClientPerformanceArgs::totalGranularity;
+}
+
+double hipblasltGetTilesPerCuValue()
+{
+    return hipblasltClientPerformanceArgs::tilesPerCu;
+}
+
+double hipblasltGetTile0Granularity()
+{
+    return hipblasltClientPerformanceArgs::tile0Granularity;
+}
+double hipblasltGetTile1Granularity()
+{
+    return hipblasltClientPerformanceArgs::tile1Granularity;
+}
+double hipblasltGetCuGranularity()
+{
+    return hipblasltClientPerformanceArgs::cuGranularity;
+}
+double hipblasltGetWaveGranularity()
+{
+    return hipblasltClientPerformanceArgs::waveGranularity;
+}
+int hipblasltGetCUs()
+{
+    return hipblasltClientPerformanceArgs::CUs;
+}
+size_t hipblasltGetMemWriteBytesD()
+{
+    return hipblasltClientPerformanceArgs::memWriteBytesD;
+}
+size_t hipblasltGetMemReadBytes()
+{
+    return hipblasltClientPerformanceArgs::memReadBytes;
 }
