@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -67,6 +67,16 @@ inline rocblaslt_status getOriginalSizes(hipblasOperation_t opA,
     return rocblaslt_status_success;
 }
 
+inline bool isValidOrderForDatatype(hipDataType datatype, hipblasLtOrder_t order)
+{
+    if((datatype == HIP_R_16F && order != HIPBLASLT_ORDER_COL16_4R8)
+       || (datatype == HIP_R_8F_E4M3_FNUZ && order != HIPBLASLT_ORDER_COL16_4R16))
+    {
+        return false;
+    }
+    return true;
+}
+
 /*******************************************************************************
  * Validate Matmul Descr. init Arguments - matrix init.
  ******************************************************************************/
@@ -137,6 +147,49 @@ inline rocblaslt_status validateMatmulDescrArgs(rocblaslt_handle       handle,
     return rocblaslt_status_success;
 }
 
+/*******************************************************************************
+ * Validate Matmul Swizzle Arguments
+ ******************************************************************************/
+inline rocblaslt_status validateMatmulSwizzleArgs(const rocblaslt_matmul_desc   matmul_descr,
+                                                  const rocblaslt_matrix_layout matA,
+                                                  const rocblaslt_matrix_layout matB,
+                                                  hipDataType                   a_type,
+                                                  hipDataType                   b_type,
+                                                  bool                          swizzleA,
+                                                  bool                          swizzleB)
+{
+    //support TN for swizzle
+    if(swizzleA && matmul_descr->op_A != HIPBLAS_OP_T)
+        return rocblaslt_status_invalid_value;
+
+    if(swizzleB && matmul_descr->op_B != HIPBLAS_OP_N)
+        return rocblaslt_status_invalid_value;
+
+    if(swizzleA && !isValidOrderForDatatype(a_type, matA->order))
+    {
+        log_error(__func__, "Error: Invalid Order for hipDataType A.");
+        return rocblaslt_status_invalid_value;
+    }
+
+    if(swizzleB && !isValidOrderForDatatype(b_type, matB->order))
+    {
+        log_error(__func__, "Error: Invalid Order for hipDataType B.");
+        return rocblaslt_status_invalid_value;
+    }
+
+    if(swizzleA && matA->ld != matA->m)
+        log_hints(__func__,
+                  "Warning: The lda parameter is ignored and disabled when swizzle_a is true. "
+                  "There's no need to set lda.");
+
+    if(swizzleB && matB->ld != matB->m)
+        log_hints(__func__,
+                  "Warning: The ldb parameter is ignored and disabled when swizzle_b is true. "
+                  "There's no need to set ldb.");
+
+    return rocblaslt_status_continue;
+}
+
 /*******************************************************************************
  * Validate Matmul Arguments
  ******************************************************************************/
@@ -177,9 +230,12 @@ inline rocblaslt_status validateMatmulArgs(int64_t                       m,
        && compute_type == rocblaslt_compute_i32)
         status = rocblaslt_status_not_implemented;
     if(!strcmp(hip_datatype_to_string(type_a), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
-       || !strcmp(hip_datatype_to_string(type_b), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
-       || !strcmp(hip_datatype_to_string(type_c), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
-       || !strcmp(hip_datatype_to_string(type_d), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_b),
+                  hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_c),
+                  hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_d),
+                  hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
        || !strcmp(rocblaslt_compute_type_string(compute_type),
                   rocblaslt_compute_type_string(ROCBLASLT_COMPUTE_TYPE_INVALID)))
         status = rocblaslt_status_not_implemented;
@@ -252,26 +308,33 @@ inline rocblaslt_status validateMatmulArgs(int64_t                       m,
     return rocblaslt_status_continue;
 }
 
-inline rocblaslt_status rocblaslt_epilogue_valid_args(const rocblaslt_epilogue& epilogue,
-                                                      const int64_t&            num_rows_e,
-                                                      const int64_t&            num_cols_e,
-                                                      const hipDataType&        d_type,
-                                                      const hipDataType&        original_bias_type,
-                                                      const void*               e_ptr,
-                                                      const int64_t&            original_lde,
-                                                      const int64_t&            original_stride_e,
-                                                      const void*               original_bias,
-                                                      const void*  original_scaleAlphaVec,
-                                                      const void*  alpha,
-                                                      const bool   isScaleAVec,
-                                                      const bool   isScaleBVec,
-                                                      void*&       E,
-                                                      int64_t&     lde,
-                                                      int64_t&     batch_stride_e,
-                                                      void*&       bias,
-                                                      hipDataType& bias_type,
-                                                      void*&       scaleAlphaVec,
-                                                      bool&        gradient)
+inline rocblaslt_status
+    rocblaslt_epilogue_valid_args(const rocblaslt_epilogue& epilogue,
+                                  const int64_t&            num_rows_e,
+                                  const int64_t&            num_cols_e,
+                                  const hipDataType&        d_type,
+                                  const hipDataType&        original_bias_type,
+                                  const void*               e_ptr,
+                                  const hipDataType&        original_aux_type,
+                                  const int64_t&            original_lde,
+                                  const int64_t&            original_stride_e,
+                                  const void*               original_bias,
+                                  const void*               original_scaleAlphaVec,
+                                  const void*               alpha,
+                                  const RocblasltContractionProblem::ScalingFormat scaleAType,
+                                  const RocblasltContractionProblem::ScalingFormat scaleBType,
+                                  const uint32_t scaleABlockRowSize,
+                                  const uint32_t scaleABlockColSize,
+                                  const uint32_t scaleBBlockRowSize,
+                                  const uint32_t scaleBBlockColSize,
+                                  void*&         E,
+                                  hipDataType&   aux_type,
+                                  int64_t&       lde,
+                                  int64_t&       batch_stride_e,
+                                  void*&         bias,
+                                  hipDataType&   bias_type,
+                                  void*&         scaleAlphaVec,
+                                  bool&          gradient)
 {
     // Set status
     rocblaslt_status status = rocblaslt_status_continue;
@@ -300,15 +363,34 @@ inline rocblaslt_status rocblaslt_epilogue_valid_args(const rocblaslt_epilogue&
             status = rocblaslt_status_invalid_pointer;
         E = (void*)e_ptr;
     }
+    aux_type       = original_aux_type;
     lde            = original_lde > 0 ? original_lde : num_rows_e;
     batch_stride_e = original_stride_e > 0 ? original_stride_e : original_lde * num_cols_e;
     if(E != nullptr && ((lde < num_rows_e) || (batch_stride_e < (num_cols_e * num_rows_e))))
         status = rocblaslt_status_invalid_value;
-    if(isScaleAVec != isScaleBVec)
+    if(scaleAType != RocblasltContractionProblem::ScalingFormat::None
+       && scaleBType != RocblasltContractionProblem::ScalingFormat::None
+       && scaleAType != scaleBType)
     {
-        log_error(__func__, "Scale A and Scale B must be both scalar or vector.");
+        log_error(__func__, "Scale A and Scale B must be both scalar, vector or block.");
         status = rocblaslt_status_invalid_value;
     }
+    if(scaleAType == RocblasltContractionProblem::ScalingFormat::Block)
+    {
+        if(scaleABlockRowSize != 32 || scaleABlockColSize != 1)
+        {
+            log_error(__func__, "ScaleA block row and column sizes currently only support 32x1");
+            status = rocblaslt_status_invalid_value;
+        }
+    }
+    if(scaleBType == RocblasltContractionProblem::ScalingFormat::Block)
+    {
+        if(scaleBBlockRowSize != 1 || scaleBBlockColSize != 32)
+        {
+            log_error(__func__, "ScaleB block row and column sizes currently only support 1x32");
+            status = rocblaslt_status_invalid_value;
+        }
+    }
     return status;
 }
 
@@ -344,11 +426,21 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
                                                     hipDataType&                bias_type,
                                                     void*&                      scaleAlphaVec,
                                                     void*&                      E,
+                                                    hipDataType&                aux_type,
                                                     bool&                       gradient,
-                                                    rocblaslt_compute_type&     compute_type)
+                                                    rocblaslt_compute_type&     compute_type,
+                                                    bool                        swizzleA,
+                                                    bool                        swizzleB)
 {
     // Internal assign
     hipblasOperation_t opA = matmul_descr->op_A;
+    hipblasOperation_t opB = matmul_descr->op_B;
+
+    auto matmul_swizzle_status
+        = validateMatmulSwizzleArgs(matmul_descr, matA, matB, a_type, b_type, swizzleA, swizzleB);
+
+    if(matmul_swizzle_status != rocblaslt_status_continue)
+        return matmul_swizzle_status;
 
     // matrix A
     int64_t num_rows_a    = matA->m;
@@ -417,14 +509,20 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
                                                          matD->type,
                                                          matmul_descr->bias_type,
                                                          matmul_descr->e,
+                                                         matmul_descr->aux_type,
                                                          matmul_descr->lde,
                                                          matmul_descr->stride_e,
                                                          matmul_descr->bias,
                                                          alphaVecPtr,
                                                          alpha,
-                                                         matmul_descr->isScaleAVec,
-                                                         matmul_descr->isScaleBVec,
+                                                         matmul_descr->scaleAType,
+                                                         matmul_descr->scaleBType,
+                                                         matmul_descr->scaleABlockRowSize,
+                                                         matmul_descr->scaleABlockColSize,
+                                                         matmul_descr->scaleBBlockRowSize,
+                                                         matmul_descr->scaleBBlockColSize,
                                                          E,
+                                                         aux_type,
                                                          lde,
                                                          batch_stride_e,
                                                          bias,
