@@ -3,7 +3,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -57,250 +57,6 @@ constexpr double value_category(const T& beta)
     return beta == T(0) ? 0.0 : beta == T(1) ? 1.0 : beta == T(-1) ? -1.0 : 2.0;
 }
 
-/********************************************************************
- * RocblasltContractionProblem captures the arguments for a GEMM-like *
- * contraction problem, to be passed to runContractionProblem.      *
- ********************************************************************/
-struct RocblasltContractionProblem
-{
-    hipblasOperation_t trans_a;
-    hipblasOperation_t trans_b;
-
-    // The RocblasltContractionProblem data members should exactly match
-    // Tensile's parameter types, even if rocBLAS uses differently
-    // sized or signed types. The constructors should convert rocBLAS
-    // types into the corresponding Tensile types stored in this class.
-    size_t m;
-    size_t n;
-    size_t k;
-
-    const void* alpha;
-
-    hipDataType        a_type;
-    const void*        A;
-    const void* const* batch_A;
-    size_t             row_stride_a;
-    size_t             col_stride_a;
-    size_t             batch_stride_a;
-
-    hipDataType        b_type;
-    const void*        B;
-    const void* const* batch_B;
-    size_t             row_stride_b;
-    size_t             col_stride_b;
-    size_t             batch_stride_b;
-
-    const void* beta;
-
-    hipDataType        c_type;
-    const void*        C;
-    const void* const* batch_C;
-    size_t             row_stride_c;
-    size_t             col_stride_c;
-    size_t             batch_stride_c;
-
-    hipDataType  d_type;
-    void*        D;
-    void* const* batch_D;
-    size_t       row_stride_d;
-    size_t       col_stride_d;
-    size_t       batch_stride_d;
-
-    void*        E;
-    void* const* batch_E;
-    size_t       row_stride_e;
-    size_t       col_stride_e;
-    size_t       batch_stride_e;
-
-    size_t batch_count;
-    bool   strided_batch;
-    bool   grouped_gemm;
-    bool   gradient;
-
-    rocblaslt_compute_type compute_type;
-
-    const void*        bias;
-    const void*        scaleA;
-    const void*        scaleB;
-    const void*        scaleC;
-    const void*        scaleD;
-    const void*        scaleE;
-    const void*        scaleAlphaVec;
-    bool               isScaleAVec;
-    bool               isScaleBVec;
-    hipDataType        bias_type;
-    rocblaslt_epilogue epilogue;
-    void*              amaxD;
-    void*              workspace;
-    size_t             workspaceSize;
-
-    hipStream_t stream;
-    void*       Synchronizer;
-
-    // gemm_ex
-    // gemm_strided_batched_ex
-    RocblasltContractionProblem(hipblasOperation_t     trans_a,
-                                hipblasOperation_t     trans_b,
-                                int64_t                m,
-                                int64_t                n,
-                                int64_t                k,
-                                const void*            alpha,
-                                hipDataType            a_type,
-                                const void*            A,
-                                const void* const*     batch_A,
-                                int64_t                ld_a,
-                                int64_t                batch_stride_a,
-                                hipDataType            b_type,
-                                const void*            B,
-                                const void* const*     batch_B,
-                                int64_t                ld_b,
-                                int64_t                batch_stride_b,
-                                const void*            beta,
-                                hipDataType            c_type,
-                                const void*            C,
-                                const void* const*     batch_C,
-                                int64_t                ld_c,
-                                int64_t                batch_stride_c,
-                                hipDataType            d_type,
-                                void*                  D,
-                                void* const*           batch_D,
-                                int64_t                ld_d,
-                                int64_t                batch_stride_d,
-                                void*                  E,
-                                void* const*           batch_E,
-                                int64_t                ld_e,
-                                int64_t                batch_stride_e,
-                                int64_t                batch_count,
-                                bool                   strided_batch,
-                                bool                   grouped_gemm,
-                                bool                   gradient,
-                                rocblaslt_compute_type compute_type,
-                                const void*            bias,
-                                const void*            scaleA,
-                                const void*            scaleB,
-                                const void*            scaleC,
-                                const void*            scaleD,
-                                const void*            scaleE,
-                                const void*            scaleAlphaVec,
-                                bool                   isScaleAVec,
-                                bool                   isScaleBVec,
-                                hipDataType            bias_type,
-                                rocblaslt_epilogue     epilogue,
-                                void*                  amaxD,
-                                void*                  workspace,
-                                size_t                 workspaceSize,
-                                hipStream_t            stream,
-                                void*                  Synchronizer)
-        : trans_a(trans_a)
-        , trans_b(trans_b)
-        , m(m)
-        , n(n)
-        , k(k)
-        , alpha(alpha)
-        , a_type(a_type)
-        , A(A)
-        , batch_A(batch_A)
-        , row_stride_a(1)
-        , col_stride_a(ld_a)
-        , batch_stride_a(batch_stride_a)
-        , b_type(b_type)
-        , B(B)
-        , batch_B(batch_B)
-        , row_stride_b(1)
-        , col_stride_b(ld_b)
-        , batch_stride_b(batch_stride_b)
-        , beta(beta)
-        , c_type(c_type)
-        , C(C)
-        , batch_C(batch_C)
-        , row_stride_c(1)
-        , col_stride_c(ld_c)
-        , batch_stride_c(batch_stride_c)
-        , d_type(d_type)
-        , D(D)
-        , batch_D(batch_D)
-        , row_stride_d(1)
-        , col_stride_d(ld_d)
-        , batch_stride_d(batch_stride_d)
-        , E(E)
-        , batch_E(batch_E)
-        , row_stride_e(1)
-        , col_stride_e(ld_e)
-        , batch_stride_e(batch_stride_e)
-        , batch_count(batch_count)
-        , strided_batch(strided_batch)
-        , grouped_gemm(grouped_gemm)
-        , gradient(gradient)
-        , compute_type(compute_type)
-        , bias(bias)
-        , scaleA(scaleA)
-        , scaleB(scaleB)
-        , scaleC(scaleC)
-        , scaleD(scaleD)
-        , scaleE(scaleE)
-        , scaleAlphaVec(scaleAlphaVec)
-        , isScaleAVec(isScaleAVec)
-        , isScaleBVec(isScaleBVec)
-        , bias_type(bias_type)
-        , epilogue(epilogue)
-        , amaxD(amaxD)
-        , workspace(workspace)
-        , workspaceSize(workspaceSize)
-        , stream(stream)
-        , Synchronizer(Synchronizer)
-    {
-        if(this->bias_type == HIPBLASLT_DATATYPE_INVALID)
-        {
-            // FIXME: Currently the default bias_type is set to match the yamls' configuration, should add the default type when the yamls are fixed.
-            if(this->compute_type == rocblaslt_compute_i32)
-            {
-                this->bias_type = HIP_R_32I;
-            }
-            else if(this->compute_type == rocblaslt_compute_f32_fast_xf32)
-            {
-                this->bias_type = HIP_R_32F;
-            }
-            else if((this->a_type == HIP_R_8F_E4M3_FNUZ || this->a_type == HIP_R_8F_E5M2_FNUZ)
-                    && (this->b_type == HIP_R_8F_E4M3_FNUZ || this->b_type == HIP_R_8F_E5M2_FNUZ))
-            {
-                if(this->d_type == HIP_R_32F || this->d_type == HIP_R_16BF)
-                    this->bias_type = HIP_R_16BF;
-                else if(this->d_type == HIP_R_16F)
-                    this->bias_type = HIP_R_16F;
-                else //more default cases once support C != D
-                    this->bias_type = HIP_R_16F;
-            }
-#ifdef ROCM_USE_FLOAT8
-            else if((this->a_type == HIP_R_8F_E4M3 || this->a_type == HIP_R_8F_E5M2)
-                    && (this->b_type == HIP_R_8F_E4M3 || this->b_type == HIP_R_8F_E5M2))
-            {
-                if(this->d_type == HIP_R_32F || this->d_type == HIP_R_16BF)
-                    this->bias_type = HIP_R_16BF;
-                else if(this->d_type == HIP_R_16F)
-                    this->bias_type = HIP_R_16F;
-                else //more default cases once support C != D
-                    this->bias_type = HIP_R_16F;
-            }
-#endif
-            else
-            {
-                this->bias_type = this->d_type;
-            }
-        }
-
-        if(this->trans_a == HIPBLAS_OP_C)
-        {
-            if(rocblaslt_is_complex_datatype(this->a_type))
-                this->trans_a = HIPBLAS_OP_T;
-        }
-        if(this->trans_b == HIPBLAS_OP_C)
-        {
-            if(rocblaslt_is_complex_datatype(this->b_type))
-                this->trans_b = HIPBLAS_OP_T;
-        }
-    }
-};
-
 void initTensileGemmData(rocblaslt_handle       handle,
                          rocblaslt::RocGemmType gemmType,
                          hipblasOperation_t     opA,
@@ -460,41 +216,65 @@ rocblaslt_status getBestSolutions(rocblaslt_handle       handle,
 /******************************************************
  * Map a hipblaslt data type to a corresponding Tensile type *
  ******************************************************/
-inline TensileLite::DataType hipDataType_to_tensile_type(hipDataType type)
+inline rocisa::DataType hipDataType_to_tensile_type(hipDataType type)
 {
     switch(type)
     {
     case HIP_R_16F:
-        return TensileLite::DataType::Half;
+        return rocisa::DataType::Half;
     case HIP_R_32F:
-        return TensileLite::DataType::Float;
+        return rocisa::DataType::Float;
     case HIP_R_64F:
-        return TensileLite::DataType::Double;
+        return rocisa::DataType::Double;
     case HIP_R_16BF:
-        return TensileLite::DataType::BFloat16;
+        return rocisa::DataType::BFloat16;
     case HIP_R_8F_E4M3_FNUZ:
-        return TensileLite::DataType::Float8;
+        return rocisa::DataType::Float8_fnuz;
     case HIP_R_8F_E5M2_FNUZ:
-        return TensileLite::DataType::BFloat8;
-#ifdef ROCM_USE_FLOAT8
+        return rocisa::DataType::BFloat8_fnuz;
     case HIP_R_8F_E4M3:
-        return TensileLite::DataType::Float8;
+        return rocisa::DataType::Float8;
     case HIP_R_8F_E5M2:
-        return TensileLite::DataType::BFloat8;
-#endif
+        return rocisa::DataType::BFloat8;
     case HIP_R_8I:
-        return TensileLite::DataType::Int8;
+        return rocisa::DataType::Int8;
     case HIP_R_32I:
-        return TensileLite::DataType::Int32;
+        return rocisa::DataType::Int32;
     default:
         assert(!"hipDataType_to_tensile_type: non-supported type");
-        return TensileLite::DataType::None;
+        return rocisa::DataType::None;
     }
 }
 
-namespace
+inline rocisa::DataType rocComputeType_to_tensile_type(rocblaslt_compute_type type)
 {
-    TensileLite::DataType roc2TensileType(rocblaslt_compute_type, bool);
+    switch(type)
+    {
+    case rocblaslt_compute_f32_fast_xf32:
+        return rocisa::DataType::XFloat32;
+    case rocblaslt_compute_f32_fast_f16:
+        return rocisa::DataType::Half;
+    case rocblaslt_compute_f32_fast_bf16:
+        return rocisa::DataType::BFloat16;
+    case rocblaslt_compute_f16:
+    case rocblaslt_compute_f32:
+    case rocblaslt_compute_f32_fast_f8_fnuz:
+    case rocblaslt_compute_f32_fast_bf8_fnuz:
+    case rocblaslt_compute_f32_fast_f8bf8_fnuz:
+    case rocblaslt_compute_f32_fast_bf8f8_fnuz:
+    case rocblaslt_compute_f32_fast_f8:
+    case rocblaslt_compute_f32_fast_bf8:
+    case rocblaslt_compute_f32_fast_f8bf8:
+    case rocblaslt_compute_f32_fast_bf8f8:
+        return rocisa::DataType::Float;
+    case rocblaslt_compute_f64:
+        return rocisa::DataType::Double;
+    case rocblaslt_compute_i32:
+        return rocisa::DataType::Int32;
+    default:
+        assert(!"rocDataType_to_tensile_type: non-supported type");
+        return rocisa::DataType::None;
+    }
 }
 
 namespace TensileLite
@@ -506,3 +286,5 @@ TensileLite::ProblemOverride
     RocblasltContractionProblem2ProblemOverride(const RocblasltContractionProblem&);
 
 TensileLite::ProblemOverride TensileDataGemm2ProblemOverride(std::shared_ptr<void>);
+
+TensileLite::ContractionProblemGemm* ExtractProblemGemm(std::shared_ptr<void>);
