@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,21 +24,33 @@
  *
  * ************************************************************************ */
 
+#ifdef _WIN32
+#include <Windows.h>
+#include <io.h>
+#include <libloaderapi.h>
+
+// Remove defines that conflict locally.
+#undef CONST
+#else
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
+#include <dlfcn.h>
+#include <link.h>
+#include <unistd.h>
+#endif
+
 #include "UserDrivenTuningParser.hpp"
 #include "definitions.h"
 #include "handle.h"
 #include "rocblaslt.h"
 #include "rocblaslt_mat_utils.hpp"
+#include "rocroller_host.hpp"
 #include "tensile_host.hpp"
 #include "utility.hpp"
 
-#ifndef WIN32
-#include <link.h>
-#endif
-
 #include <hip/hip_runtime_api.h>
 #include <map>
-#include <unistd.h>
 #include <utility>
 
 #define TO_STR2(x) #x
@@ -68,6 +80,33 @@ inline void assignAlphaBeta1(const rocblaslt_compute_type& compute_type, void* a
     }
 }
 
+inline void setDefaultSwizzledBatchedStride(const rocblaslt_matrix_layout& matLayout,
+                                            int64_t&                       batch_stride)
+{
+    size_t MiM = 16, MiK = 0, MiKv = 0, PackK = 0;
+    if(matLayout->order == HIPBLASLT_ORDER_COL16_4R8)
+    {
+        //f16
+        MiK   = 16;
+        MiKv  = 4;
+        PackK = 16 / MiKv / 2;
+    }
+    else if(matLayout->order == HIPBLASLT_ORDER_COL16_4R16)
+    {
+        //f8
+        MiK   = 32;
+        MiKv  = 8;
+        PackK = 16 / MiKv / 1;
+    }
+    else
+        return;
+
+    size_t K_block = MiK * PackK;
+    //align to k for swizzleK and to m for 16
+    batch_stride = ((matLayout->n + MiM - 1) / MiM) * MiM * ((matLayout->m + K_block - 1) / K_block)
+                   * K_block;
+}
+
 inline void heuristicResult_copy(rocblaslt_matmul_heuristic_result* heuristicResultsDest,
                                  rocblaslt_matmul_heuristic_result* heuristicResultsSrc,
                                  size_t&                            maxWorkSpaceBytes,
@@ -111,11 +150,11 @@ inline bool
 
 // Preload problem/solution mappings
 bool problem_override_from_file(rocblaslt_handle&                 handle,
-                                rocblaslt_matmul_preference&      pref,
                                 RocblasltContractionProblem&      problem,
                                 rocblaslt_matmul_desc&            matmul_desc,
                                 rocblaslt_matmul_heuristic_result heuristicResultsArray[],
-                                const std::string&                file_path)
+                                const std::string&                file_path,
+                                size_t                            max_workspace_bytes)
 {
 
     bool success = false;
@@ -133,15 +172,13 @@ bool problem_override_from_file(rocblaslt_handle&                 handle,
         TensileLite::ProblemOverride prob_key(RocblasltContractionProblem2ProblemOverride(problem));
         auto                         sol_iter = m_override.find(prob_key);
 
-        for(auto sol_idx = std::make_reverse_iterator(sol_iter.second);
-            !success && sol_idx != std::make_reverse_iterator(sol_iter.first);
-            sol_idx++)
+        for(auto sol_idx = sol_iter.first; !success && sol_idx != sol_iter.second; sol_idx++)
         {
             solutionIndex[0] = sol_idx->second;
 
             if(rocblaslt_status_success
                == getSolutionsFromIndex(
-                   handle, solutionIndex, overrideResults, pref->max_workspace_bytes))
+                   handle, solutionIndex, overrideResults, max_workspace_bytes))
             {
 
                 size_t required_workspace_size = 0;
@@ -153,13 +190,36 @@ bool problem_override_from_file(rocblaslt_handle&                 handle,
                                           tensile_data,
                                           &overrideResults[0].algo,
                                           &required_workspace_size))
+                {
+                    success = true;
+                }
+                else
+                { // there is no solution for xfloat32, fallback comput_type to fp32
+                    if(problem.compute_type == rocblaslt_compute_f32_fast_xf32)
+                    {
+                        problem.compute_type = rocblaslt_compute_f32;
+                        if(rocblaslt_status_success
+                           == isSolutionSupported(handle,
+                                                  problem,
+                                                  tensile_data,
+                                                  &overrideResults[0].algo,
+                                                  &required_workspace_size))
+                        {
+                            success = true;
+                            log_info(__func__, "Use the fallback fp32 solution");
+                        }
+
+                        problem.compute_type = rocblaslt_compute_f32_fast_xf32;
+                    }
+                }
+
+                if(success)
                 {
 
                     heuristicResult_copy(&heuristicResultsArray[0],
                                          &overrideResults[0],
-                                         pref->max_workspace_bytes,
+                                         max_workspace_bytes,
                                          required_workspace_size);
-                    success = true;
                 }
             }
         }
@@ -183,9 +243,9 @@ bool problem_override_from_file_cpp(
     rocblaslt_handle&                               handle,
     rocblaslt::RocGemmType&                         gemmType,
     std::shared_ptr<void>                           gemmData,
-    size_t                                          workspaceSizeInBytes,
     std::vector<rocblaslt_matmul_heuristic_result>& heuristicResultsArray,
-    const std::string&                              file_path)
+    const std::string&                              file_path,
+    size_t                                          max_workspace_bytes)
 {
 
     bool success = false;
@@ -203,28 +263,53 @@ bool problem_override_from_file_cpp(
         TensileLite::ProblemOverride prob_key(TensileDataGemm2ProblemOverride(gemmData));
         auto                         sol_iter = m_override.find(prob_key);
 
-        for(auto sol_idx = std::make_reverse_iterator(sol_iter.second);
-            !success && sol_idx != std::make_reverse_iterator(sol_iter.first);
-            sol_idx++)
+        for(auto sol_idx = sol_iter.first; !success && sol_idx != sol_iter.second; sol_idx++)
         {
-            solutionIndex[0]        = sol_idx->second;
-            size_t maxWorkspaceSize = std::numeric_limits<size_t>::max();
+            solutionIndex[0] = sol_idx->second;
             if(rocblaslt_status_success
-               == getSolutionsFromIndex(handle, solutionIndex, overrideResults, maxWorkspaceSize))
+               == getSolutionsFromIndex(
+                   handle, solutionIndex, overrideResults, max_workspace_bytes))
             {
-                rocblaslt::RocTuningV2* tuning = nullptr;
+
+                size_t                  required_workspace_size = 0;
+                rocblaslt::RocTuningV2* tuning                  = nullptr;
+
                 if(rocblaslt_status_success
                    == isSolutionSupported(handle,
                                           static_cast<const rocblaslt::RocGemmType>(gemmType),
                                           gemmData,
                                           overrideResults[0].algo,
                                           tuning,
-                                          workspaceSizeInBytes))
+                                          required_workspace_size))
                 {
-                    overrideResults[0].workspaceSize = workspaceSizeInBytes;
-                    heuristicResultsArray.push_back(overrideResults[0]);
                     success = true;
                 }
+                else
+                { // there is no solution for xfloat32, fallback comput_type to fp32
+                    auto problem = ExtractProblemGemm(gemmData);
+                    if(problem->f32XdlMathOp() == rocisa::DataType::XFloat32)
+                    {
+                        problem->setF32XdlMathOp(rocisa::DataType::Float);
+                        if(rocblaslt_status_success
+                           == isSolutionSupported(
+                               handle,
+                               static_cast<const rocblaslt::RocGemmType>(gemmType),
+                               gemmData,
+                               overrideResults[0].algo,
+                               tuning,
+                               required_workspace_size))
+                        {
+                            success = true;
+                            log_info(__func__, "Use the fallback fp32 solution");
+                        }
+                    }
+                }
+
+                if(success)
+                {
+                    overrideResults[0].workspaceSize = required_workspace_size;
+                    heuristicResultsArray.push_back(overrideResults[0]);
+                }
             }
         }
 
@@ -262,11 +347,21 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
     int64_t     m, n, k, lda, ldb, ldc, ldd, lde, batch_stride_a, batch_stride_b, batch_stride_c,
         batch_stride_d, batch_stride_e;
     hipDataType            bias_type;
+    hipDataType            aux_type;
     hipDataType            a_type, b_type, c_type, d_type;
     rocblaslt_compute_type compute_type;
     void *                 bias = nullptr, *scaleAlphaVec = nullptr, *e = nullptr;
     bool                   gradient = false;
-    rocblaslt_status       isValid  = rocblaslt_matmul_valid_args(matmul_descr,
+    bool swizzleA = matA->order != HIPBLASLT_ORDER_COL && matA->order != HIPBLASLT_ORDER_ROW;
+    bool swizzleB = matB->order != HIPBLASLT_ORDER_COL && matB->order != HIPBLASLT_ORDER_ROW;
+
+    if(swizzleA && matA->batch_stride == 0)
+    {
+        //If batch_stride has never been assigned for swizzle, set it to the default value
+        setDefaultSwizzledBatchedStride(matA, matA->batch_stride);
+    }
+
+    rocblaslt_status isValid = rocblaslt_matmul_valid_args(matmul_descr,
                                                            dummy_ptr,
                                                            dummy_ptr,
                                                            dummy_ptr,
@@ -298,8 +393,11 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
                                                            bias_type,
                                                            scaleAlphaVec,
                                                            e,
+                                                           aux_type,
                                                            gradient,
-                                                           compute_type);
+                                                           compute_type,
+                                                           swizzleA,
+                                                           swizzleB);
     if(isValid != rocblaslt_status_continue)
     {
         m = 0;
@@ -365,6 +463,7 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
                                         grouped_gemm,
                                         gradient,
                                         compute_type,
+                                        matmul_descr->scale_type,
                                         bias,
                                         scaleA,
                                         scaleB,
@@ -372,15 +471,22 @@ RocblasltContractionProblem construct_rocblaslt_problem(rocblaslt_handle
                                         scaleD,
                                         scaleE,
                                         scaleAlphaVec,
-                                        matmul_descr->isScaleAVec,
-                                        matmul_descr->isScaleBVec,
+                                        matmul_descr->scaleAType,
+                                        matmul_descr->scaleBType,
+                                        matmul_descr->scaleABlockRowSize,
+                                        matmul_descr->scaleABlockColSize,
+                                        matmul_descr->scaleBBlockRowSize,
+                                        matmul_descr->scaleBBlockColSize,
                                         bias_type,
+                                        aux_type,
                                         epilogue,
                                         amaxD,
                                         nullptr,
                                         maxWorkSpaceBytes,
                                         nullptr,
-                                        handle->Synchronizer};
+                                        handle->Synchronizer,
+                                        swizzleA,
+                                        swizzleB};
 
     return problem;
 }
@@ -432,7 +538,11 @@ rocblaslt_status rocblaslt_destroy(const rocblaslt_handle handle)
         return rocblaslt_status_invalid_value;
     }
     log_api(__func__, "handle", handle);
-    // Destruct
+// Destruct
+#ifdef HIPBLASLT_USE_ROCROLLER
+    if(handle->rocroller_handle)
+        rocroller_destroy_handle(handle->rocroller_handle);
+#endif
     try
     {
         delete handle;
@@ -744,12 +854,10 @@ rocblaslt_status rocblaslt_matmul_desc_create(rocblaslt_matmul_desc* matmulDesc,
             case rocblaslt_compute_f32_fast_bf8_fnuz:
             case rocblaslt_compute_f32_fast_f8bf8_fnuz:
             case rocblaslt_compute_f32_fast_bf8f8_fnuz:
-#ifdef ROCM_USE_FLOAT8
-            case rocblaslt_compute_f32_fast_f8_ocp:
-            case rocblaslt_compute_f32_fast_bf8_ocp:
-            case rocblaslt_compute_f32_fast_f8bf8_ocp:
-            case rocblaslt_compute_f32_fast_bf8f8_ocp:
-#endif
+            case rocblaslt_compute_f32_fast_f8:
+            case rocblaslt_compute_f32_fast_bf8:
+            case rocblaslt_compute_f32_fast_f8bf8:
+            case rocblaslt_compute_f32_fast_bf8f8:
                 break;
             default:
                 log_error(__func__, "invalid compute type", computeType);
@@ -847,16 +955,14 @@ rocblaslt_compute_type _matmul_desc_determine_compute_type(rocblaslt_matmul_desc
             return rocblaslt_compute_f32_fast_f8bf8_fnuz;
         else if(tciA == HIP_R_8F_E5M2_FNUZ && tciB == HIP_R_8F_E4M3_FNUZ)
             return rocblaslt_compute_f32_fast_bf8f8_fnuz;
-#ifdef ROCM_USE_FLOAT8
         else if(tciA == tciB && tciA == HIP_R_8F_E4M3)
-            return rocblaslt_compute_f32_fast_f8_ocp;
+            return rocblaslt_compute_f32_fast_f8;
         else if(tciA == tciB && tciA == HIP_R_8F_E5M2)
-            return rocblaslt_compute_f32_fast_bf8_ocp;
+            return rocblaslt_compute_f32_fast_bf8;
         else if(tciA == HIP_R_8F_E4M3 && tciB == HIP_R_8F_E5M2)
-            return rocblaslt_compute_f32_fast_f8bf8_ocp;
+            return rocblaslt_compute_f32_fast_f8bf8;
         else if(tciA == HIP_R_8F_E5M2 && tciB == HIP_R_8F_E4M3)
-            return rocblaslt_compute_f32_fast_bf8f8_ocp;
-#endif
+            return rocblaslt_compute_f32_fast_bf8f8;
     }
     return matmulDesc->compute_type_original;
 }
@@ -929,11 +1035,11 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
-            case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT:
-                matmulDesc->isScaleAVec = true;
             case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER:
-                if(matmulAttr == ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER)
-                    matmulDesc->isScaleAVec = false;
+                if(matmulDesc->scaleAType == RocblasltContractionProblem::ScalingFormat::None)
+                {
+                    matmulDesc->scaleAType = RocblasltContractionProblem::ScalingFormat::Scalar;
+                }
                 if(sizeof(void*) <= sizeInBytes)
                     memcpy(&matmulDesc->scaleA, buf, sizeof(void*));
                 else
@@ -942,11 +1048,47 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
-            case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT:
-                matmulDesc->isScaleBVec = true;
+            case ROCBLASLT_MATMUL_DESC_A_SCALE_MODE:
+                if(sizeof(hipblasLtMatmulMatrixScale_t) <= sizeInBytes)
+                {
+                    hipblasLtMatmulMatrixScale_t mode;
+                    memcpy(&mode, buf, sizeof(hipblasLtMatmulMatrixScale_t));
+                    switch(mode)
+                    {
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC32_UE8M0:
+                        matmulDesc->scaleABlockRowSize = 32;
+                        matmulDesc->scaleABlockColSize = 1;
+                        matmulDesc->scaleAType = RocblasltContractionProblem::ScalingFormat::Block;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F:
+                        matmulDesc->scaleABlockRowSize = 1;
+                        matmulDesc->scaleABlockColSize = 1;
+                        matmulDesc->scaleAType = RocblasltContractionProblem::ScalingFormat::Scalar;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_OUTER_VEC_32F:
+                        matmulDesc->scaleABlockRowSize = 1;
+                        matmulDesc->scaleABlockColSize = 1;
+                        matmulDesc->scaleAType = RocblasltContractionProblem::ScalingFormat::Vector;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC16_UE4M3:
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC128_32F:
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_BLK128x128_32F:
+                    default:
+                        log_error(__func__, "invalid A scale mode: ", mode);
+                        return rocblaslt_status_invalid_value;
+                    }
+                }
+                else
+                {
+                    log_error(__func__, "invalid A scale mode buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
             case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER:
-                if(matmulAttr == ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER)
-                    matmulDesc->isScaleBVec = false;
+                if(matmulDesc->scaleBType == RocblasltContractionProblem::ScalingFormat::None)
+                {
+                    matmulDesc->scaleBType = RocblasltContractionProblem::ScalingFormat::Scalar;
+                }
                 if(sizeof(void*) <= sizeInBytes)
                     memcpy(&matmulDesc->scaleB, buf, sizeof(void*));
                 else
@@ -955,6 +1097,42 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
+            case ROCBLASLT_MATMUL_DESC_B_SCALE_MODE:
+                if(sizeof(hipblasLtMatmulMatrixScale_t) <= sizeInBytes)
+                {
+                    hipblasLtMatmulMatrixScale_t mode;
+                    memcpy(&mode, buf, sizeof(hipblasLtMatmulMatrixScale_t));
+                    switch(mode)
+                    {
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC32_UE8M0:
+                        matmulDesc->scaleBBlockRowSize = 1;
+                        matmulDesc->scaleBBlockColSize = 32;
+                        matmulDesc->scaleBType = RocblasltContractionProblem::ScalingFormat::Block;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F:
+                        matmulDesc->scaleBBlockRowSize = 1;
+                        matmulDesc->scaleBBlockColSize = 1;
+                        matmulDesc->scaleBType = RocblasltContractionProblem::ScalingFormat::Scalar;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_OUTER_VEC_32F:
+                        matmulDesc->scaleBBlockRowSize = 1;
+                        matmulDesc->scaleBBlockColSize = 1;
+                        matmulDesc->scaleBType = RocblasltContractionProblem::ScalingFormat::Vector;
+                        break;
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC16_UE4M3:
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_VEC128_32F:
+                    case HIPBLASLT_MATMUL_MATRIX_SCALE_BLK128x128_32F:
+                    default:
+                        log_error(__func__, "invalid B scale mode: ", mode);
+                        return rocblaslt_status_invalid_value;
+                    }
+                }
+                else
+                {
+                    log_error(__func__, "invalid B scale mode buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
             case ROCBLASLT_MATMUL_DESC_C_SCALE_POINTER:
                 if(sizeof(void*) <= sizeInBytes)
                     memcpy(&matmulDesc->scaleC, buf, sizeof(void*));
@@ -1036,6 +1214,15 @@ rocblaslt_status rocblaslt_matmul_desc_set_attribute(rocblaslt_matmul_desc
                     return rocblaslt_status_invalid_value;
                 }
                 break;
+            case ROCBLASLT_MATMUL_DESC_EPILOGUE_AUX_DATA_TYPE:
+                if(sizeof(int32_t) <= sizeInBytes)
+                    memcpy(&matmulDesc->aux_type, buf, sizeof(int32_t));
+                else
+                {
+                    log_error(__func__, "invalid buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                break;
             case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT:
                 if(sizeof(int32_t) <= sizeInBytes)
                 {
@@ -1158,7 +1345,6 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 memcpy(buf, &matmulDesc->bias, sizeof(void*));
                 break;
             case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER:
-            case ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT:
                 if(sizeWritten)
                     *sizeWritten = sizeof(void*);
                 if(sizeInBytes < sizeof(void*))
@@ -1168,8 +1354,42 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 }
                 memcpy(buf, &matmulDesc->scaleA, sizeof(void*));
                 break;
+            case ROCBLASLT_MATMUL_DESC_A_SCALE_MODE: //TODO: May need to handle default value too.
+                if(sizeWritten)
+                    *sizeWritten = sizeof(uint32_t);
+                if(sizeInBytes < sizeof(uint32_t))
+                {
+                    log_error(__func__, "invalid scale block A scale mode size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                else
+                {
+                    hipblasLtMatmulMatrixScale_t mode = HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F;
+                    if(matmulDesc->scaleABlockRowSize == 32 && matmulDesc->scaleABlockColSize == 1
+                       && matmulDesc->scaleAType
+                              == RocblasltContractionProblem::ScalingFormat::Block)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_VEC32_UE8M0;
+                    }
+                    else if(matmulDesc->scaleAType
+                            == RocblasltContractionProblem::ScalingFormat::Scalar)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F;
+                    }
+                    else if(matmulDesc->scaleAType
+                            == RocblasltContractionProblem::ScalingFormat::Vector)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_OUTER_VEC_32F;
+                    }
+                    else
+                    {
+                        log_error(__func__, "invalid A scale mode", mode);
+                        return rocblaslt_status_invalid_value;
+                    }
+                    memcpy(buf, &mode, sizeof(uint32_t));
+                }
+                break;
             case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER:
-            case ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT:
                 if(sizeWritten)
                     *sizeWritten = sizeof(void*);
                 if(sizeInBytes < sizeof(void*))
@@ -1179,6 +1399,41 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 }
                 memcpy(buf, &matmulDesc->scaleB, sizeof(void*));
                 break;
+            case ROCBLASLT_MATMUL_DESC_B_SCALE_MODE: //TODO: May need to handle default value too.
+                if(sizeWritten)
+                    *sizeWritten = sizeof(uint32_t);
+                if(sizeInBytes < sizeof(uint32_t))
+                {
+                    log_error(__func__, "invalid scale block B scale mode size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                else
+                {
+                    hipblasLtMatmulMatrixScale_t mode = HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F;
+                    if(matmulDesc->scaleBBlockRowSize == 1 && matmulDesc->scaleBBlockColSize == 32
+                       && matmulDesc->scaleBType
+                              == RocblasltContractionProblem::ScalingFormat::Block)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_VEC32_UE8M0;
+                    }
+                    else if(matmulDesc->scaleBType
+                            == RocblasltContractionProblem::ScalingFormat::Scalar)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_SCALAR_32F;
+                    }
+                    else if(matmulDesc->scaleBType
+                            == RocblasltContractionProblem::ScalingFormat::Vector)
+                    {
+                        mode = HIPBLASLT_MATMUL_MATRIX_SCALE_OUTER_VEC_32F;
+                    }
+                    else
+                    {
+                        log_error(__func__, "invalid B scale mode", mode);
+                        return rocblaslt_status_invalid_value;
+                    }
+                    memcpy(buf, &mode, sizeof(uint32_t));
+                }
+                break;
             case ROCBLASLT_MATMUL_DESC_POINTER_MODE:
                 if(sizeWritten)
                     *sizeWritten = sizeof(int32_t);
@@ -1209,6 +1464,16 @@ rocblaslt_status rocblaslt_matmul_desc_get_attribute(rocblaslt_matmul_desc
                 }
                 memcpy(buf, &matmulDesc->amaxD, sizeof(void*));
                 break;
+            case ROCBLASLT_MATMUL_DESC_EPILOGUE_AUX_DATA_TYPE:
+                if(sizeWritten)
+                    *sizeWritten = sizeof(int32_t);
+                if(sizeInBytes < sizeof(int32_t))
+                {
+                    log_error(__func__, "invalid buf size", sizeInBytes);
+                    return rocblaslt_status_invalid_value;
+                }
+                memcpy(buf, &matmulDesc->aux_type, sizeof(int32_t));
+                break;
             case ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT:
                 if(sizeWritten)
                     *sizeWritten = sizeof(int32_t);
@@ -1538,12 +1803,16 @@ rocblaslt_status
         bool               override_success = false;
         if(override.env_mode)
         {
-            override_success = problem_override_from_file(
-                handle, pref, prob, matmul_desc, heuristicResultsArray, override.file_path);
+            override_success = problem_override_from_file(handle,
+                                                          prob,
+                                                          matmul_desc,
+                                                          heuristicResultsArray,
+                                                          override.file_path,
+                                                          pref->max_workspace_bytes);
             if(override_success)
                 requestedAlgoCount--;
 
-            log_api(__func__, "returnAlgoCount", override_success ? 1 : 0);
+            log_api(__func__, "OverrideAlgoCount", override_success ? 1 : 0);
         }
 
         if(requestedAlgoCount > 0)
@@ -1749,16 +2018,6 @@ rocblaslt_status rocblaslt_matmul_get_algos_from_index_cpp(
     return rocblaslt_status_success;
 }
 
-rocblaslt_status rocblaslt_is_algo_supported_cpp(rocblaslt_handle            handle,
-                                                 rocblaslt::RocGemmType      gemmType,
-                                                 std::shared_ptr<void>       gemmData,
-                                                 rocblaslt_matmul_algo&      algo,
-                                                 const rocblaslt::RocTuning* tuning,
-                                                 size_t&                     workspaceSizeInBytes)
-{
-    return isSolutionSupported(handle, gemmType, gemmData, algo, tuning, workspaceSizeInBytes);
-}
-
 rocblaslt_status rocblaslt_is_algo_supported_cpp(rocblaslt_handle              handle,
                                                  rocblaslt::RocGemmType        gemmType,
                                                  std::shared_ptr<void>         gemmData,
@@ -1773,7 +2032,7 @@ rocblaslt_status
     rocblaslt_algo_get_heuristic_cpp(rocblaslt_handle       handle,
                                      rocblaslt::RocGemmType gemmType,
                                      std::shared_ptr<void>  gemmData,
-                                     const int              workspaceBytes,
+                                     const size_t           maxWorkspaceBytes,
                                      const int              requestedAlgoCount,
                                      std::vector<rocblaslt_matmul_heuristic_result>& results)
 {
@@ -1798,9 +2057,9 @@ rocblaslt_status
         if(override.env_mode)
         {
             override_success = problem_override_from_file_cpp(
-                handle, gemmType, gemmData, workspaceBytes, override_result, override.file_path);
+                handle, gemmType, gemmData, override_result, override.file_path, maxWorkspaceBytes);
 
-            log_api(__func__, "returnAlgoCount", override_success ? 1 : 0);
+            log_api(__func__, "OverrideAlgoCount", override_success ? 1 : 0);
         }
 
         if(requestedAlgoCount - override_result.size() > 0)
@@ -1808,7 +2067,7 @@ rocblaslt_status
                 = getBestSolutions(handle,
                                    gemmType,
                                    gemmData,
-                                   workspaceBytes,
+                                   maxWorkspaceBytes,
                                    override_success ? requestedAlgoCount - 1 : requestedAlgoCount,
                                    results);
 
@@ -1827,10 +2086,10 @@ rocblaslt_status
         if(requestedAlgoCount > results.size())
         {
             std::vector<rocblaslt_matmul_heuristic_result> allSolutionsResults;
-            size_t                                         workspaceSizeInBytes = workspaceBytes;
+            size_t                                         workspaceSizeInBytes = 0;
             if(rocblaslt_status_success
                == getAllSolutions(
-                   gemmData, handle, gemmType, allSolutionsResults, workspaceSizeInBytes))
+                   gemmData, handle, gemmType, allSolutionsResults, maxWorkspaceBytes))
             {
                 int oriReturnAlgoCount = results.size();
                 for(int i = 0;
@@ -1911,34 +2170,111 @@ std::string rocblaslt_internal_get_arch_name()
 
 bool rocblaslt_internal_test_path(const std::string& path)
 {
-#ifdef WIN32
+#ifdef _WIN32
     return ((_access(path.c_str(), 4) != -1) || (_access(path.c_str(), 6) != -1));
 #else
     return access(path.c_str(), R_OK) == 0;
 #endif
 }
 
-#ifndef WIN32
-int hipblaslt_dl_iterate_phdr_callback(struct dl_phdr_info* hdr_info, size_t size, void* data)
+#ifdef _WIN32
+std::string rocblaslt_internal_get_so_path()
 {
-    // uncomment to see all dependent .so files
-    // fprintf(stderr, "hipblaslt so file: %s\n", hdr_info->dlpi_name);
-    std::pair<std::string, std::string>* typedData
-        = reinterpret_cast<std::pair<std::string, std::string>*>(data);
-    if(hdr_info->dlpi_name && strstr(hdr_info->dlpi_name, typedData->second.c_str()))
+    HMODULE hModule = NULL;
+    if(!GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
+                          // Should be the address of code in this library.
+                          (LPCTSTR)rocblaslt_internal_get_so_path,
+                          &hModule))
     {
-        typedData->first.assign(hdr_info->dlpi_name);
-        return 1;
+        throw std::runtime_error("Cannot get module for function");
     }
-    return 0;
+
+    std::string path;
+    path.resize(256);
+    for(;;)
+    {
+        auto stored_size = GetModuleFileNameA(hModule, path.data(), path.size());
+        if(stored_size < path.size())
+        {
+            // Success: size to what was stored (which does not include NUL).
+            path.resize(stored_size);
+            return path;
+        }
+        // Insufficient size.
+        path.resize(path.size() * 2);
+    }
+}
+#else
+std::string rocblaslt_internal_get_so_path()
+{
+    Dl_info info;
+    if(dladdr(reinterpret_cast<void*>(&rocblaslt_internal_get_so_path), &info) == 0)
+    {
+        throw std::runtime_error("Cannot get address of module function");
+    }
+    if(!info.dli_fname)
+    {
+        throw std::runtime_error("Containing binary does not have a file system path");
+    }
+    return std::string(info.dli_fname);
 }
 #endif
 
-std::string rocblaslt_internal_get_so_path(const std::string& keyword)
+std::optional<std::filesystem::path> rocblaslt_find_library_relative_path(
+    const std::optional<std::filesystem::path>& relpath,
+    const std::optional<std::filesystem::path>& default_lib_dir)
 {
-    std::pair<std::string, std::string> result{"", keyword};
-    dl_iterate_phdr(hipblaslt_dl_iterate_phdr_callback, &result);
-    return result.first;
+    auto pathIfExists
+        = [&](const std::filesystem::path& p) -> std::optional<std::filesystem::path> {
+        if(relpath)
+        {
+            auto full_path = p / (*relpath);
+            if(std::filesystem::exists(full_path))
+                return full_path;
+        }
+
+        if(std::filesystem::exists(p))
+            return p;
+        return {};
+    };
+
+    auto probeLibDir
+        = [&](const std::filesystem::path& lib_dir) -> std::optional<std::filesystem::path> {
+        // There are a few fallback locations that have grown over time:
+        //   {lib_dir}/hipblaslt/library
+        // Legacy:
+        //   {lib_dir}/../Tensile/library
+        //   {lib_dir}/library
+        if(auto p = pathIfExists(lib_dir / "hipblaslt" / "library"))
+            return *p;
+        if(auto p = pathIfExists(lib_dir.parent_path() / "Tensile" / "library"))
+            return *p;
+        if(auto p = pathIfExists(lib_dir / "library"))
+            return *p;
+        return std::nullopt;
+    };
+
+    if(default_lib_dir)
+    {
+        return probeLibDir(*default_lib_dir);
+    }
+
+    auto so_path       = std::filesystem::path(rocblaslt_internal_get_so_path()).parent_path();
+    bool windows_style = false;
+#ifdef _WIN32
+    windows_style = true;
+#endif
+
+    // If on Windows, probe the sibling lib directory first, as that is non-deprecated.
+    // Then fall back to the same-directory (bin) path.
+    if(windows_style)
+    {
+        auto sibling = probeLibDir(so_path.parent_path() / "lib");
+        if(sibling)
+            return sibling;
+    }
+
+    return probeLibDir(so_path);
 }
 
 void rocblaslt_log_error(const char* func, const char* var, const char* msg)
