@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,16 +24,25 @@ import ctypes
 import math
 import struct
 from collections import OrderedDict
-from enum import IntFlag
+from copy import deepcopy
+from enum import Enum, IntFlag
+from typing import List, Union
 
-from .TensileInstructions import Module, TextBlock, HolderContainer, RegisterContainer, \
-                          VCC, EXEC, vgpr, sgpr, Holder, fastdeepcopy, DataType, SNop, \
-                          TensileInstructions
-from .TensileInstructions.Enums import *
-from .TensileInstructions.Instructions import *
-from .Common import printExit, printWarning, globalParameters
+from rocisa import rocIsa
+from rocisa.code import Module, TextBlock
+from rocisa.container import VCC, EXEC, vgpr, sgpr, HolderContainer, RegisterContainer, Holder
+from rocisa.enum import InstType
+from rocisa.instruction import Instruction, SMovB32, SNop, SSetMask, VAddF16, VAddF32, \
+    VAddPKF16, VAndB32, VCmpGEF16, VCmpGEF32, VCmpGEF64, VCmpGEI32, VCmpGTF16, VCmpGTF32, \
+    VCmpGTF64, VCmpGTI32, VCmpXClassF32, VCmpXLtF32, VCndMaskB32, VExpF16, VExpF32, VFmaF16, \
+    VFmaF32, VFmaPKF16, VMaxF32, VMaxF64, VMaxI32, VMaxPKF16, VMed3I32, VMinF16, VMinF32, \
+    VMinF64, VMinI32, VMovB32, VMulF16, VMulF32, VMulF64, VMulLOU32, VMulPKF16, VRcpF16, \
+    VRcpF32, VSubF32, VSubI32
 
-from dataclasses import dataclass, field
+from Tensile.Common.DataType import DataType
+from Tensile.Common.Utilities import printExit, printWarning
+
+from dataclasses import dataclass
 
 ################################################################################
 # How to add an activation
@@ -169,6 +178,9 @@ class ActivationType:
                             'supported_by': SupportedBy.TENSILE}), \
                           ('silu', { \
                             'instance': ActivationTypeRegister('silu', False, 0,        True,  True, False,   False, False, False, False), \
+                            'supported_by': SupportedBy.TENSILE | SupportedBy.HIPBLASLT}), \
+                          ('swish', { \
+                            'instance': ActivationTypeRegister('swish', False, 1,        True,  True, False,   False, False, False, False), \
                             'supported_by': SupportedBy.TENSILE}), \
                           ('hipblaslt_all', { \
                             'instance': ActivationTypeRegister('hipblaslt_all', False, 0), \
@@ -324,6 +336,9 @@ class ActivationModule:
         self.enableGuard = False
         self.isAlt       = False
 
+    def __reduce__(self):
+        return (ActivationModule, ())
+
     # Public function
     def getModule(self, cDataType, activationType, vgprIn, vgprOut):
         if self.useCache:
@@ -355,6 +370,8 @@ class ActivationModule:
             module = self.getDGeluModule(cDataType, vgprIn, vgprOut)
         elif (activationType == 'silu'):
             module = self.getSiluModule(cDataType, vgprIn, vgprOut)
+        elif (activationType == 'swish'):
+            module = self.getSwishModule(cDataType, vgprIn, vgprOut, "activationAlpha")
         elif (activationType == 'none'):
             return Module("No activation")
         else:
@@ -506,7 +523,7 @@ class ActivationModule:
         return module
 
     def getExpModule(self, cDataType, vgprIn, vgprOut):
-        ti = TensileInstructions()
+        ti = rocIsa.getInstance()
         module = Module("Exp")
         if cDataType.isHalf():
             sgprMagic = self.getSgpr(1)
@@ -655,7 +672,7 @@ class ActivationModule:
         return module
 
     def getSigmoidModule(self, cDataType, vgprIn, vgprOut):
-        ti = TensileInstructions()
+        ti = rocIsa.getInstance()
         self.needCombine = True
         module = Module("Sigmoid")
         if cDataType.isHalf():
@@ -690,7 +707,7 @@ class ActivationModule:
         return module
 
     def getTanhModule(self, cDataType, vgprIn, vgprOut, activationAlpha, activationBeta):
-        ti = TensileInstructions()
+        ti = rocIsa.getInstance()
         self.needCombine = True
         module = Module("Tanh")
         if cDataType.isHalf():
@@ -750,7 +767,7 @@ class ActivationModule:
         return module
 
     def getDGeluModule(self, cDataType, vgprIn, vgprOut):
-        ti = TensileInstructions()
+        ti = rocIsa.getInstance()
         self.needCombine = True
         module = Module("Gradient Gelu")
         # x1 = (0.0535161 * pow(x, 3) + 0.398942 * x)
@@ -831,6 +848,25 @@ class ActivationModule:
         module.add(mulFunction(dst=self.vgprPrefix(vgprOut), src0=self.vgprPrefix(vgprIn), src1=self.vgprPrefix(Holder(idx=vgprTemp)), comment="x / (1 + exp(-x))"))
         return module
 
+    def getSwishModule(self, cDataType, vgprIn, vgprOut, activationAlpha):
+        self.needCombine = True
+        module = Module("Swish")
+        if cDataType.isHalf():
+            if self.usePK:
+                mulFunction = VMulPKF16
+            else:
+                mulFunction = VMulF16
+        elif cDataType.isSingle():
+            mulFunction = VMulF32
+        else:
+            raise RuntimeError("Unsupported data type %s."%cDataType.toDevice("HIP"))
+        vgprTempIn = self.getVgpr(1)
+        vgprTempOut = self.getVgpr(1)
+        module.add(mulFunction(dst=self.vgprPrefix(Holder(idx=vgprTempIn)), src0=self.vgprPrefix(vgprIn), src1=sgpr(activationAlpha), comment="x * beta"))
+        module.addModuleAsFlatItems(self.getSigmoidModule(cDataType, Holder(idx=vgprTempIn), Holder(idx=vgprTempOut)))
+        module.add(mulFunction(dst=self.vgprPrefix(vgprOut), src0=self.vgprPrefix(vgprIn), src1=self.vgprPrefix(Holder(idx=vgprTempOut)), comment="x / (1 + exp(-x * beta))"))
+        return module
+
     ################################################################################
     ################################################################################
     ###
@@ -844,7 +880,7 @@ class ActivationModule:
         if activationType not in self.cacheDict:
             self.cacheDict[activationType] = {}
         actDict = self.cacheDict[activationType]
-        copied = fastdeepcopy(module)
+        copied = deepcopy(module)
         # Get reg name
         regName = self.vgprPrefixFormat.split("+")[0] if self.vgprPrefixFormat else ""
         vgprIdxList = createVgprIdxList(copied, [vgprIn, vgprOut], regName)
@@ -867,9 +903,9 @@ class ActivationModule:
                                       enableGuard=self.enableGuard, prefix=self.vgprPrefixFormat):
                         if self.vgprPrefixFormat:
                             for vgpr in actInfo.vgprIdxList[0]:
-                                vgpr.regName.offsets[0] = vgprIn
+                                vgpr.regName.setOffset(0, vgprIn)
                             for vgpr in actInfo.vgprIdxList[1]:
-                                vgpr.regName.offsets[0] = vgprOut
+                                vgpr.regName.setOffset(0, vgprOut)
                         else:
                             for vgpr in actInfo.vgprIdxList[0]:
                                 vgpr.regIdx = vgprIn
@@ -877,7 +913,7 @@ class ActivationModule:
                                 vgpr.regIdx = vgprOut
                         self.vgprCounter = actInfo.vgprCounter
                         self.sgprCounter = actInfo.sgprCounter
-                        return fastdeepcopy(actInfo.module)
+                        return deepcopy(actInfo.module)
         return None
 
 ################################################################################
@@ -922,7 +958,7 @@ def RemoveEmptyBlocks(module):
     for idx, item in enumerate(module.items()):
         if isinstance(item, Module):
             newItem = RemoveEmptyBlocks(item)
-            module.items()[idx] = newItem
+            module.setItem(idx, newItem)
     if len(module.items()) == 1 and isinstance(module.items()[0], Module):
         return module.items()[0]
     return module
@@ -961,7 +997,7 @@ def FuseInstruction(currentInst, moduleAndIndex, fuseDebug):
                     # used before the current instruction
                     if not FindAssignAndUse(oldInst, currentInst, outVgpr, outVgpr):
                         newInst = type(oldInst)(oldInst.dst, *oldInst.srcs, oldInst.sdwa)
-                        newInst.srcs[2] = addConst + newInst.srcs[2]
+                        newInst.setSrc(2, addConst + newInst.srcs[2])
                         newInst.comment += " ( + 1 (fused))"
                         replaceInst(currentInst, newInst, fuseDebug)
                         removeOldInst(oldInst, currentInst, newInst, fuseDebug)
@@ -1011,11 +1047,11 @@ def FuseInstruction(currentInst, moduleAndIndex, fuseDebug):
                                 newValue = param * mulConst
                                 formatting = " (fused %f)" if isinstance(param, float) else " (fused %d)"
                                 if newFuseInst:
-                                    newFuseInst.srcs[0] = newValue
-                                    newInst.srcs[paramIdx] = newFuseInst.dst
+                                    newFuseInst.setSrc(0, newValue)
+                                    newInst.setSrc(paramIdx, newFuseInst.dst)
                                     newFuseInst.comment += formatting%newValue
                                 else:
-                                    newInst.srcs[paramIdx] = newValue
+                                    newInst.setSrc(paramIdx, newValue)
                                 newInst.comment += formatting%newValue
                                 replaceInst(currentInst, newInst, fuseDebug)
                                 removeOldInst(oldInst, currentInst, newInst, fuseDebug)
@@ -1108,7 +1144,7 @@ def removeOldInst(removeInst, dstInst, fusedInst, debug):
             if debug:
                 tb = TextBlock("\n/* Fused to block %s + %s -> %s */\n"%(str(removeInst), str(dstInst), str(fusedInst)))
                 tb.name = __FUSE_MAGIC_NAME__
-                module.items()[idx] = tb
+                module.setItem(idx, tb)
             else:
                 targetIdx = idx
             break
@@ -1166,20 +1202,20 @@ def HexToStr(cDataType, isPack, *args):
 
 def ConvertCoeffToHex(module, cDataType, isPack):
     if (module.name == "Exp"):
-        param = module.items()[0].srcs[0]
-        module.items()[0].srcs[0] = getMagic(cDataType, param, isPack)
+        param = module.getItem(0).srcs[0]
+        module.getItem(0).setSrc(0, getMagic(cDataType, param, isPack))
         return module
     for itemIdx, item in enumerate(module.items()):
         if isinstance(item, Module):
             newItem = ConvertCoeffToHex(item, cDataType, isPack)
-            module.items()[itemIdx] = newItem
+            module.setItem(itemIdx, newItem)
     return module
 
 def HolderToGpr(module, idx, pf):
     for itemIdx, item in enumerate(module.items()):
         if isinstance(item, Module):
             newItem = HolderToGpr(item, idx, pf)
-            module.items()[itemIdx] = newItem
+            module.setItem(itemIdx, newItem)
         elif isinstance(item, SNop):
             pass
         elif isinstance(item, Instruction):
@@ -1190,7 +1226,7 @@ def HolderToGpr(module, idx, pf):
                 for itemIdx, param in enumerate(item.srcs):
                     if isinstance(param, HolderContainer) and param.regType == pf:
                         param.setRegNum(idx)
-                        item.srcs[itemIdx] = param.getCopiedRC()
+                        item.setSrc(itemIdx, param.getCopiedRC())
     return module
 
 def addSpace(alignStr, str):
@@ -1277,7 +1313,7 @@ class ActivationInline:
       kStr += (asm + " // geluscaling\n")
       module = activation.getGeluModule(self.dataType, 0, 0, 1)
       kStr += self.getActivationAsmStr(activation, module, (len(asm) * " "))
-      kStr += addSpace(asm, ": \"+v\"(value) : \"s\"(alpha)\n")        
+      kStr += addSpace(asm, ": \"+v\"(value) : \"s\"(alpha)\n")
       kStr += self.getRequiredRegStr(asm, activation.vgprCounter, activation.sgprCounter)
     elif (activationType == 'leakyrelu'):
       if (self.dataType.isSingle() or self.dataType.isHalf() or self.dataType.isDouble()):
@@ -1318,6 +1354,12 @@ class ActivationInline:
       kStr += self.getActivationAsmStr(activation, module, (len(asm) * " "))
       kStr += addSpace(asm, ": \"+v\"(value) : \n")
       kStr += self.getRequiredRegStr(asm, activation.vgprCounter, activation.sgprCounter)
+    elif (activationType == 'swish'):
+      kStr += (asm + " // Swish\n")
+      module = activation.getSwishModule(self.dataType, 0, 0, 1)
+      kStr += self.getActivationAsmStr(activation, module, (len(asm) * " "))
+      kStr += addSpace(asm, ": \"+v\"(value) : \"s\"(alpha)\n")
+      kStr += self.getRequiredRegStr(asm, activation.vgprCounter, activation.sgprCounter)
     else:
       if (activationType != 'none'):
         raise RuntimeError("Unrecognized type %s."%activationType)
@@ -1354,7 +1396,7 @@ def createVgprIdxList(module, vgprList: list, regName):
             for param in item.getParams():
                 if isinstance(param, RegisterContainer):
                     for index, vgprIdx in enumerate(vgprList):
-                        if param.regName and (param.regName.name == regName) and (param.regName.offsets[0] == vgprIdx):
+                        if param.regName and (param.regName.name == regName) and (param.regName.getOffset()[0] == vgprIdx):
                             vlist[index].append(param)
                         elif param.regIdx == vgprIdx:
                             vlist[index].append(param)
