@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,18 +22,21 @@
 #
 ################################################################################
 
+from typing import Dict
+
 from .Activation import ActivationType
-from .Common import internalParameters, printExit
-from .TensileInstructions import DataType
 from . import Hardware
 from . import Properties
-from .SolutionStructs import getBiasDataTypeListDefault
-from .SolutionStructs import Solution as OriginalSolution
-from .Utils import state, state_key_ordering
-
-from . import Common
-from . Common import globalParameters
+from Tensile.Common import state, state_key_ordering, IsaInfo
+from Tensile.Common.Architectures import gfxToIsa
+from Tensile.Common.DataType import DataType
+from Tensile.Common.GlobalParameters import internalParameters
+from Tensile.SolutionStructs import Solution as OriginalSolution
+from Tensile.SolutionStructs.Problem import getBiasDataTypeListDefault
+from Tensile.Toolchain.Component import Assembler
+from math import ceil
 
+MIN_K_FOR_GSU = 32
 @state_key_ordering
 class FreeIndex:
     StateKeys = ['isA', 'i', 'c', 'd']
@@ -141,9 +144,15 @@ class ProblemType:
         if rv.aType.isFloat8BFloat8() or rv.bType.isFloat8BFloat8():
             rv.aType = DataType("F8")
             rv.bType = DataType("B8")
+        elif rv.aType.isFloat8BFloat8_fnuz() or rv.bType.isFloat8BFloat8_fnuz():
+            rv.aType = DataType("F8N")
+            rv.bType = DataType("B8N")
         elif rv.aType.isBFloat8Float8() or rv.bType.isBFloat8Float8():
             rv.aType = DataType("B8")
             rv.bType = DataType("F8")
+        elif rv.aType.isBFloat8Float8_fnuz() or rv.bType.isBFloat8Float8_fnuz():
+            rv.aType = DataType("B8N")
+            rv.bType = DataType("F8N")
 
         if 'DataTypeE' in d:
             rv.eType = DataType(d['DataTypeE'])
@@ -369,6 +378,7 @@ class ProblemType:
             predicates.append(ProblemPredicate("UseGradient", value=self.useGradient))
             predicates.append(ProblemPredicate("UseBias", value=self.useBias))
             predicates.append(ProblemPredicate("UseE", value=self.useE))
+            predicates.append(ProblemPredicate("DataTypeE", value=self.eType))
             predicates.append(ProblemPredicate("StridedBatched", value=self.stridedBatched))
             predicates.append(ProblemPredicate("GroupedGemm", value=self.groupedGemm))
             predicates.append(ProblemPredicate("UseScaleAB", value=self.useScaleAB))
@@ -396,6 +406,19 @@ def extractDimPredicate(cls, key, value, predicateName):
     elif len(predicates) > 1:
         return cls.And(predicates)
 
+class TaskPredicate(Properties.Predicate):
+    @classmethod
+    def FromOriginalKeyPair(cls, pair):
+        (key, value) = pair
+        if key == "_WorkspaceSizePerElemC" and value > 0:
+            return cls("WorkspaceCheck")
+        return None
+
+    @classmethod
+    def FromOriginalState(cls, d, problemType, morePreds=[]):
+        predicates = [p for p in map(cls.FromOriginalKeyPair, d.items()) if p is not None]
+        return cls.And(predicates)
+
 class ProblemPredicate(Properties.Predicate):
     @classmethod
     def FromOriginalKeyPair(cls, pair):
@@ -424,9 +447,6 @@ class ProblemPredicate(Properties.Predicate):
 
             return cls(tag, index=index, value=value)
 
-        if key == "WorkspaceCheck" and (not all(val == 0 for val in value)):
-            return cls("WorkspaceCheck", index=0, value=value)
-
         if key.startswith('Assert'):
             raise RuntimeError("Unknown assertion key: {}".format(key))
 
@@ -441,12 +461,15 @@ class ProblemPredicate(Properties.Predicate):
             valuepredicates = []
             valuepredicates.append(state["MacroTile0"])
             valuepredicates.append(state["MacroTile1"])
-            valuepredicates.append(state["MIWaveTile"][0]*state["MIWaveTile"][1])
+            if state["EnableMatrixInstruction"]:
+                valuepredicates.append(state["MIWaveTile"][0]*state["MIWaveTile"][1])
+            else:
+                valuepredicates.append(state["ThreadTile0"]*state["ThreadTile1"])
             if state["NumElementsPerBatchStore"] != 0:
                 valuepredicates.append(int((state["NumElementsPerThread"])/state["NumElementsPerBatchStore"]))
             else:
                 valuepredicates.append(1)
-            valuepredicates.append(state["NumThreads"])
+            valuepredicates.append(ceil(state["NumThreads"] / state["WavefrontSize"]))
             rv += [cls('SynchronizerSizeCheck', index=0, value=valuepredicates)]
 
         if state["InternalSupportParams"]["KernArgsVersion"] >= 1 and \
@@ -472,7 +495,7 @@ class ProblemPredicate(Properties.Predicate):
         if "KernelLanguage" in state:
             rv += [cls("KernelLanguageCompatible", value=state["KernelLanguage"])]
 
-        if ('GlobalSplitU' in state) and (state['GlobalSplitU'] > 1):
+        if ('GlobalSplitU' in state) and (state['GlobalSplitU'] > 1 or state['GlobalSplitU'] == -1):
             if ('_GlobalAccumulation' not in state) or (state['_GlobalAccumulation'] != 'MultipleBuffer'):
                 rv += [cls("DeterministicMode", value = False)]
 
@@ -509,7 +532,7 @@ class ProblemPredicate(Properties.Predicate):
             rv += [cls('BufferStoreOffsetLimitCheck', value=state['MacroTile1'])]
 
         if '_GlobalAccumulation' in state and state['_GlobalAccumulation'] != None and not state["StreamK"]:
-            value = globalParameters['MinKForGSU']
+            value = MIN_K_FOR_GSU
             rv += [cls('GlobalSplitUCheckMinK', value=[value, state["GlobalSplitU"]])]
 
         if ('WorkGroupMappingXCC' in state) and ('WorkGroupMappingXCCGroup' in state):
@@ -517,15 +540,9 @@ class ProblemPredicate(Properties.Predicate):
 
         if state['ProblemType']['SwizzleTensorA']:
             rv += [cls('SwizzleTensorA', value=state['ProblemType']['SwizzleTensorA'])]
-            # TODO- (TT + DTVA) tail-loop is not working yet.
-            if state['ProblemType']['TransposeB']:
-                rv += [cls("BoundSizeMultiple", index=-1, value=state['DepthU'])]
 
-        # TODO- Will remove the size predicate once we have SWZ-B request
         if state['ProblemType']['SwizzleTensorB']:
             rv += [cls('SwizzleTensorB', value=state['ProblemType']['SwizzleTensorB'])]
-            rv += [cls("Free1SizeMultiple", index=0, value=state['MacroTile1'])]
-            rv += [cls("BoundSizeMultiple", index=-1, value=state['DepthU'])]
 
         return rv
 
@@ -568,7 +585,11 @@ class SizeMapping:
                  'workGroupMappingXCC',
                  'workGroupMappingXCCGroup',
                  'globalSplitUCoalesced',
-                 'globalSplitUWorkGroupMappingRoundRobin'
+                 'globalSplitUWorkGroupMappingRoundRobin',
+                 'CUOccupancy',
+                 'PrefetchGlobalRead',
+                 'MathClocksUnrolledLoop',
+                 'synchronizerSizePerWG'
                  ]
 
     @classmethod
@@ -582,6 +603,11 @@ class SizeMapping:
             globalAccum = 3
         if d['_GlobalAccumulation'] == 'PartialsBuffer':
             globalAccum = 4
+        pgr = int(d['PrefetchGlobalRead'])
+        synchronizerSizePerWG = ceil((d['MIWaveTile'][0]*d['MIWaveTile'][1] if d['EnableMatrixInstruction'] else d['ThreadTile0']*d['ThreadTile1']        \
+                                    * ceil((d['NumElementsPerThread'])/d['NumElementsPerBatchStore']) if d['NumElementsPerBatchStore'] != 0 else 1        \
+                                    * ceil(d["NumThreads"] / d["WavefrontSize"])))
+
         return cls(waveNum                  = d['NumThreads'] // d['WavefrontSize'],
                    workGroup                = d['WorkGroup'],
                    macroTile                = cls.ReadOriginalMacroTile(d),
@@ -611,9 +637,12 @@ class SizeMapping:
                    workGroupMappingXCC      = d['WorkGroupMappingXCC'],
                    workGroupMappingXCCGroup = d['WorkGroupMappingXCCGroup'],
                    globalSplitUCoalesced    = d['GlobalSplitUCoalesced'],
-                   globalSplitUWorkGroupMappingRoundRobin = d['GlobalSplitUWorkGroupMappingRoundRobin']
+                   globalSplitUWorkGroupMappingRoundRobin = d['GlobalSplitUWorkGroupMappingRoundRobin'],
+                   CUOccupancy              = d['CUOccupancy'],
+                   PrefetchGlobalRead       = pgr,
+                   MathClocksUnrolledLoop   = d['MathClocksUnrolledLoop'],
+                   synchronizerSizePerWG    = synchronizerSizePerWG
                    )
-
     @classmethod
     def ReadOriginalMacroTile(cls, d):
         rv = [1,1,1]
@@ -650,6 +679,7 @@ class Solution:
                 'problemType',
                 'hardwarePredicate',
                 'problemPredicate',
+                'taskPredicate',
                 'sizeMapping',
                 'internalArgsSupport',
                 'debugKernel',
@@ -660,14 +690,40 @@ class Solution:
     HiddenKeys = ['originalSolution']
 
     @classmethod
-    def FromSolutionStruct(cls, solution, cxxCompiler: str):
-        return cls.FromOriginalState(solution._state, cxxCompiler)
+    def FromSolutionStruct(
+        cls,
+        solution,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        assembler: Assembler,
+        isaInfoMap: Dict[str, IsaInfo]
+    ):
+        return cls.FromOriginalState(
+                   solution._state,
+                   splitGSU,
+                   printSolutionRejectionReason,
+                   printIndexAssignmentInfo,
+                   assembler,
+                   isaInfoMap,
+                   solution.srcName
+               )
 
     @classmethod
-    def FromOriginalState(cls, d, cxxCompiler, deviceInfo=None):
+    def FromOriginalState(
+            cls,
+            d,
+            splitGSU: bool,
+            printSolutionRejectionReason: bool,
+            printIndexAssignmentInfo: bool,
+            #mink
+            assembler,
+            isaInfoMap,
+            srcName = "",
+            deviceInfo=None
+        ):
         rv = cls()
 
-
         if 'SolutionNameMin' in d:
             rv.name = d['SolutionNameMin']
 
@@ -677,6 +733,7 @@ class Solution:
         rv.problemType = ProblemType.FromOriginalState(d['ProblemType'])
 
         rv.problemPredicate = ProblemPredicate.FromOriginalState(d, rv.problemType)
+        rv.taskPredicate = TaskPredicate.FromOriginalState(d, rv.problemType)
 
         if 'DebugKernel' in d:
             rv.debugKernel = d['DebugKernel']
@@ -703,15 +760,22 @@ class Solution:
 
         if 'ISA' not in d:
             if d['KernelLanguage'] == 'Assembly':
-                d['ISA'] = Common.gfxArch(deviceInfo[1])
-            else:
-                d['ISA'] = [0,0,0]
+                d['ISA'] = gfxToIsa(deviceInfo[1])
 
         if 'CUCount' not in d:
             d['CUCount'] = None
 
         rv.hardwarePredicate = Hardware.HardwarePredicate.FromHardware(d['ISA'], d['CUCount'])
-        rv.originalSolution = OriginalSolution(d, cxxCompiler)
+        rv.originalSolution = OriginalSolution(
+                                  d,
+                                  splitGSU,
+                                  printSolutionRejectionReason,
+                                  printIndexAssignmentInfo,
+                                  assembler,
+                                  isaInfoMap,
+                                  srcName
+                              )
+        rv.srcName = srcName
 
         return rv
 
@@ -724,11 +788,13 @@ class Solution:
         self.problemType = None
         self.hardwarePredicate = Hardware.HardwarePredicate('TruePred')
         self.problemPredicate = ProblemPredicate('TruePred')
+        self.taskPredicate = TaskPredicate('TruePred')
         self.sizeMapping = None
         self.debugKernel = False
         self.libraryLogicIndex = {}
         self.index = None
         self.ideals = {}
+        self.srcName = ""
 
         for key, value in kwargs:
             if key not in Solution.StateKeys and key not in Solution.HiddenKeys:
