@@ -22,7 +22,6 @@
 
 from copy import deepcopy
 
-from .Common import globalParameters, CHeader
 from .Activation import ActivationType
 from .KernelWriterBase import KernelWriterBase
 
@@ -45,9 +44,17 @@ class KernelWriterActivationEnumHeader(KernelWriterBase):
   def keys(self):
     return self.getKernelName()
 
+  @staticmethod
+  def kernelName(solution):
+    state = solution._state if hasattr(solution, "_state") else solution.state
+    s = "Gradient" if state["ProblemType"]["Gradient"] else ""
+    return "Tensile%sActivationEnum_%s"%(s,
+                                         state["ProblemType"]["ActivationComputeDataType"].toChar())
+
   def getKernelName(self):
-    return "Tensile%sActivationEnum_%s"%(self.actGradientPrefix, \
-                                         self.state["ProblemType"]["ActivationComputeDataType"].toChar())
+    return KernelWriterActivationEnumHeader.kernelName(self)
+
+   
 
   def getSourceFileString(self):
     fileString = "// This is a dummy file."
@@ -55,10 +62,6 @@ class KernelWriterActivationEnumHeader(KernelWriterBase):
 
   def getHeaderFileString(self):
     fileString = "" # CHeader
-    if not globalParameters["MergeFiles"]:
-      fileString += CHeader
-      fileString += "#pragma once\n\n"
-
     activationCDataType = self.state["ProblemType"]["ActivationComputeDataType"]
     supportedBy = ActivationType.SupportedBy.ALL if self.state["ProblemType"]["ActivationType"] == 'all' else ActivationType.SupportedBy.HIPBLASLT
     enumName = "%sActivationType_%s"%(self.actGradientPrefix, activationCDataType.toChar())
