@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,15 +22,18 @@
 #
 ################################################################################
 
+from typing import Dict
 from copy import deepcopy
 
-from .Common import globalParameters, CHeader, gfxArch, getGfxName
 from .KernelWriterBase import KernelWriterBase
-from .TensileInstructions import DataType
+
+from Tensile.Common.Architectures import isaToGfx
+from Tensile.Common import INDEX_CHARS, IsaInfo
+from Tensile.Common.DataType import DataType
 
 class KernelWriterConversion(KernelWriterBase):
 
-  def __init__(self, state, load_vw):
+  def __init__(self, state, load_vw, isaInfoMap: Dict[str, IsaInfo]):
     super().__init__()
 
     self.state["ProblemType"] = deepcopy(state["ProblemType"])
@@ -49,9 +52,22 @@ class KernelWriterConversion(KernelWriterBase):
     # setup load vector width
     self.num_elements_load = load_vw
 
+    # Macro guards for f8 types
+    # For now, it is enough to check dest type to determine if we are using f8 types
+    # May need to include checks for input data type in the future.
+    self.f8MacroGuardStart = "";
+    self.f8MacroGuardEnd   = "";
+    if (self.state["ProblemType"]["DestDataType"].isFloat8() or self.state["ProblemType"]["DestDataType"].isBFloat8()):
+      self.f8MacroGuardStart = "\n#if HIP_FP8_TYPE_OCP\n"
+      self.f8MacroGuardEnd   = "\n#endif // F8 macro guard\n"
+    if (self.state["ProblemType"]["DestDataType"].isFloat8_fnuz() or self.state["ProblemType"]["DestDataType"].isBFloat8_fnuz()):
+      self.f8MacroGuardStart = "\n#if HIP_FP8_TYPE_FNUZ\n"
+      self.f8MacroGuardEnd   = "\n#endif // F8 macro guard\n"
+
     # derive parameter
     self.language = "HIP"
     self.kernelName = self.getKernelName()
+    self.isaInfoMap = isaInfoMap
     self.datatype = self.state["ProblemType"]["ComputeDataType"].toDevice(self.language)
     self.int32Str = DataType('int32').toDevice(self.language)
     if self.state["ProblemType"]["DataType"].isInt8() and self.state["ProblemType"]["ComputeDataType"].isSingle() and self.state["ProblemType"]["HighPrecisionAccumulate"]:
@@ -59,24 +75,13 @@ class KernelWriterConversion(KernelWriterBase):
 
     # determine chars for fast access
     self.indexChars = []
-    for i in range(0, len(globalParameters["IndexChars"])):
-      self.indexChars.append(globalParameters["IndexChars"][i])
+    for i in range(0, len(INDEX_CHARS)):
+      self.indexChars.append(INDEX_CHARS[i])
     self.indexChars[self.state["ProblemType"]["Index0"]] = "0" + self.indexChars[self.state["ProblemType"]["Index0"]]
     self.indexChars[self.state["ProblemType"]["Index1"]] = "1" + self.indexChars[self.state["ProblemType"]["Index1"]]
     self.tileChar0 = self.indexChars[self.state["ProblemType"]["Index0"]]
     self.tileChar1 = self.indexChars[self.state["ProblemType"]["Index1"]]
 
-    # Get supported archs
-    if ";" in globalParameters["Architecture"]:
-      self.supportedArchs = globalParameters["Architecture"].split(";")
-    else:
-      self.supportedArchs = globalParameters["Architecture"].split("_")
-    if "all" in self.supportedArchs:
-      self.supportedArchs = deepcopy(globalParameters['SupportedISA'])
-    else:
-      for idx, arch in enumerate(self.supportedArchs):
-        self.supportedArchs[idx] = gfxArch(''.join(map(str, arch)))
-
     self.gsuKernels = [self.state["GlobalSplitU"]]
     if self.state["GenPGRPostKernels"]:
       pgrgsu = int(self.state["GlobalSplitU"] / 2)
@@ -462,7 +467,7 @@ class KernelWriterConversion(KernelWriterBase):
       kStr += "  auto idxW_ori = idxW;%s"%self.endLine
 
     typeStr = "int" if self.state["ProblemType"]["DataType"].isInt8() or self.state["ProblemType"]["DataType"].isInt32() else ("double" if self.state["ProblemType"]["DataType"].isDouble() else "float")
-    typeStr2 = "int16_t" if self.state["ProblemType"]["DestDataType"].isInt8() else ("tensile_half" if self.state["ProblemType"]["DestDataType"].isFloat8() else "tensile_bfloat16")
+    typeStr2 = "int16_t" if self.state["ProblemType"]["DestDataType"].isInt8() else ("tensile_half" if self.state["ProblemType"]["DestDataType"].isAnyFloat8() else "tensile_bfloat16")
     loadTypeStr = "%s%s" % (typeStr, "" if self.num_dword_load == 1 else self.num_dword_load)
     storeTypeStr = "%s%s" % (typeStr, self.num_dword_store) if self.num_dword_store >= 1 else typeStr2 if self.num_dword_store == 0.5 else destTypeStr
 
@@ -519,15 +524,14 @@ class KernelWriterConversion(KernelWriterBase):
           if self.num_dword_load > 2:
             kStr += "  float2 accumVec2(accum[2], accum[3]);" + self.endLine
       canPKF32Arch = []
-      for arch in self.supportedArchs:
-        archTuple = tuple(arch)
-        if globalParameters["AsmCaps"][archTuple]['v_pk_add_f32']:
-          canPKF32Arch.append(arch)
+      for isa in self.isaInfoMap.keys():
+        if self.isaInfoMap[isa].asmCaps['v_pk_add_f32']:
+          canPKF32Arch.append(isa)
       defineStr = []
       if len(canPKF32Arch) > 0:
-        defineStr = "#if defined(__%s__)"%getGfxName(canPKF32Arch[0])
+        defineStr = "#if defined(__%s__)"%isaToGfx(canPKF32Arch[0])
         for arch in canPKF32Arch[1:]:
-          defineStr += "|| defined(__%s__)"%getGfxName(arch)
+          defineStr += "|| defined(__%s__)"%isaToGfx(arch)
       else:
         defineStr = "#if 0"
       # PGR=2
@@ -777,81 +781,72 @@ class KernelWriterConversion(KernelWriterBase):
     return kStr
 
 
-  def getKernelName(self):
-    indexChars = globalParameters["IndexChars"]
+  @staticmethod
+  def kernelName(solution, num_elements_load, btype=None):
+    state = solution._state if hasattr(solution, "_state") else solution.state
+    indexChars = INDEX_CHARS
     # C dimensions
     name = "C"
-    for i in range(0, self.state["ProblemType"]["NumIndicesC"]):
+    for i in range(0, state["ProblemType"]["NumIndicesC"]):
       name += indexChars[i].lower()
     name += "_"
 
     # add input datatype into kernel name (the datatype of workspace)
-    inputTypeStr = DataType("I").toChar() if self.state["ProblemType"]["DataType"].isInt8() or self.state["ProblemType"]["DataType"].isInt32() else \
-                                  (DataType("D").toChar() if self.state["ProblemType"]["DataType"].isDouble() else DataType("S").toChar())
+    inputTypeStr = DataType("I").toChar() if state["ProblemType"]["DataType"].isInt8() or state["ProblemType"]["DataType"].isInt32() else \
+                                  (DataType("D").toChar() if state["ProblemType"]["DataType"].isDouble() else DataType("S").toChar())
 
-    name += (inputTypeStr + self.state["ProblemType"]["DestDataType"].toChar())
+    name += (inputTypeStr + state["ProblemType"]["DestDataType"].toChar())
 
-    if self.state["ProblemType"]["GroupedGemm"]:
+    if state["ProblemType"]["GroupedGemm"]:
       name += "_GG"
     else:
-      name += "" if self.state["ProblemType"]["StridedBatched"] else "_GB"
-    if self.state["ProblemType"]["UseBias"]:
-      if self.state["ProblemType"]["Gradient"]:
-        name += "_DBias%s"%(self.state["ProblemType"]["BiasDataType"].toChar())
-        name += "_BiasSrc%s"%(self.state["ProblemType"]["BiasSrc"])
+      name += "" if state["ProblemType"]["StridedBatched"] else "_GB"
+    if btype:
+      if state["ProblemType"]["Gradient"]:
+        name += "_DBias%s"%(btype.toChar())
+        name += "_BiasSrc%s"%(state["ProblemType"]["BiasSrc"])
       else:
-        name += "_Bias%s"%self.state["ProblemType"]["BiasDataType"].toChar()
+        name += "_Bias%s"%btype.toChar()
 
-    factorDim =  0 if self.state["ProblemType"]["Gradient"] else self.state["ProblemType"]["UseBias"]
-    factorDim =  max(factorDim, self.state["ProblemType"]["UseScaleAlphaVec"])
+    factorDim =  0 if state["ProblemType"]["Gradient"] else state["ProblemType"]["UseBias"]
+    factorDim =  max(factorDim, state["ProblemType"]["UseScaleAlphaVec"])
     if factorDim > 1:
         name += "_FD%s"%("N" if factorDim == 2 else "MN")
 
-    if self.state["ProblemType"]["UseE"]:
-      if self.state["ProblemType"]["Gradient"]:
-        name += "_Grad%s"%self.state["ProblemType"]["DataTypeE"].toChar()
+    if state["ProblemType"]["UseE"]:
+      if state["ProblemType"]["Gradient"]:
+        name += "_Grad%s"%state["ProblemType"]["DataTypeE"].toChar()
       else:
-        name += "_Aux%s"%self.state["ProblemType"]["DataTypeE"].toChar()
+        name += "_Aux%s"%state["ProblemType"]["DataTypeE"].toChar()
 
-    if ((self.state["ProblemType"]["ActivationType"] != 'none') and self.state["ActivationFused"]):
-      if self.state["ProblemType"]["ActivationType"] == 'all':
+    if ((state["ProblemType"]["ActivationType"] != 'none') and state["ActivationFused"]):
+      if state["ProblemType"]["ActivationType"] == 'all':
         name += "_A"
-      elif self.state["ProblemType"]["ActivationType"] == 'hipblaslt_all':
+      elif state["ProblemType"]["ActivationType"] == 'hipblaslt_all':
         name += "_HA"
       else:
-        name += "_%s"%str(self.state["ProblemType"]["ActivationType"]).upper()
-      name += self.state["ProblemType"]["ActivationComputeDataType"].toChar()
-      name += ("ng" if self.state["ProblemType"]["ActivationNoGuard"] else "")
-    if self.state["ProblemType"]["UseScaleAB"] == "Scalar":
+        name += "_%s"%str(state["ProblemType"]["ActivationType"]).upper()
+      name += state["ProblemType"]["ActivationComputeDataType"].toChar()
+      name += ("ng" if state["ProblemType"]["ActivationNoGuard"] else "")
+    if state["ProblemType"]["UseScaleAB"] == "Scalar":
       name += "_ScaleAB"
-    elif self.state["ProblemType"]["UseScaleAB"] == "Vector":
+    elif state["ProblemType"]["UseScaleAB"] == "Vector":
       name += "_ScaleABVec"
-    name += "_ScaleCD" if self.state["ProblemType"]["UseScaleCD"] else ""
-    name += "_ScaleAlphaVec" if self.state["ProblemType"]["UseScaleAlphaVec"] else ""
-    name += "_PostGSU" + str(self.state["GlobalSplitU"])
-    if self.num_elements_load != None:
-      name += "_VW" + str(self.num_elements_load)
+    name += "_ScaleCD" if state["ProblemType"]["UseScaleCD"] else ""
+    name += "_ScaleAlphaVec" if state["ProblemType"]["UseScaleAlphaVec"] else ""
+    name += "_PostGSU" + str(state["GlobalSplitU"])
+    if num_elements_load != None:
+      name += "_VW" + str(num_elements_load)
     return name
 
 
+  def getKernelName(self):
+    btype = self.state["ProblemType"]["BiasDataType"] if self.state["ProblemType"]["UseBias"] else None
+    return KernelWriterConversion.kernelName(self, self.num_elements_load, btype)
+
+
   def getHeaderFileString(self):
     fileString = "" # CHeader
-    if not globalParameters["MergeFiles"]:
-      fileString += CHeader
-      fileString += "#pragma once\n\n"
-      fileString += "\n"
-      fileString += "#include <KernelHeader.h>\n\n"
-      fileString += "#include <hip/hip_runtime.h>\n"
-      fileString += "#include <hip/hip_fp16.h>\n"
-      fileString += "\n"
-      activationCDataType = self.state["ProblemType"]["ActivationComputeDataType"]
-      if self.state["ProblemType"]["ActivationType"] in ['all', 'hipblaslt_all']:
-        fileString += "#include \"Tensile%sActivation%s_%s_%s.h\"\n"%(self.actGradientPrefix, \
-                                                                      self.gaurdStr, \
-                                                                      activationCDataType.toChar(), \
-                                                                      self.state["ProblemType"]["ActivationType"])
-      fileString += "\n"
-
     backupGSU    = self.state["GlobalSplitU"]
     backupUnroll = self.state["UnrollOnly"]
     for gsu in self.gsuKernels:
@@ -859,9 +854,11 @@ class KernelWriterConversion(KernelWriterBase):
         self.state["GlobalSplitU"] = gsu
         self.state["ProblemType"]["GroupedGemm"] = toggle
         self.kernelName = self.getKernelName()
+        fileString += self.f8MacroGuardStart
         fileString += self.functionArgument()
         fileString += self.functionSignature()
         fileString += ";\n"
+        fileString += self.f8MacroGuardEnd
       if not self.state["UnrollOnly"]:
         self.state["UnrollOnly"] = True
     self.state["GlobalSplitU"] = backupGSU
@@ -873,11 +870,6 @@ class KernelWriterConversion(KernelWriterBase):
 
   def getSourceFileString(self):
     fileString = ""
-    if not globalParameters["MergeFiles"]:
-      fileString += "\n"
-      fileString += "#include \"%s.h\"\n" % self.kernelName
-      fileString += "\n"
-
     backupGSU    = self.state["GlobalSplitU"]
     backupUnroll = self.state["UnrollOnly"]
     for gsu in self.gsuKernels:
@@ -885,8 +877,10 @@ class KernelWriterConversion(KernelWriterBase):
         self.state["GlobalSplitU"] = gsu
         self.state["ProblemType"]["GroupedGemm"] = toggle
         self.kernelName = self.getKernelName()
+        fileString += self.f8MacroGuardStart
         fileString += self.functionSignature()
         fileString += self.kernelBody()
+        fileString += self.f8MacroGuardEnd
       if not self.state["UnrollOnly"]:
         self.state["UnrollOnly"] = True
     self.state["GlobalSplitU"] = backupGSU
@@ -953,7 +947,7 @@ class KernelWriterConversion(KernelWriterBase):
         kStr += defineStr + self.endLine
         kStr += space + "    \"v_pk_add_f32 %0, %2, %0 \\n\\t\"" + self.endLine
         kStr += space + "    \"v_pk_add_f32 %1, %3, %1 \\n\\t\"" + self.endLine
-        kStr += space + "    : \"+v\"(accumVec), \"+v\"(accumVec2): \"v\"(%stemp[%d].data.xy), \"v\"(%stemp[%d].data.zw)"% (castToIntermidate, gsuIdx, castToIntermidate, gsuIdx) + self.endLine
+        kStr += space + "    : \"+v\"(accumVec), \"+v\"(accumVec2): \"v\"(%smake_float2(temp[%d].x,temp[%d].y)), \"v\"(%smake_float2(temp[%d].z,temp[%d].w))"% (castToIntermidate, gsuIdx, gsuIdx, castToIntermidate, gsuIdx, gsuIdx) + self.endLine
         kStr += "#else" + self.endLine
         kStr += space + "    \"v_add_f32 %0, %4, %0 \\n\\t\"" + self.endLine
         kStr += space + "    \"v_add_f32 %1, %5, %1 \\n\\t\"" + self.endLine
