@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,19 +20,23 @@
 # CTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ################################################################################
 
-from .TensileInstructions import DataType, Label, Module, vgpr, sgpr, accvgpr, \
-                                 Holder, SBranchIfNotZero
-from .TensileInstructions.Instructions import *
+from rocisa.code import Label, Module
+from rocisa.container import vgpr, sgpr, accvgpr, Holder
+from rocisa.instruction import SBarrier, SBranch, SMovB32, SMovB64, SWaitCnt, \
+  VAccvgprReadB32, VAccvgprWriteB32, VFmaF32, VFmaF64, VLShiftLeftB64, VMovB32, \
+  VMulF32, VMulF64, VMulLOU32, VMulPKF16
+from rocisa.functions import BranchIfNotZero
+
+from Tensile.Common.DataType import DataType
 
 def allocPostLoopSrdSuppressRaw(ch: str, chAddress: str, labelStr: str, sgprLength) -> Module:
     module = Module("allocPostLoopSrdSuppress")
     label  = Label("%sAddrValid"%labelStr, "")
     label2 = Label("%sAddrValid_End"%labelStr, "")
     # Buffer-load uses one base read pointer stored in the SRD - set it here:
-    module.add(SMovB32(dst=sgpr("Srd%s+0"%ch), src=sgpr("Address%s+0"%chAddress), comment="init SRD base address (lower)" ))
-    module.add(SMovB32(dst=sgpr("Srd%s+1"%ch), src=sgpr("Address%s+1"%chAddress), comment="init SRD base address (upper) + other fields" ))
+    module.add(SMovB64(dst=sgpr("Srd%s+0"%ch, 2), src=sgpr("Address%s+0"%chAddress, 2), comment="init SRD base address" ))
     module.add(SMovB32(dst=sgpr("Srd%s+3"%ch), src="Srd127_96", comment="Set bits 127_96 in post-loop SRD"))
-    module.add(SBranchIfNotZero("Address%s"%chAddress, DataType('int64'), label))
+    module.add(BranchIfNotZero("Address%s"%chAddress, DataType('int64').toEnum(), label))
     module.add(SMovB32(dst=sgpr("Srd%s+2"%ch), src=0))
     module.add(SBranch(label2.getLabelName()))
     module.add(label)
@@ -117,7 +121,7 @@ def wait(states, kernel, tPA, tPB, skipGlobalRead, skipLocalWrite, \
     # This line is added for backward compatibility
     vscnt = vmcnt if lgkmcnt != -1 and vmcnt != -1 and states.archCaps["SeparateVscnt"] else -1
 
-    waitcnt = SWaitCnt(lgkmcnt,vmcnt, vscnt, comment)
+    waitcnt = SWaitCnt(lgkmcnt,vmcnt, vscnt, comment=comment)
     return waitcnt
 
 ##############################################################################
@@ -127,7 +131,7 @@ def syncThreads(kernel, archCaps, comment="", skipForceWaitcnt0=False):
     imod = Module("syncThreads")
     if kernel["NumThreads"] > kernel["WavefrontSize"]:
         if archCaps["SeparateVscnt"]:
-            imod.add(SWaitCnt(lgkmcnt="null", comment="extra navi wait"))
+            imod.add(SWaitCnt(lgkmcnt=-2, comment="extra navi wait"))
         elif kernel["ScheduleIterAlg"] == 2 \
           or kernel["PrefetchGlobalRead"] == 2 \
           or skipForceWaitcnt0:
@@ -202,8 +206,7 @@ def mapAcctoArchRegs(kernel, maxAgpr=256, write=False):
   acc2arch, _ = accToArchMapper(kernel)
 
   complexMultiplier = 2 if kernel["ProblemType"]["DataType"].isComplex() else 1
-  imod = Module("AccVgpr{}".format("Write" if write else "Read"))
-  imod.itemList = [None] * kernel["MIRegPerOut"] * complexMultiplier * len(acc2arch)
+  itemList = [None] * kernel["MIRegPerOut"] * complexMultiplier * len(acc2arch)
   accImOffset = accVgprImagNumOffset(kernel)
   for i in range(len(acc2arch)):
     for cm in range(complexMultiplier):
@@ -219,31 +222,33 @@ def mapAcctoArchRegs(kernel, maxAgpr=256, write=False):
           accStr = gprfunc(srcIdx)
           if srcIdx >= maxAgpr:
             if write:
-              imod.itemList[destIdx] = VMovB32(dst=vgpr("ValuC+%u"%(srcIdx-maxAgpr)),
+              itemList[destIdx] = VMovB32(dst=vgpr("ValuC+%u"%(srcIdx-maxAgpr)),
                                              src=vgpr(Holder(name="ValuC")),
                                              comment="copy vreg[%u] to MI out reg" % destIdx)
             else:
-              imod.itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
+              itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
                                               src=vgpr("ValuC+%u"%(srcIdx-maxAgpr)),
                                               comment="copy MI out reg to vreg[%u]" % destIdx)
           else:
             if write:
-              imod.itemList[destIdx] = VAccvgprWriteB32(dst=accStr,
+              itemList[destIdx] = VAccvgprWriteB32(dst=accStr,
                                                         src=vgpr(Holder(name="ValuC")),
                                                         comment="copy vreg[%u] to acc" % destIdx)
             else:
-              imod.itemList[destIdx] = VAccvgprReadB32(dst=vgpr(Holder(name="ValuC")),
+              itemList[destIdx] = VAccvgprReadB32(dst=vgpr(Holder(name="ValuC")),
                                                       src=accStr,
                                                       comment="copy acc to vreg[%u]" % destIdx)
         else:
           if write:
-            imod.itemList[destIdx] = VMovB32(dst=vgpr("ValuC+%u"%srcIdx),
+            itemList[destIdx] = VMovB32(dst=vgpr("ValuC+%u"%srcIdx),
                                              src=vgpr(Holder(name="ValuC")),
                                              comment="copy vreg[%u] to MI out reg" % destIdx)
           else:
-            imod.itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
+            itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
                                              src=vgpr("ValuC+%u"%srcIdx),
                                              comment="copy MI out reg to vreg[%u]" % destIdx)
+  imod = Module("AccVgpr{}".format("Write" if write else "Read"))
+  imod.setItems(itemList)
   return imod
 
 ##############################################################################
@@ -253,26 +258,25 @@ def mapAcctoArchRegs(kernel, maxAgpr=256, write=False):
 def mulMIoutAlphaToArch(kernel, startVgprAlphaTmp):
   acc2arch, _ = accToArchMapper(kernel)
 
-  imod = Module("MulAlpha")
-  imod.itemList = [None] * len(acc2arch)
+  itemList = [None] * len(acc2arch)
   for i in range(len(acc2arch)):
     destIdx = acc2arch[i]
     srcIdx  = i * kernel["MIRegPerOut"]
     if kernel["ProblemType"]["ComputeDataType"].isDouble():
-      imod.itemList[destIdx] = VMulF64(dst=vgpr(Holder(name="ValuC"),2),
+      itemList[destIdx] = VMulF64(dst=vgpr(Holder(name="ValuC"),2),
                                                     src0=sgpr("Alpha",2), src1=vgpr("ValuC+%u"%srcIdx,2),
                                                     comment="Multiply MI out reg with alpha")
     elif kernel["ProblemType"]["ComputeDataType"].isSingle() or \
         (kernel["ProblemType"]["ComputeDataType"].isHalf() and kernel["ProblemType"]["HighPrecisionAccumulate"]):
-      imod.itemList[destIdx] = VMulF32(dst=vgpr(Holder(name="ValuC")),
+      itemList[destIdx] = VMulF32(dst=vgpr(Holder(name="ValuC")),
                                                     src0=sgpr("Alpha"), src1=vgpr("ValuC+%u"%srcIdx),
                                                     comment="Multiply MI out reg with alpha")
     elif (kernel["ProblemType"]["ComputeDataType"].isHalf() and not kernel["ProblemType"]["HighPrecisionAccumulate"]):
-        imod.itemList[destIdx] = VMulPKF16(dst=vgpr(Holder(name="ValuC")),
+        itemList[destIdx] = VMulPKF16(dst=vgpr(Holder(name="ValuC")),
                                                        src0=sgpr("Alpha"),
                                                        src1=vgpr("ValuC+%u"%srcIdx), comment="Multiply MI out reg with alpha")
     elif kernel["ProblemType"]["ComputeDataType"].isInt32():
-      imod.itemList[destIdx] = VMulLOU32(dst=vgpr(Holder(name="ValuC")),
+      itemList[destIdx] = VMulLOU32(dst=vgpr(Holder(name="ValuC")),
                                                       src0=sgpr("Alpha"), src1=vgpr("ValuC+%u"%srcIdx),
                                                        comment="Multiply MI out reg with alpha")
     elif kernel["ProblemType"]["ComputeDataType"].isSingleComplex():
@@ -289,7 +293,7 @@ def mulMIoutAlphaToArch(kernel, startVgprAlphaTmp):
         cimod.add(VFmaF32(dst=vgpr(Holder(name="ValuC")), src0=sgpr("Alpha+1"), src1=vgpr("ValuC+%u"%(srcIdx+accImOffset)), src2=vgpr(vtmp1)))
         # c.imag = a.real * b.imag + a.imag * b.real = a.real * b.imag + tmp2
         cimod.add(VFmaF32(dst=vgpr(Holder(name="ValuC+1")), src0=sgpr("Alpha+0"), src1=vgpr("ValuC+%u"%(srcIdx+accImOffset)), src2=vgpr(vtmp2)))
-        imod.itemList[destIdx] = cimod
+        itemList[destIdx] = cimod
     elif kernel["ProblemType"]["ComputeDataType"].isDoubleComplex():
       accImOffset = accVgprImagNumOffset(kernel)
       cimod = Module()
@@ -304,7 +308,10 @@ def mulMIoutAlphaToArch(kernel, startVgprAlphaTmp):
       cimod.add(VFmaF64(dst=vgpr(Holder(name="ValuC"),2), src0=sgpr("Alpha+2",2), src1=vgpr("ValuC+%u"%(srcIdx+accImOffset),2), src2=vgpr(vtmp1,2)))
       # c.imag = a.real * b.imag + a.imag * b.real = a.real * b.imag + tmp2
       cimod.add(VFmaF64(dst=vgpr(Holder(name="ValuC+2"),2), src0=sgpr("Alpha+0",2), src1=vgpr("ValuC+%u"%(srcIdx+accImOffset),2), src2=vgpr(vtmp2,2)))
-      imod.itemList[destIdx] = cimod
+      itemList[destIdx] = cimod
+
+  imod = Module("MulAlpha")
+  imod.setItems(itemList)
   return imod
 
   ##############################################################################
@@ -314,31 +321,30 @@ def mulMIoutAlphaToArch(kernel, startVgprAlphaTmp):
 def moveMIoutToArch(kernel, startVgprAlphaTmp):
   acc2arch, _ = accToArchMapper(kernel)
 
-  imod = Module("MulAlpha")
-  imod.itemList = [None] * len(acc2arch)
+  itemList = [None] * len(acc2arch)
   for i in range(len(acc2arch)):
     destIdx = acc2arch[i]
     srcIdx  = i * kernel["MIRegPerOut"]
     if kernel["ProblemType"]["ComputeDataType"].isDouble():
-      imod.itemList[destIdx] = VLShiftLeftB64(dst=vgpr(Holder(name="ValuC"), 2),
+      itemList[destIdx] = VLShiftLeftB64(dst=vgpr(Holder(name="ValuC"), 2),
                                                      shiftHex=0,
                                                      src=vgpr("ValuC+%u"%srcIdx,2), comment="Rearrange MI out reg")
     elif kernel["ProblemType"]["ComputeDataType"].isSingle() or \
         (kernel["ProblemType"]["ComputeDataType"].isHalf() and kernel["ProblemType"]["HighPrecisionAccumulate"]):
-      imod.itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
+      itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
                                                      src=vgpr("ValuC+%u"%srcIdx), comment="Rearrange MI out reg")
     elif (kernel["ProblemType"]["ComputeDataType"].isHalf() and not kernel["ProblemType"]["HighPrecisionAccumulate"]):
-      imod.itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
+      itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
                                                      src=vgpr("ValuC+%u"%srcIdx), comment="Rearrange MI out reg")
     elif kernel["ProblemType"]["ComputeDataType"].isInt32():
-      imod.itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
+      itemList[destIdx] = VMovB32(dst=vgpr(Holder(name="ValuC")),
                                                      src=vgpr("ValuC+%u"%srcIdx), comment="Rearrange MI out reg")
     elif kernel["ProblemType"]["ComputeDataType"].isSingleComplex():
         accImOffset = accVgprImagNumOffset(kernel, lrvwB)
         cimod = Module()
         cimod.add(VMovB32(dst=vgpr(Holder(name="ValuC")), src=vgpr("ValuC+%u"%srcIdx), comment="Rearrange MI out reg"))
         cimod.addInst(VMovB32(dst=vgpr(Holder(name="ValuC+1")), src=vgpr("ValuC+%u"%(srcIdx+accImOffset)), comment="Rearrange MI out reg"))
-        imod.itemList[destIdx] = cimod
+        itemList[destIdx] = cimod
     elif kernel["ProblemType"]["ComputeDataType"].isDoubleComplex():
       accImOffset = accVgprImagNumOffset(kernel, lrvwB)
       cimod = Module()
@@ -346,7 +352,9 @@ def moveMIoutToArch(kernel, startVgprAlphaTmp):
       cimod.add(VLShiftLeftB64(dst=vgpr(Holder(name="ValuC"), 2), shiftHex=0, src=vgpr("ValuC+%u"%srcIdx,2), comment="Rearrange MI out reg"))
       # tmp2 = a.imag * b.real
       cimod.add(VLShiftLeftB64(dst=vgpr(Holder(name="ValuC+2"), 2), shiftHex=0, src=vgpr("ValuC+%u"%(srcIdx+accImOffset),2), comment="Rearrange MI out reg"))
-      imod.itemList[destIdx] = cimod
+      itemList[destIdx] = cimod
 
+  imod = Module("MulAlpha")
+  imod.setItems(itemList)
   return imod
 
