@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,17 +22,21 @@
 #
 ################################################################################
 
-from .Common import printExit, printWarning, print2, versionIsCompatible
 from .CustomKernels import getCustomKernelConfig
-from .SolutionStructs import Solution, ProblemSizes, ProblemType
-from . import __version__
-from . import Common
 from . import SolutionLibrary
 from .CustomYamlLoader import load_yaml_stream
+from Tensile import __version__
+from Tensile.Common import printExit, printWarning, print2, \
+                           versionIsCompatible, IsaInfo
+from Tensile.Common.Architectures import gfxToIsa
+from Tensile.SolutionStructs import Solution, ProblemSizes
+from Tensile.SolutionStructs.Problem import ProblemType
 
-from typing import NamedTuple, List
+from typing import NamedTuple, List, Dict
 import os
 import sys
+import subprocess
+import re
 
 try:
     import orjson as json
@@ -149,6 +153,8 @@ def writeSolutions(filename, problemSizes, biasTypeArgs, activationArgs, solutio
             if "DataTypeMetadata" in solutionState["ProblemType"]:
                 solutionState["ProblemType"]["DataTypeMetadata"] = \
                     solutionState["ProblemType"]["DataTypeMetadata"].value
+            isa = solutionState["ISA"]
+            solutionState["ISA"] = [isa[0], isa[1], isa[2]]
             solutionStates.append(solutionState)
     # write dictionaries
     with open(filename, "w") as f:
@@ -181,24 +187,50 @@ def read(filename, customizedLoader=False):
     else:
         printExit("Unrecognized read format {}".format(extension))
 
+
 def readYAML(filename):
     """Reads and returns YAML data from file."""
     with open(filename, "r") as f:
         data = yaml.load(f, yamlLoader)
     return data
 
+
 def readJson(filename):
     """Reads and returns JSON data from file."""
     with open(filename, "r") as f:
         data = json.loads(f.read())
     return data
 
-def parseSolutionsFile(filename, cxxCompiler):
+
+def parseSolutionsFile(
+        filename,
+        assembler,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        isaInfoMap
+    ):
     """Wrapper function to read and parse a solutions file."""
-    return parseSolutionsData(read(filename), filename, cxxCompiler)
+    return parseSolutionsData(
+               read(filename),
+               filename,
+               assembler,
+               splitGSU,
+               printSolutionRejectionReason,
+               printIndexAssignmentInfo,
+               isaInfoMap
+            )
 
 
-def parseSolutionsData(data, srcFile, cxxCompiler):
+def parseSolutionsData(
+        data,
+        srcFile,
+        assembler,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        isaInfoMap
+    ):
     """Parses problem sizes and solutions from the data of a solutions file."""
     if len(data) < 3:
         printExit("Solution file {} is missing required fields (len = {} < 3" \
@@ -225,7 +257,15 @@ def parseSolutionsData(data, srcFile, cxxCompiler):
         # force redo the deriving of parameters, make sure old version logic yamls can be validated
         solutionState["AssignedProblemIndependentDerivedParameters"] = False
         solutionState["AssignedDerivedParameters"] = False
-        solutionObject = Solution(solutionState, cxxCompiler)
+        solutionObject = Solution(
+                             solutionState,
+                             splitGSU,
+                             printSolutionRejectionReason,
+                             printIndexAssignmentInfo,
+                             assembler,
+                             isaInfoMap,
+                             srcFile
+                         )
         solutions.append(solutionObject)
     problemType = solutions[0]["ProblemType"]
     problemSizes = ProblemSizes(problemType, problemSizesConfig)
@@ -240,29 +280,43 @@ class LibraryLogic(NamedTuple):
     solutions: list
     exactLogic: list
     library: SolutionLibrary.MasterSolutionLibrary
-    srcFile: str
 
-
-def parseLibraryLogicFile(filename, cxxCompiler, archs=None):
+def parseLibraryLogicFile(
+        filename,
+        assembler,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        isaInfoMap: Dict[str, IsaInfo],
+        lazyLibraryLoading: bool
+    ):
     """Wrapper function to read and parse a library logic file."""
-    return parseLibraryLogicData(read(filename, True), filename, cxxCompiler, archs)
+    return parseLibraryLogicData(
+               read(filename, True),
+               filename,
+               assembler,
+               splitGSU,
+               printSolutionRejectionReason,
+               printIndexAssignmentInfo,
+               isaInfoMap,
+               lazyLibraryLoading
+           )
 
 
-def parseLibraryLogicData(data, srcFile, cxxCompiler, archs=None):
+def parseLibraryLogicData(
+        data,
+        srcFile,
+        assembler,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        isaInfoMap: Dict[str, IsaInfo],
+        lazyLibraryLoading: bool
+    ):
     """Parses the data of a library logic file."""
     if isinstance(data, List):
         data = parseLibraryLogicList(data, srcFile)
 
-    is_arch_valid = lambda cArch, tArch : (cArch == tArch or cArch == "all")
-    if not (archs is None) and "ArchitectureName" in data:
-        if isinstance(archs, List):
-            if len(archs) > 0 and not archs[0] == "all":
-                if not (any(is_arch_valid(arch.split(":")[0], data["ArchitectureName"]) for arch in archs)):
-                    return LibraryLogic("", "", None, [], [], None, srcFile)
-        elif isinstance(archs, str):
-            if not is_arch_valid(archs.split(":")[0], data["ArchitectureName"]):
-                return LibraryLogic("", "", None, [], [], None, srcFile)
-
     if "CUCount" not in data:
         data["CUCount"] = None
 
@@ -271,14 +325,12 @@ def parseLibraryLogicData(data, srcFile, cxxCompiler, archs=None):
                 .format(srcFile, data["MinimumRequiredVersion"], __version__) )
 
     # unpack problemType
-    problemType = ProblemType(data["ProblemType"])
+    problemType = ProblemType(data["ProblemType"], printIndexAssignmentInfo)
 
     # unpack solution
-    def solutionStateToSolution(solutionState, cxxCompiler) -> Solution:
+    def solutionStateToSolution(solutionState, assembler, isaInfoMap) -> Solution:
         if solutionState["KernelLanguage"] == "Assembly":
-            solutionState["ISA"] = Common.gfxArch(data["ArchitectureName"])
-        else:
-            solutionState["ISA"] = (0, 0, 0)
+            solutionState["ISA"] = gfxToIsa(data["ArchitectureName"])
         solutionState["CUCount"] = data["CUCount"]
         # force redo the deriving of parameters, make sure old version logic yamls can be validated
         solutionState["AssignedProblemIndependentDerivedParameters"] = False
@@ -290,10 +342,23 @@ def parseLibraryLogicData(data, srcFile, cxxCompiler, archs=None):
             customConfig = getCustomKernelConfig(solutionState["CustomKernelName"], isp)
             for key, value in customConfig.items():
                 solutionState[key] = value
+
+            if "MatrixInstruction" in customConfig and len(customConfig["MatrixInstruction"]) != 4:
+                raise ValueError(f"Custom kernel MatrixInstruction can only be of length 4, found {customConfig['MatrixInstruction']}")
+
             # The ActivationType setting in YAML is meaningless in customKernel case.
             # Therefore, we override the customKernel setting with the ActivationType value from ProblemType to avoid false alarms during subsequent problemType checks.
             solutionState["ProblemType"]["ActivationType"] = problemType["ActivationType"]
-        solutionObject = Solution(solutionState, cxxCompiler)
+
+        solutionObject = Solution(
+                             solutionState,
+                             splitGSU,
+                             printSolutionRejectionReason,
+                             printIndexAssignmentInfo,
+                             assembler,
+                             isaInfoMap,
+                             srcFile
+                         )
         solutionProblemType = solutionObject["ProblemType"]
         if problemType != solutionProblemType:
             # find the mismatched items in ProblemType
@@ -305,12 +370,21 @@ def parseLibraryLogicData(data, srcFile, cxxCompiler, archs=None):
             printExit(f"ProblemType in library logic file {srcFile} doesn't match solution(idx={solIdx}): \n{results}")
         return solutionObject
 
-    solutions = [solutionStateToSolution(solutionState, cxxCompiler) for solutionState in data["Solutions"]]
+    solutions = [solutionStateToSolution(solutionState, assembler, isaInfoMap) for solutionState in data["Solutions"]]
 
-    newLibrary, _ = SolutionLibrary.MasterSolutionLibrary.FromOriginalState(data, solutions, cxxCompiler)
+    newLibrary, _ = SolutionLibrary.MasterSolutionLibrary.FromOriginalState(
+        data,
+        solutions,
+        splitGSU,
+        printSolutionRejectionReason,
+        printIndexAssignmentInfo,
+        assembler,
+        isaInfoMap,
+        lazyLibraryLoading
+    )
 
     return LibraryLogic(data["ScheduleName"], data["ArchitectureName"], problemType, solutions, \
-            data.get("ExactLogic"), newLibrary, srcFile)
+            data.get("ExactLogic"), newLibrary)
 
 
 def parseLibraryLogicList(data, srcFile="?"):
@@ -354,6 +428,12 @@ def parseLibraryLogicList(data, srcFile="?"):
         rv["Library"]["indexOrder"] = None
         rv["Library"]["table"] = [0, len(data[5])]
         rv["Library"]["distance"] = None
+    elif libraryType == "Prediction":
+        rv["LibraryType"] = "Prediction"
+        rv["Library"] = {}
+        rv["Library"]["indexOrder"] = None
+        rv["Library"]["table"] = [0, len(data[5])]
+        rv["Library"]["distance"] = None
     else:
         rv["LibraryType"] = "Matching"
         rv["Library"] = {}
@@ -389,6 +469,17 @@ def rawLibraryLogic(data):
 #################
 # Other functions
 #################
+def getCUCount() -> int:
+    """Return the number of CU Count in current Hardware."""
+    CU = os.environ.get("CU", None)
+    if CU is None:
+        res = subprocess.run("rocminfo | grep Compute", stdout=subprocess.PIPE, shell=True, env={"ROCR_VISIBLE_DEVICES":"0"})
+        CU_RE = r"Compute Unit:(?P<COMPUTE_UNIT>[\w ]+)"
+        match = re.search(CU_RE, res.stdout.decode("utf-8").split('\n')[-2])
+        if match:
+            CU = int(match.group('COMPUTE_UNIT').strip())
+    return CU
+
 def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryType, logicTuple):
     """Creates the data for a library logic file suitable for writing to YAML."""
     problemType = logicTuple[0]
@@ -406,7 +497,9 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
     data.append({"MinimumRequiredVersion": __version__})
     # schedule name
     data.append(schedulePrefix)  # change from Tensile to vega10
-    data.append(architectureName)
+    # schedule architecture name and get CU count
+    CUCount=getCUCount()
+    data.append({"Architecture": architectureName, "CUCount": CUCount} if architectureName=="gfx942" and CUCount and CUCount!=304 else architectureName)
     # schedule device names
     data.append(deviceNames)
     # problem type
@@ -467,6 +560,8 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
         if "DataTypeMetadata" in solutionState["ProblemType"]:
             solutionState["ProblemType"]["DataTypeMetadata"] = \
                     solutionState["ProblemType"]["DataTypeMetadata"].value
+        isa = solutionState["ISA"]
+        solutionState["ISA"] = [isa[0], isa[1], isa[2]]
         solutionList.append(solutionState)
 
     if tileSelection:
