@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,14 +22,14 @@
 #
 ################################################################################
 
-import itertools
+from typing import Dict
 
 from . import Properties
 from . import Hardware
-from . import Common
 from . import Contractions
-from .SolutionStructs import Solution as OriginalSolution
-from .Utils import state
+from Tensile.Common import state, IsaInfo
+from Tensile.Common.Architectures import gfxToIsa
+from Tensile.SolutionStructs.Naming import getSolutionNameMin, getKernelNameMin
 
 class SingleSolutionLibrary:
     Tag = "Single"
@@ -173,33 +173,63 @@ class FreeSizeLibrary:
     def __init__(self, table):
         self.table = table
 
-class DecisionTreeLibrary:
-    Tag = "DecisionTree"
-    StateKeys = [("type", "tag"), "features", "trees", "nullValue"]
+class PredictionLibrary:
+    Tag = "Prediction"
+    StateKeys = [("type", "tag"), "table"]
 
     @classmethod
     def FromOriginalState(cls, d, solutions):
-        features = d["features"]
-        origTrees = d["trees"]
+        origTable = d["table"]
 
-        trees = []
-        
-        if "fallback" in d:
-            fallbackIndex = d["fallback"]
-            nullValue = SingleSolutionLibrary(solutions[fallbackIndex])
-        else:
-            raise RuntimeError(
-                "DecisionTree has no fallback; likely a legacy model which is not supported."
-            )
+        table = []
 
-        for tree in origTrees:
-            index = tree["solution"]
-            value = SingleSolutionLibrary(solutions[index])
+        try:
+            indexStart  = origTable[0]
+            indexOffset = origTable[1]
+            for index in range(indexStart, indexStart + indexOffset):
+                value = IndexSolutionLibrary(solutions[index])
+                table.append(value)
+        except KeyError:
+            pass
 
-            entry = {"tree": tree["tree"], "value": value}
-            trees.append(entry)
+        return cls(table)
 
-        return cls(features, trees, nullValue)
+    @property
+    def tag(self):
+        return self.__class__.Tag
+
+    def merge(self, other):
+        assert self.__class__ == other.__class__
+
+        self.table += other.table
+
+    def remapSolutionIndices(self, indexMap):
+        pass
+
+    def __init__(self, table):
+        self.table = table
+
+class MLPClassificationLibrary:
+    Tag = "MLPClassification"
+    StateKeys = [("type", "tag"), "table", "mlp", "problemFeatures"]
+
+    @classmethod
+    def FromOriginalState(cls, d, solutions):
+        origTable = d["table"]
+        table = []
+
+        try:
+            indexStart  = origTable[0]
+            indexOffset = origTable[1]
+            for index in range(indexStart, indexStart + indexOffset):
+                value = IndexSolutionLibrary(solutions[index])
+                table.append(value)
+        except KeyError:
+            pass
+
+        mlp = d["mlp"]
+        problem_features = d["problemFeatures"]
+        return cls(table, mlp, problem_features)
 
     @property
     def tag(self):
@@ -207,16 +237,16 @@ class DecisionTreeLibrary:
 
     def merge(self, other):
         raise RuntimeError(
-            "DecisionTreeLibrary does not support merging; ensure each library row has a unique predicate"
+            "MLPClassificationLibrary does not support merging."
         )
 
     def remapSolutionIndices(self, indexMap):
         pass
 
-    def __init__(self, features, trees, nullValue):
-        self.features = features
-        self.trees = trees
-        self.nullValue = nullValue
+    def __init__(self, table, mlp, problem_features):
+        self.table = table
+        self.mlp = mlp
+        self.problemFeatures = problem_features
 
 
 class ProblemMapLibrary:
@@ -300,7 +330,12 @@ class MasterSolutionLibrary:
     def FromOriginalState(cls,
                           origData,
                           origSolutions,
-                          cxxCompiler,
+                          splitGSU: bool,
+                          printSolutionRejectionReason: bool,
+                          printIndexAssignmentInfo: bool,
+                          assembler,
+                          isaInfoMap: Dict[str, IsaInfo],
+                          lazyLibraryLoading: bool,
                           solutionClass=Contractions.Solution,
                           libraryOrder=None,
                           placeholderName='TensileLibrary'):
@@ -314,7 +349,7 @@ class MasterSolutionLibrary:
             if devicePart == "fallback":
                 pred = Hardware.HardwarePredicate("TruePred")
             else:
-                pred = Hardware.HardwarePredicate.FromHardware(Common.gfxArch(devicePart), cuCount)
+                pred = Hardware.HardwarePredicate.FromHardware(gfxToIsa(devicePart), cuCount)
 
             newLib.rows.append({"predicate": pred, "library": library})
 
@@ -381,23 +416,22 @@ class MasterSolutionLibrary:
                 freesizeLib = FreeSizeLibrary.FromOriginalState(d["Library"], solutions)
                 library = PredicateLibrary(tag="Problem")
                 library.rows.append({"predicate": predicate, "library": freesizeLib})
-            elif d["LibraryType"] == "DecisionTree":
+            elif d["LibraryType"] == "Prediction":
+                predicate = Properties.Predicate(tag="FreeSizeMatching") # TODO Do we need a new predicate here?
+
+                predictionLib = PredictionLibrary.FromOriginalState(d["Library"], solutions)
                 library = PredicateLibrary(tag="Problem")
-                for lib in d["Library"]:
-                    preds = lib["region"]
-                    predObjs = [Properties.Predicate.FromOriginalState(p) for p in preds]
+                library.rows.append({"predicate": predicate, "library": predictionLib})
+            elif d["LibraryType"] == "MLPClassification":
+                predicate = Properties.Predicate(tag="TruePred")
 
-                    if len(predObjs) == 1:
-                        predicate = predObjs[0]
-                    else:
-                        predicate = Properties.Predicate.And(predObjs)
-
-                    treeLib = DecisionTreeLibrary.FromOriginalState(lib, solutions)
-                    library.rows.append({"predicate": predicate, "library": treeLib})
+                regressionLib = MLPClassificationLibrary.FromOriginalState(d["Library"], solutions)
+                library = PredicateLibrary(tag="Problem")
+                library.rows.append({"predicate": predicate, "library": regressionLib})
             else:
                 assert 0 and "Unrecognized LibraryType."
 
-            if Common.globalParameters["LazyLibraryLoading"]:
+            if lazyLibraryLoading:
                 placeholderName += '_' + str(problemType.aType) + str(problemType.bType)
                 placeholderName += '_' + str(problemType.cType) + str(problemType.computeInputType)
                 if problemType.activationType != 'none':
@@ -442,7 +476,7 @@ class MasterSolutionLibrary:
         # end library creation functions
 
         if libraryOrder is None:
-            if Common.globalParameters["LazyLibraryLoading"]:
+            if lazyLibraryLoading:
                 libraryOrder = [
                     hardware, operationIdentifier, performanceMetric, predicates,
                     placeholder, selection
@@ -460,7 +494,12 @@ class MasterSolutionLibrary:
             lazyLibrary, placeholderName = \
                 MasterSolutionLibrary.FromOriginalState(origData,
                                                         origSolutions,
-                                                        cxxCompiler,
+                                                        splitGSU,
+                                                        printSolutionRejectionReason,
+                                                        printIndexAssignmentInfo,
+                                                        assembler,
+                                                        isaInfoMap,
+                                                        lazyLibraryLoading,
                                                         solutionClass,
                                                         libraryOrder[placeholderIndex:],
                                                         placeholderName)
@@ -468,7 +507,14 @@ class MasterSolutionLibrary:
             origSolutions = []
 
         problemType = Contractions.ProblemType.FromOriginalState(origData["ProblemType"])
-        allSolutions = [solutionClass.FromSolutionStruct(s, cxxCompiler) for s in origSolutions]
+        allSolutions = [solutionClass.FromSolutionStruct(
+                            s,
+                            splitGSU,
+                            printSolutionRejectionReason,
+                            printIndexAssignmentInfo,
+                            assembler,
+                            isaInfoMap
+                        ) for s in origSolutions]
         cls.FixSolutionIndices(allSolutions)
 
         # library is constructed in reverse order i.e. bottom-up
@@ -489,8 +535,23 @@ class MasterSolutionLibrary:
         return rv, placeholderName
 
     @classmethod
-    def BenchmarkingLibrary(cls, solutions, cxxCompiler):
-        solutionObjs = list([Contractions.Solution.FromOriginalState(s._state, cxxCompiler) for s in solutions])
+    def BenchmarkingLibrary(
+        cls,
+        solutions,
+        assembler,
+        splitGSU: bool,
+        printSolutionRejectionReason: bool,
+        printIndexAssignmentInfo: bool,
+        isaInfoMap
+    ):
+        solutionObjs = list([Contractions.Solution.FromOriginalState(
+                                 s._state,
+                                 splitGSU,
+                                 printSolutionRejectionReason,
+                                 printIndexAssignmentInfo,
+                                 assembler,
+                                 isaInfoMap)
+                            for s in solutions])
         cls.FixSolutionIndices(solutionObjs)
 
         predRows = list([{
@@ -519,14 +580,10 @@ class MasterSolutionLibrary:
             rv["version"] = self.version
         return rv
 
-    def applyNaming(self, naming=None):
-        if naming is None:
-            kernels = itertools.chain(s.originalSolution.getKernels() for s in self.solutions.values())
-            naming = OriginalSolution.getMinNaming(kernels)
-
+    def applyNaming(self, splitGSU: bool):
         for s in list(self.solutions.values()):
-            s.name = OriginalSolution.getNameMin(s.originalSolution.getKernels()[0], naming)
-            s.kernelName = OriginalSolution.getNameMin(s.originalSolution.getKernels()[0], naming, True)
+            s.name = getSolutionNameMin(s.originalSolution.getKernels()[0], splitGSU)
+            s.kernelName = getKernelNameMin(s.originalSolution.getKernels()[0], splitGSU)
 
     def remapSolutionIndicesStartingFrom(self, curIndex):
         reIndexMap = {}
