@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,7 @@
 #include <Tensile/ContractionProblem.hpp>
 #include <Tensile/ContractionSolution.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Client
     {
@@ -66,7 +66,9 @@ namespace Tensile
             virtual size_t numWarmupRuns() override;
             virtual void   setNumWarmupRuns(size_t count) override;
             virtual void   preWarmup() override;
-            virtual void   postWarmup() override;
+            virtual void   postWarmup(TimingEvents const& startEvents,
+                                      TimingEvents const& stopEvents,
+                                      hipStream_t const&  stream) override;
             virtual void   validateWarmups(std::shared_ptr<ProblemInputs> inputs,
                                            TimingEvents const&            startEvents,
                                            TimingEvents const&            stopEvents) override;
@@ -128,7 +130,11 @@ namespace Tensile
 
             double_millis m_timeInSolution;
             double_millis m_totalGPUTime;
+            double_millis m_currentBestWarmUpTime;
             float         m_flushTimeUs;
+            float         m_skip_slow_solution_ratio;
+            bool          m_skip_slow_solution;
+            size_t        m_numSolutionSkip;
         };
     } // namespace Client
-} // namespace Tensile
+} // namespace TensileLite
