@@ -29,7 +29,7 @@
 #include "Reference.hpp"
 #include <iostream>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Client
     {
@@ -376,4 +376,4 @@ namespace Tensile
         };
 
     } // namespace Client
-} // namespace Tensile
+} // namespace TensileLite
