@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
  *******************************************************************************/
 
 #include <Tensile/Contractions.hpp>
+#include <Tensile/DataTypes.hpp>
 #include <Tensile/EmbeddedLibrary.hpp>
 #include <Tensile/MasterSolutionLibrary.hpp>
 #include <Tensile/Tensile.hpp>
@@ -57,6 +58,7 @@
 
 #include <chrono>
 #include <cstddef>
+#include <memory>
 
 namespace po = boost::program_options;
 
@@ -192,20 +194,20 @@ namespace TensileLite
                 ("problem-identifier",       po::value<std::string>(), "Problem identifer (Einstein notation). Either "
                                                                        "this or free/batch/bound must be specified.")
 
-                ("type",                     po::value<DataType>()->default_value(DataType::None), "Data type")
-                ("a-type",                   po::value<DataType>()->default_value(DataType::None), "A data type")
-                ("b-type",                   po::value<DataType>()->default_value(DataType::None), "B data type")
-                ("c-type",                   po::value<DataType>()->default_value(DataType::None), "C data type")
-                ("d-type",                   po::value<DataType>()->default_value(DataType::None), "D data type")
-                ("e-type",                   po::value<DataType>()->default_value(DataType::None), "E data type")
-                ("amaxD-type",               po::value<DataType>()->default_value(DataType::None), "amaxD data type")
-                ("alpha-type",               po::value<DataType>()->default_value(DataType::None), "alpha data type")
-                ("beta-type",                po::value<DataType>()->default_value(DataType::None), "beta data type")
-                ("compute-input-type",       po::value<DataType>()->default_value(DataType::None), "compute input data type")
-                ("f32-xdl-math-op",          po::value<DataType>()->default_value(DataType::None), "Use xf32 compute for float input and output matrices.")
+                ("type",                     po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "Data type")
+                ("a-type",                   po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "A data type")
+                ("b-type",                   po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "B data type")
+                ("c-type",                   po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "C data type")
+                ("d-type",                   po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "D data type")
+                ("e-type",                   po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "E data type")
+                ("amaxD-type",               po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "amaxD data type")
+                ("alpha-type",               po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "alpha data type")
+                ("beta-type",                po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "beta data type")
+                ("compute-input-type",       po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "compute input data type")
+                ("f32-xdl-math-op",          po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "Use xf32 compute for float input and output matrices.")
                 ("swizzle-tensor-a",         po::value<bool>()->default_value(false), "Swizzle input tensor A.")
                 ("swizzle-tensor-b",         po::value<bool>()->default_value(false), "Swizzle input tensor B.")
-                ("activation-compute-type",  po::value<DataType>()->default_value(DataType::None), "Activation compute type.")
+                ("activation-compute-type",  po::value<rocisa::DataType>()->default_value(rocisa::DataType::None), "Activation compute type.")
                 ("high-precision-accumulate", po::value<bool>()->default_value(false), "Use high-precision accumulate.")
                 ("sparse",                   po::value<int>()->default_value(0), "A or B matrix is sparse matrix.")
                 ("strided-batched",          po::value<bool>()->default_value(true), "Use strided-batched or general batched")
@@ -245,13 +247,14 @@ namespace TensileLite
                 "4:Memory bound check by both side guard page")
                 ("prune-mode",               po::value<PruneSparseMode>()->default_value(PruneSparseMode::PruneRandom), "prune Sparse mode")
 
-                ("print-tensor-a",           po::value<bool>()->default_value(false), "Print tensor A.")
-                ("print-tensor-b",           po::value<bool>()->default_value(false), "Print tensor B.")
-                ("print-tensor-c",           po::value<bool>()->default_value(false), "Print tensor C.")
-                ("print-tensor-d",           po::value<bool>()->default_value(false), "Print tensor D.")
-                ("print-tensor-ref",         po::value<bool>()->default_value(false), "Print reference tensor D.")
-                ("print-tensor-bias",        po::value<bool>()->default_value(false), "Print tensor Bias.")
-                ("print-tensor-amaxd",       po::value<bool>()->default_value(false), "Print tensor AmaxD value from both CPU and GPU.")
+                ("print-tensor-a",                  po::value<bool>()->default_value(false), "Print tensor A.")
+                ("print-tensor-b",                  po::value<bool>()->default_value(false), "Print tensor B.")
+                ("print-tensor-c",                  po::value<bool>()->default_value(false), "Print tensor C.")
+                ("print-tensor-d",                  po::value<bool>()->default_value(false), "Print tensor D.")
+                ("print-tensor-ref",                po::value<bool>()->default_value(false), "Print reference tensor D.")
+                ("print-tensor-bias",               po::value<bool>()->default_value(false), "Print tensor Bias.")
+                ("print-tensor-scale-alpha-vec",    po::value<bool>()->default_value(false), "Print tensor ScaleAlphaVec.")
+                ("print-tensor-amaxd",              po::value<bool>()->default_value(false), "Print tensor AmaxD value from both CPU and GPU.")
 
                 ("dump-tensors",             po::value<bool>()->default_value(false), "Binary dump tensors instead of printing.")
 
@@ -346,7 +349,7 @@ namespace TensileLite
                 ("use-scaleAB",               po::value<std::string>()->default_value(""), "Use scaleAB.")
                 ("use-scaleCD",               po::value<bool>()->default_value(false), "Use scaleCD.")
                 ("use-scaleAlphaVec",         po::value<int>()->default_value(0), "Use scaleAlphaVec.")
-                ("bias-type-args",            po::value<std::vector<DataType>>()->default_value(std::vector<DataType>(1, DataType::None), "[]"), "Bias data type args.")
+                ("bias-type-args",            po::value<std::vector<rocisa::DataType>>()->default_value(std::vector<rocisa::DataType>(1, rocisa::DataType::None), "[]"), "Bias data type args.")
                 ("factor-dim-args",           po::value<std::vector<int>>()->default_value(std::vector<int>(1, 0), "[]"), "factor dimensions args.")
                 ("icache-flush-args",         po::value<std::vector<bool>>()->default_value(std::vector<bool>(1, false), "[]"), "ICache flush args.")
                 ("use-e",                     po::value<bool>()->default_value(false), "Use E.")
@@ -491,7 +494,7 @@ namespace TensileLite
 
         void parse_bias_type_args(po::variables_map& args, std::string const& name)
         {
-            auto type             = args[name].as<std::vector<DataType>>();
+            auto type             = args[name].as<std::vector<rocisa::DataType>>();
             args.at(name).value() = boost::any(type);
         }
 
@@ -510,12 +513,13 @@ namespace TensileLite
 
         void fix_data_types(po::variables_map& args)
         {
-            auto type = args["type"].as<DataType>();
+            auto type = args["type"].as<rocisa::DataType>();
 
             // These types use the same data type for all inputs/outputs, so we allow
             // using the overarching 'type' parameter.
-            if(type == DataType::Float || type == DataType::Double || type == DataType::ComplexFloat
-               || type == DataType::ComplexDouble || type == DataType::Int32)
+            if(type == rocisa::DataType::Float || type == rocisa::DataType::Double
+               || type == rocisa::DataType::ComplexFloat || type == rocisa::DataType::ComplexDouble
+               || type == rocisa::DataType::Int32)
             {
                 args.at("a-type").value()     = boost::any(type);
                 args.at("b-type").value()     = boost::any(type);
@@ -646,8 +650,7 @@ int main(int argc, const char* argv[])
         iter--;
     }
 
-    auto* ptr      = new DataInitialization(args, problemFactory);
-    auto  dataInit = std::shared_ptr<DataInitialization>(ptr);
+    auto dataInit = std::make_shared<DataInitialization>(args, problemFactory);
 
     auto solutionIterator = SolutionIterator::Default(library, hardware, args);
 
@@ -740,6 +743,9 @@ int main(int argc, const char* argv[])
                     if(solution == nullptr)
                         throw std::runtime_error("Could not find a solution");
 
+                    // for a new problem, autoGSU should be re-calculated
+                    solution->autoGSU = 0;
+
                     listeners.preSolution(*solution);
                     if(solutionIterator->runCurrentSolution() && runKernels)
                     {
