@@ -37,19 +37,20 @@
 
 #include "ResultReporter.hpp"
 
-#define RSMI_CHECK_EXC(expr)                                                                                   \
-    do                                                                                                         \
-    {                                                                                                          \
-        rsmi_status_t e = (expr);                                                                              \
-        if(e)                                                                                                  \
-        {                                                                                                      \
-            const char* errName = nullptr;                                                                     \
-            rsmi_status_string(e, &errName);                                                                   \
-            std::ostringstream msg;                                                                            \
-            msg << "Error " << e << "(" << errName << ") " << __FILE__ << ":" << __LINE__ << ": " << std::endl \
-                << #expr << std::endl;                                                                         \
-            throw std::runtime_error(msg.str());                                                               \
-        }                                                                                                      \
+#define RSMI_CHECK_EXC(expr)                                                                      \
+    do                                                                                            \
+    {                                                                                             \
+        rsmi_status_t e = (expr);                                                                 \
+        if(e)                                                                                     \
+        {                                                                                         \
+            const char* errName = nullptr;                                                        \
+            rsmi_status_string(e, &errName);                                                      \
+            std::ostringstream msg;                                                               \
+            msg << "Error " << e << "(" << errName << ") " << __FILE__ << ":" << __LINE__ << ": " \
+                << std::endl                                                                      \
+                << #expr << std::endl;                                                            \
+            throw std::runtime_error(msg.str());                                                  \
+        }                                                                                         \
     } while(0)
 
 namespace TensileLite
@@ -68,8 +69,9 @@ namespace TensileLite
             HIP_CHECK_EXC(hipRuntimeGetVersion(&hip_version));
             if(hip_version >= 50220730)
             {
-                HIP_CHECK_EXC(hipDeviceGetAttribute(
-                    &props.multiProcessorCount, hipDeviceAttributePhysicalMultiProcessorCount, hipDeviceIndex));
+                HIP_CHECK_EXC(hipDeviceGetAttribute(&props.multiProcessorCount,
+                                                    hipDeviceAttributePhysicalMultiProcessorCount,
+                                                    hipDeviceIndex));
             }
 #endif
 
@@ -102,7 +104,8 @@ namespace TensileLite
             }
 
             msg << "]" << std::endl;
-            std::time_t now = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
+            std::time_t now
+                = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
             msg << std::put_time(gmtime(&now), "%F %T %z");
 
             throw std::runtime_error(concatenate("RSMI Can't find a device with PCI ID ",
@@ -136,7 +139,7 @@ namespace TensileLite
 #if rocm_smi_VERSION_MAJOR >= 7
             auto status2 = rsmi_dev_metrics_xcd_counter_get(m_smiDeviceIndex, &m_XCDCount);
 
-            if(status2 != RSMI_STATUS_SUCCESS)
+            if(status2 != RSMI_STATUS_SUCCESS || m_XCDCount == 0)
             {
                 m_XCDCount = 1;
             }
@@ -156,7 +159,7 @@ namespace TensileLite
 #if rocm_smi_VERSION_MAJOR >= 7
             auto status2 = rsmi_dev_metrics_xcd_counter_get(m_smiDeviceIndex, &m_XCDCount);
 
-            if(status2 != RSMI_STATUS_SUCCESS)
+            if(status2 != RSMI_STATUS_SUCCESS || m_XCDCount == 0)
             {
                 m_XCDCount = 1;
             }
@@ -179,7 +182,8 @@ namespace TensileLite
             m_thread = std::thread([this]() { this->runLoop(); });
         }
 
-        void HardwareMonitor::addTempMonitor(rsmi_temperature_type_t sensorType, rsmi_temperature_metric_t metric)
+        void HardwareMonitor::addTempMonitor(rsmi_temperature_type_t   sensorType,
+                                             rsmi_temperature_metric_t metric)
         {
             assertNotActive();
 
@@ -203,7 +207,8 @@ namespace TensileLite
             m_fanValues.resize(m_fanMetrics.size());
         }
 
-        double HardwareMonitor::getAverageTemp(rsmi_temperature_type_t sensorType, rsmi_temperature_metric_t metric)
+        double HardwareMonitor::getAverageTemp(rsmi_temperature_type_t   sensorType,
+                                               rsmi_temperature_metric_t metric)
         {
             assertNotActive();
 
@@ -222,8 +227,8 @@ namespace TensileLite
                 }
             }
 
-            throw std::runtime_error(
-                concatenate("Can't read temp value that wasn't requested: ", sensorType, " - ", metric));
+            throw std::runtime_error(concatenate(
+                "Can't read temp value that wasn't requested: ", sensorType, " - ", metric));
         }
 
         double HardwareMonitor::getAverageClock(rsmi_clk_type_t clockType)
@@ -252,7 +257,8 @@ namespace TensileLite
                 }
             }
 
-            throw std::runtime_error(concatenate("Can't read clock value that wasn't requested: ", clockType));
+            throw std::runtime_error(
+                concatenate("Can't read clock value that wasn't requested: ", clockType));
         }
 
         double HardwareMonitor::getAverageFanSpeed(uint32_t sensorIndex)
@@ -274,7 +280,8 @@ namespace TensileLite
                 }
             }
 
-            throw std::runtime_error(concatenate("Can't read fan value that wasn't requested: ", sensorIndex));
+            throw std::runtime_error(
+                concatenate("Can't read fan value that wasn't requested: ", sensorIndex));
         }
 
         void HardwareMonitor::start()
@@ -303,7 +310,8 @@ namespace TensileLite
 
                 m_hasStopEvent = stopEvent != nullptr;
 
-                m_task   = std::move(Task([this, startEvent, stopEvent]() { this->collect(startEvent, stopEvent); }));
+                m_task   = std::move(Task(
+                    [this, startEvent, stopEvent]() { this->collect(startEvent, stopEvent); }));
                 m_future = m_task.get_future();
 
                 m_stop = false;
@@ -326,8 +334,9 @@ namespace TensileLite
             m_lastCollection = clock::time_point();
             m_nextCollection = clock::time_point();
 
-            m_SYSCLK_sum   = std::vector<uint64_t>(m_XCDCount, 0);
-            m_SYSCLK_array = std::vector<std::vector<uint64_t>>(m_XCDCount, std::vector<uint64_t>{});
+            m_SYSCLK_sum = std::vector<uint64_t>(m_XCDCount, 0);
+            m_SYSCLK_array
+                = std::vector<std::vector<uint64_t>>(m_XCDCount, std::vector<uint64_t>{});
         }
 
         void HardwareMonitor::collectOnce()
@@ -345,7 +354,8 @@ namespace TensileLite
                 std::tie(sensorType, metric) = m_tempMetrics[i];
 
                 int64_t newValue = 0;
-                auto    status   = rsmi_dev_temp_metric_get(m_smiDeviceIndex, sensorType, metric, &newValue);
+                auto    status
+                    = rsmi_dev_temp_metric_get(m_smiDeviceIndex, sensorType, metric, &newValue);
                 if(status != RSMI_STATUS_SUCCESS)
                     m_tempValues[i] = std::numeric_limits<int64_t>::max();
                 else
@@ -372,14 +382,16 @@ namespace TensileLite
                         for(uint32_t xcd = 0; xcd < m_XCDCount; xcd++)
                         {
                             m_SYSCLK_sum[xcd] += gpuMetrics.current_gfxclks[xcd] * cMhzToHz;
-                            m_SYSCLK_array[xcd].push_back(gpuMetrics.current_gfxclks[xcd] * cMhzToHz);
+                            m_SYSCLK_array[xcd].push_back(gpuMetrics.current_gfxclks[xcd]
+                                                          * cMhzToHz);
                             sysclkSum += gpuMetrics.current_gfxclks[xcd] * cMhzToHz;
                         }
                         m_clockValues[i] += sysclkSum;
                     }
 #else
                     // XCD0
-                    auto status = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, m_clockMetrics[i], &freq);
+                    auto status
+                        = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, m_clockMetrics[i], &freq);
                     if(status != RSMI_STATUS_SUCCESS)
                     {
                         m_clockValues[i] = std::numeric_limits<uint64_t>::max();
@@ -392,7 +404,8 @@ namespace TensileLite
                 }
                 else
                 {
-                    auto status = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, m_clockMetrics[i], &freq);
+                    auto status
+                        = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, m_clockMetrics[i], &freq);
                     if(status != RSMI_STATUS_SUCCESS)
                     {
                         m_clockValues[i] = std::numeric_limits<uint64_t>::max();
@@ -413,7 +426,7 @@ namespace TensileLite
                 rsmi_frequencies_t freq;
 
                 int64_t newValue = 0;
-                auto    status   = rsmi_dev_fan_rpms_get(m_smiDeviceIndex, m_fanMetrics[i], &newValue);
+                auto status = rsmi_dev_fan_rpms_get(m_smiDeviceIndex, m_fanMetrics[i], &newValue);
                 if(status != RSMI_STATUS_SUCCESS)
                     m_fanValues[i] = std::numeric_limits<int64_t>::max();
                 else
@@ -422,23 +435,42 @@ namespace TensileLite
 
             // Retrieves the maximum hardware supported frequency.
             rsmi_frequencies_t freqs;
-            auto               status = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, RSMI_CLK_TYPE_SYS, &freqs);
-            if(status != RSMI_STATUS_SUCCESS)
+            const int          MAX_RETRY  = 10;
+            const int          SLEEP_TIME = 100; // sleep time in milliseconds
+            bool               success    = false;
+
+            if(!has_maxFreqValues && !m_hasInvalidGpuFreqStatus)
             {
-                m_hasInvalidGpuFreqStatus = true;
-            }
-            else
-            {
-                if(!m_hasInvalidGpuFreqStatus && !has_maxFreqValues)
+                for(int retry = 0; retry < MAX_RETRY; ++retry)
                 {
-                    m_maxFreqValues = 0;
-                    for(auto freq : freqs.frequency)
+                    auto status
+                        = rsmi_dev_gpu_clk_freq_get(m_smiDeviceIndex, RSMI_CLK_TYPE_SYS, &freqs);
+
+                    if(status == RSMI_STATUS_SUCCESS)
                     {
-                        m_maxFreqValues = std::max(m_maxFreqValues, freq);
+                        success = true;
+                        break;
                     }
+                    // Sleep before next retry
+                    std::this_thread::sleep_for(std::chrono::milliseconds(SLEEP_TIME));
+                }
+
+                if(!success)
+                {
+                    m_hasInvalidGpuFreqStatus = true;
+                }
+                else if(freqs.num_supported > 0)
+                {
+                    m_maxFreqValues
+                        = *std::max_element(freqs.frequency, freqs.frequency + freqs.num_supported);
+
                     has_maxFreqValues = true;
                     m_maxFreqValues /= cMhzToHz; // Convert to MHz
                 }
+                else
+                {
+                    m_hasInvalidGpuFreqStatus = true;
+                }
             }
             m_dataPoints++;
         }
