@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -170,7 +170,24 @@ namespace TensileLite
         }
 
         template <typename T, typename Accumulator>
-        typename std::enable_if<!std::is_same<int8_t, T>::value, T>::type
+        typename std::enable_if<
+            std::is_same<Float8, T>::value || std::is_same<Float8_fnuz, T>::value
+                || std::is_same<BFloat8, T>::value || std::is_same<BFloat8_fnuz, T>::value,
+            T>::type
+            SaturateCast(Accumulator val)
+        {
+            if constexpr(std::is_same<Accumulator, BFloat16>::value)
+                return static_cast<T>(static_cast<float>(val));
+            else
+                return static_cast<T>(val);
+        }
+
+        template <typename T, typename Accumulator>
+        typename std::enable_if<!std::is_same<int8_t, T>::value && !std::is_same<Float8, T>::value
+                                    && !std::is_same<Float8_fnuz, T>::value
+                                    && !std::is_same<BFloat8, T>::value
+                                    && !std::is_same<BFloat8_fnuz, T>::value,
+                                T>::type
             SaturateCast(Accumulator val)
         {
             return static_cast<T>(val);
@@ -182,67 +199,87 @@ namespace TensileLite
                                     || std::is_same<double, Accumulator>::value
                                     || std::is_same<BFloat16, Accumulator>::value
                                     || std::is_same<int32_t, Accumulator>::value
+                                    || std::is_same<int64_t, Accumulator>::value
                                     || std::is_same<int8_t, Accumulator>::value,
                                 Accumulator>::type
-            GetValue(DataType dataType, void const* voidPtr, int pos, bool aConjugate)
+            GetValue(rocisa::DataType dataType, void const* voidPtr, int pos, bool aConjugate)
         {
             switch(dataType)
             {
-            case DataType::Float:
+            case rocisa::DataType::Float:
             {
                 auto typedPtr = static_cast<float const*>(voidPtr);
                 return cast<Accumulator>(Transform<float>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::Double:
+            case rocisa::DataType::Double:
             {
                 auto typedPtr = static_cast<double const*>(voidPtr);
                 return cast<Accumulator>(Transform<double>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::Half:
+            case rocisa::DataType::Half:
             {
                 auto typedPtr = static_cast<Half const*>(voidPtr);
                 return cast<Accumulator>(Transform<Half>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::Int32:
+            case rocisa::DataType::Int32:
             {
                 auto typedPtr = static_cast<int32_t const*>(voidPtr);
                 return cast<Accumulator>(Transform<int32_t>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::BFloat16:
+            case rocisa::DataType::Int64:
+            {
+                auto typedPtr = static_cast<int64_t const*>(voidPtr);
+                return cast<Accumulator>(Transform<int64_t>::Input(typedPtr[pos], aConjugate));
+            }
+            break;
+            case rocisa::DataType::BFloat16:
             {
                 auto typedPtr = static_cast<BFloat16 const*>(voidPtr);
                 return cast<Accumulator>(Transform<BFloat16>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::Int8:
+            case rocisa::DataType::Int8:
             {
                 auto typedPtr = static_cast<int8_t const*>(voidPtr);
                 return cast<Accumulator>(Transform<int8_t>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::Float8:
+            case rocisa::DataType::Float8:
             {
                 auto typedPtr = static_cast<Float8 const*>(voidPtr);
                 return cast<Accumulator>(Transform<Float8>::Input(typedPtr[pos], aConjugate));
             }
             break;
-            case DataType::BFloat8:
+            case rocisa::DataType::BFloat8:
             {
                 auto typedPtr = static_cast<BFloat8 const*>(voidPtr);
                 return cast<Accumulator>(Transform<BFloat8>::Input(typedPtr[pos], aConjugate));
             }
+            case rocisa::DataType::Float8_fnuz:
+            {
+                auto typedPtr = static_cast<Float8_fnuz const*>(voidPtr);
+                return cast<Accumulator>(Transform<Float8_fnuz>::Input(typedPtr[pos], aConjugate));
+            }
             break;
-            case DataType::XFloat32:
-            case DataType::ComplexFloat:
-            case DataType::ComplexDouble:
-            case DataType::Int8x4:
-            case DataType::Count:
-            case DataType::Float8BFloat8:
-            case DataType::BFloat8Float8:;
+            case rocisa::DataType::BFloat8_fnuz:
+            {
+                auto typedPtr = static_cast<BFloat8_fnuz const*>(voidPtr);
+                return cast<Accumulator>(Transform<BFloat8_fnuz>::Input(typedPtr[pos], aConjugate));
+            }
+            break;
+            case rocisa::DataType::XFloat32:
+            case rocisa::DataType::ComplexFloat:
+            case rocisa::DataType::ComplexDouble:
+            case rocisa::DataType::Int8x4:
+            case rocisa::DataType::Count:
+            case rocisa::DataType::Float8BFloat8:
+            case rocisa::DataType::BFloat8Float8:
+            case rocisa::DataType::Float8BFloat8_fnuz:
+            case rocisa::DataType::BFloat8Float8_fnuz:;
             }
             return DataInitialization::getValue<Accumulator, InitMode::Zero>();
         }
@@ -253,11 +290,14 @@ namespace TensileLite
                                     && !std::is_same<double, Accumulator>::value
                                     && !std::is_same<BFloat16, Accumulator>::value
                                     && !std::is_same<int32_t, Accumulator>::value
+                                    && !std::is_same<int64_t, Accumulator>::value
                                     && !std::is_same<int8_t, Accumulator>::value
                                     && !std::is_same<Float8, Accumulator>::value
-                                    && !std::is_same<BFloat8, Accumulator>::value,
+                                    && !std::is_same<BFloat8, Accumulator>::value
+                                    && !std::is_same<Float8_fnuz, Accumulator>::value
+                                    && !std::is_same<BFloat8_fnuz, Accumulator>::value,
                                 Accumulator>::type
-            GetValue(DataType biasType, void const* biasptr, int pos, bool aConjugate)
+            GetValue(rocisa::DataType biasType, void const* biasptr, int pos, bool aConjugate)
         {
             return DataInitialization::getValue<Accumulator, InitMode::Zero>();
         }
@@ -267,59 +307,88 @@ namespace TensileLite
                                        || std::is_same<float, Accumulator>::value
                                        || std::is_same<double, Accumulator>::value
                                        || std::is_same<BFloat16, Accumulator>::value
+                                       || std::is_same<Float8, Accumulator>::value
+                                       || std::is_same<BFloat8, Accumulator>::value
+                                       || std::is_same<Float8_fnuz, Accumulator>::value
+                                       || std::is_same<BFloat8_fnuz, Accumulator>::value
                                        || std::is_same<int32_t, Accumulator>::value
+                                       || std::is_same<int64_t, Accumulator>::value
                                        || std::is_same<int8_t, Accumulator>::value,
-                                   bool>
-                  = true>
-        void SetValue(DataType dataType, Accumulator& src, void* dstPtr, size_t pos)
+                                   bool> = true>
+        void SetValue(rocisa::DataType dataType, Accumulator& src, void* dstPtr, size_t pos)
         {
             switch(dataType)
             {
-            case DataType::Float:
+            case rocisa::DataType::Float:
             {
                 auto typedPtr = static_cast<float*>(dstPtr);
                 typedPtr[pos] = SaturateCast<float>(src);
             }
             break;
-            case DataType::Double:
+            case rocisa::DataType::Double:
             {
                 auto typedPtr = static_cast<double*>(dstPtr);
                 typedPtr[pos] = SaturateCast<double>(src);
             }
             break;
-            case DataType::Half:
+            case rocisa::DataType::Half:
             {
                 auto typedPtr = static_cast<Half*>(dstPtr);
                 typedPtr[pos] = SaturateCast<Half>(src);
             }
             break;
-            case DataType::Int32:
+            case rocisa::DataType::Int32:
             {
                 auto typedPtr = static_cast<int32_t*>(dstPtr);
                 typedPtr[pos] = SaturateCast<int32_t>(src);
             }
             break;
-            case DataType::BFloat16:
+            case rocisa::DataType::BFloat16:
             {
                 auto typedPtr = static_cast<BFloat16*>(dstPtr);
                 typedPtr[pos] = SaturateCast<BFloat16>(src);
             }
             break;
-            case DataType::Int8:
+            case rocisa::DataType::Int8:
             {
                 auto typedPtr = static_cast<int8_t*>(dstPtr);
                 typedPtr[pos] = SaturateCast<int8_t>(src);
             }
             break;
-            case DataType::XFloat32:
-            case DataType::ComplexFloat:
-            case DataType::ComplexDouble:
-            case DataType::Int8x4:
-            case DataType::Float8:
-            case DataType::BFloat8:
-            case DataType::Count:
-            case DataType::Float8BFloat8:
-            case DataType::BFloat8Float8:;
+            case rocisa::DataType::Float8:
+            {
+                auto typedPtr = static_cast<Float8*>(dstPtr);
+                typedPtr[pos] = SaturateCast<Float8>(src);
+            }
+            break;
+            case rocisa::DataType::BFloat8:
+            {
+                auto typedPtr = static_cast<BFloat8*>(dstPtr);
+                typedPtr[pos] = SaturateCast<BFloat8>(src);
+            }
+            break;
+            case rocisa::DataType::Float8_fnuz:
+            {
+                auto typedPtr = static_cast<Float8_fnuz*>(dstPtr);
+                typedPtr[pos] = SaturateCast<Float8_fnuz>(src);
+            }
+            break;
+            case rocisa::DataType::BFloat8_fnuz:
+            {
+                auto typedPtr = static_cast<BFloat8_fnuz*>(dstPtr);
+                typedPtr[pos] = SaturateCast<BFloat8_fnuz>(src);
+            }
+            break;
+            case rocisa::DataType::XFloat32:
+            case rocisa::DataType::ComplexFloat:
+            case rocisa::DataType::ComplexDouble:
+            case rocisa::DataType::Int8x4:
+            case rocisa::DataType::Int64:
+            case rocisa::DataType::Count:
+            case rocisa::DataType::Float8BFloat8:
+            case rocisa::DataType::BFloat8Float8:
+            case rocisa::DataType::Float8BFloat8_fnuz:
+            case rocisa::DataType::BFloat8Float8_fnuz:;
             }
         }
 
@@ -329,36 +398,43 @@ namespace TensileLite
                                        && !std::is_same<double, Accumulator>::value
                                        && !std::is_same<BFloat16, Accumulator>::value
                                        && !std::is_same<int32_t, Accumulator>::value
+                                       && !std::is_same<int64_t, Accumulator>::value
                                        && !std::is_same<int8_t, Accumulator>::value
                                        && !std::is_same<Float8, Accumulator>::value
-                                       && !std::is_same<BFloat8, Accumulator>::value,
-                                   bool>
-                  = true>
-        void SetValue(DataType dataType, Accumulator& src, void* dstPtr, size_t pos)
+                                       && !std::is_same<BFloat8, Accumulator>::value
+                                       && !std::is_same<Float8_fnuz, Accumulator>::value
+                                       && !std::is_same<BFloat8_fnuz, Accumulator>::value,
+                                   bool> = true>
+        void SetValue(rocisa::DataType dataType, Accumulator& src, void* dstPtr, size_t pos)
         {
             switch(dataType)
             {
-            case DataType::Float:
-            case DataType::Double:
-            case DataType::Half:
-            case DataType::Int32:
-            case DataType::BFloat16:
-            case DataType::Int8:
-            case DataType::Float8:
-            case DataType::BFloat8:
-            case DataType::XFloat32:
+            case rocisa::DataType::Float:
+            case rocisa::DataType::Double:
+            case rocisa::DataType::Half:
+            case rocisa::DataType::Int32:
+            case rocisa::DataType::Int64:
+            case rocisa::DataType::BFloat16:
+            case rocisa::DataType::Int8:
+            case rocisa::DataType::Float8:
+            case rocisa::DataType::BFloat8:
+            case rocisa::DataType::Float8_fnuz:
+            case rocisa::DataType::BFloat8_fnuz:
+            case rocisa::DataType::XFloat32:
                 break;
-            case DataType::ComplexFloat:
-            case DataType::ComplexDouble:
+            case rocisa::DataType::ComplexFloat:
+            case rocisa::DataType::ComplexDouble:
                 break;
-            case DataType::Int8x4:
+            case rocisa::DataType::Int8x4:
             {
                 throw std::runtime_error("Not supported yet.");
             }
             break;
-            case DataType::Count:
-            case DataType::Float8BFloat8:
-            case DataType::BFloat8Float8:;
+            case rocisa::DataType::Count:
+            case rocisa::DataType::Float8BFloat8:
+            case rocisa::DataType::BFloat8Float8:
+            case rocisa::DataType::Float8BFloat8_fnuz:
+            case rocisa::DataType::BFloat8Float8_fnuz:;
             }
         }
 
@@ -374,7 +450,7 @@ namespace TensileLite
             // Only cast to float in BFloat16
             constexpr bool needCast = std::is_same<BFloat16, T>();
             using castT             = std::conditional_t<needCast, float, T>;
-            const auto isForAll = activationType == ActivationType::All
+            const auto isForAll     = activationType == ActivationType::All
                                   || activationType == ActivationType::Hipblaslt_all;
             auto new_type = isForAll ? activationType2 : activationType;
             if(new_type == ActivationType::Abs)
@@ -454,6 +530,14 @@ namespace TensileLite
                 auto castedVal = static_cast<castT>(val);
                 return static_cast<T>(castedVal / (1.f + static_cast<castT>(exp(-castedVal))));
             }
+            else if(new_type == ActivationType::Swish)
+            {
+                auto castedVal = static_cast<castT>(val);
+                return static_cast<T>(castedVal
+                                      / (1.f
+                                         + static_cast<castT>(exp(-multiply<castT>(
+                                             castedVal, static_cast<castT>(args[0]))))));
+            }
             return val;
         }
 
@@ -465,8 +549,8 @@ namespace TensileLite
                        ActivationType activationType2,
                        std::vector<T> args)
         {
-            const auto isForAll = activationType == ActivationType::All ||
-                                  activationType == ActivationType::Hipblaslt_all;
+            const auto isForAll = activationType == ActivationType::All
+                                  || activationType == ActivationType::Hipblaslt_all;
             auto new_type = isForAll ? activationType2 : activationType;
             if(new_type == ActivationType::Abs)
             {
@@ -516,9 +600,10 @@ namespace TensileLite
                 std::is_same<Half, Input>::value || std::is_same<float, Input>::value
                     || std::is_same<double, Input>::value || std::is_same<BFloat16, Input>::value
                     || std::is_same<int32_t, Input>::value || std::is_same<int8_t, Input>::value
-                    || std::is_same<Float8, Input>::value || std::is_same<BFloat8, Input>::value,
-                bool>
-            = true>
+                    || std::is_same<Float8, Input>::value || std::is_same<BFloat8, Input>::value
+                    || std::is_same<Float8_fnuz, Input>::value
+                    || std::is_same<BFloat8_fnuz, Input>::value,
+                bool> = true>
         std::string ReductionCPU(TensorDescriptor const&  biasTensor,
                                  TensorDescriptor const&  tensor,
                                  void const*              src,
@@ -535,7 +620,7 @@ namespace TensileLite
             // For 2D bias reduction, d batch = 1
             if((tensor.dimensions() == 3 && tensor.sizes()[2] == 1) || tensor.dimensions() == 2)
             {
-omp_set_num_threads(MAX_OMP_THREADS);
+                omp_set_num_threads(MAX_OMP_THREADS);
 #pragma omp parallel for
                 for(size_t bNum = 0; bNum < biasTensor.totalLogicalElements();
                     bNum += validationStride)
@@ -582,9 +667,10 @@ omp_set_num_threads(MAX_OMP_THREADS);
                 !std::is_same<Half, Input>::value && !std::is_same<float, Input>::value
                     && !std::is_same<double, Input>::value && !std::is_same<BFloat16, Input>::value
                     && !std::is_same<int32_t, Input>::value && !std::is_same<int8_t, Input>::value
-                    && !std::is_same<Float8, Input>::value && !std::is_same<BFloat8, Input>::value,
-                bool>
-            = true>
+                    && !std::is_same<Float8, Input>::value && !std::is_same<BFloat8, Input>::value
+                    && !std::is_same<Float8_fnuz, Input>::value
+                    && !std::is_same<BFloat8_fnuz, Input>::value,
+                bool> = true>
         std::string ReductionCPU(TensorDescriptor const&  biasTensor,
                                  TensorDescriptor const&  tensor,
                                  void const*              src,
@@ -687,7 +773,7 @@ omp_set_num_threads(MAX_OMP_THREADS);
                                            && !std::is_same<Accumulator, std::complex<float>>();
 
             // gemm
-omp_set_num_threads(MAX_OMP_THREADS);
+            omp_set_num_threads(MAX_OMP_THREADS);
 #pragma omp parallel for
             for(size_t dNum = 0; dNum < d.totalLogicalElements(); dNum += validationStrideGemm)
             {
@@ -778,20 +864,37 @@ omp_set_num_threads(MAX_OMP_THREADS);
                                          && sizeof(typename Inputs::BType)
                                                 > sizeof(typename Inputs::ComputeInputType))
                             {
-                                if(std::is_same<Float8BFloat8,
-                                                typename Inputs::ComputeInputType>::value)
+                                if constexpr(std::is_same<Float8BFloat8,
+                                                          typename Inputs::ComputeInputType>::value)
                                 {
                                     auto aValCast = static_cast<TensileLite::Float8>(aVal);
                                     auto bValCast = static_cast<TensileLite::BFloat8>(bVal);
                                     value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
                                 }
-                                else if(std::is_same<BFloat8Float8,
-                                                     typename Inputs::ComputeInputType>::value)
+                                else if constexpr(std::is_same<
+                                                      BFloat8Float8,
+                                                      typename Inputs::ComputeInputType>::value)
                                 {
                                     auto aValCast = static_cast<TensileLite::BFloat8>(aVal);
                                     auto bValCast = static_cast<TensileLite::Float8>(bVal);
                                     value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
                                 }
+                                else if constexpr(std::is_same<
+                                                      Float8BFloat8_fnuz,
+                                                      typename Inputs::ComputeInputType>::value)
+                                {
+                                    auto aValCast = static_cast<TensileLite::Float8_fnuz>(aVal);
+                                    auto bValCast = static_cast<TensileLite::BFloat8_fnuz>(bVal);
+                                    value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
+                                }
+                                else if constexpr(std::is_same<
+                                                      BFloat8Float8_fnuz,
+                                                      typename Inputs::ComputeInputType>::value)
+                                {
+                                    auto aValCast = static_cast<TensileLite::BFloat8_fnuz>(aVal);
+                                    auto bValCast = static_cast<TensileLite::Float8_fnuz>(bVal);
+                                    value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
+                                }
                                 else
                                 {
                                     typename Inputs::ComputeInputType aValCast, bValCast;
@@ -983,7 +1086,7 @@ omp_set_num_threads(MAX_OMP_THREADS);
                                          actArgs);
                 }
 
-omp_set_num_threads(MAX_OMP_THREADS);
+                omp_set_num_threads(MAX_OMP_THREADS);
 #pragma omp critical
                 {
                     if constexpr(notCmplxAmaxD)
@@ -1076,7 +1179,7 @@ omp_set_num_threads(MAX_OMP_THREADS);
             }
         }
 
-        uint32_t getInputContractionInputsTypeId(ContractionProblemGemm const& problem)
+        uint64_t getInputContractionInputsTypeId(ContractionProblemGemm const& problem)
         {
             // retreive alpha/beta type set via setAlpha/BetaType()
             auto alphaType = problem.alphaType();
@@ -1084,12 +1187,13 @@ omp_set_num_threads(MAX_OMP_THREADS);
 
             // Backward-compatible: when setAlpha/BetaType() wasn't called, use the old way
             // Could remove after rocBLAS is updated
-            if(alphaType == DataType::None)
+            if(alphaType == rocisa::DataType::None)
             {
-                alphaType = problem.a().dataType() == DataType::BFloat16 ? DataType::Float
-                                                                         : problem.d().dataType();
+                alphaType = problem.a().dataType() == rocisa::DataType::BFloat16
+                                ? rocisa::DataType::Float
+                                : problem.d().dataType();
             }
-            if(betaType == DataType::None)
+            if(betaType == rocisa::DataType::None)
             {
                 betaType = alphaType;
             }
@@ -1103,16 +1207,16 @@ omp_set_num_threads(MAX_OMP_THREADS);
             }
 
             return TensileLite::GemmTypeId(problem.a().dataType(),
-                                       problem.b().dataType(),
-                                       problem.c().dataType(),
-                                       problem.d().dataType(),
-                                       alphaType,
-                                       betaType,
-                                       problem.computeInputType());
+                                           problem.b().dataType(),
+                                           problem.c().dataType(),
+                                           problem.d().dataType(),
+                                           alphaType,
+                                           betaType,
+                                           problem.computeInputType());
         }
 
         template <typename Problem, typename Inputs>
-        void SolveCPUTemplates(uint32_t const& contractionInputsTypeId,
+        void SolveCPUTemplates(uint64_t const& contractionInputsTypeId,
                                Problem const&  problem,
                                Inputs const&   inputs,
                                size_t          elementsToValidate)
@@ -1131,7 +1235,7 @@ omp_set_num_threads(MAX_OMP_THREADS);
             {
             case TypedGemm_S_S_S::TypeId():
             {
-                if(problem.f32XdlMathOp() == DataType::XFloat32)
+                if(problem.f32XdlMathOp() == rocisa::DataType::XFloat32)
                     return ReferenceSolution<TypedGemm_S_S_S, float, XFloat32>::SolveCPU(
                         problem, inputs, elementsToValidate);
                 else
@@ -1373,6 +1477,119 @@ omp_set_num_threads(MAX_OMP_THREADS);
                 return ReferenceSolution<TypedGemm_H_B8F8_H_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+
+            // F8 NANOO
+            case TypedGemm_F8N_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8N_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8N_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8N_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8N_F8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8N_F8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8N_B8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8N_B8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8N_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8N_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8N_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8N_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8N_F8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8N_F8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8N_B8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8N_B8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            // hybrid - NANOO
+            case TypedGemm_F8B8N_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8N_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8B8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8B8N_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8N_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8B8N_F8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8N_F8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8B8N_B8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8N_B8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8F8N_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8N_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8F8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8F8N_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8N_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8F8N_F8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8N_F8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8F8N_B8N_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8N_B8N_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_F8B8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_F8B8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_B8F8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_B8F8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
 #ifdef TENSILE_USE_HALF
             case TypedGemm_H_F8_H_S::TypeId():
             {
@@ -1444,6 +1661,79 @@ omp_set_num_threads(MAX_OMP_THREADS);
                 return ReferenceSolution<TypedGemm_F8H_FP8_FP8_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+
+            // F8 NANOO
+            case TypedGemm_H_F8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_F8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_B8N_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_B8N_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_HF8N_H_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_H_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_H_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_H_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_HF8N_H_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_H_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_H_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_H_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            // TODO:; why FP8, not F8... need to change it to FP8N???
+            case TypedGemm_HF8N_H_FP8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_H_FP8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_H_FP8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_H_FP8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_HF8N_FP8_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_FP8_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_FP8_S_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_FP8_S_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_HF8N_FP8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_FP8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_FP8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_FP8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_HF8N_FP8_FP8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_HF8N_FP8_FP8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_F8NH_FP8_FP8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8NH_FP8_FP8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
 #endif // TENSILE_USE_HALF
 #endif // TENSILE_USE_FP8_BF8
 
