@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@
 #include <Tensile/TensorDescriptor.hpp>
 #include <Tensile/Utils.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     /**
  * \addtogroup User defined parameters
@@ -48,12 +48,12 @@ namespace Tensile
     class TENSILE_API ContractionProblemParameters
     {
     public:
-        void setGSU(uint16_t gsu)
+        void setGSU(int16_t gsu)
         {
             m_gsu = gsu;
         }
 
-        uint8_t gsu() const
+        int16_t gsu() const
         {
             return m_gsu;
         }
@@ -88,12 +88,32 @@ namespace Tensile
             return m_wgm;
         }
 
-        void setBiasEnum(DataType dataType)
+        void setWGMXCC(uint16_t wgmxcc)
+        {
+            m_wgmxcc = wgmxcc;
+        }
+
+        uint16_t wgmxcc() const
+        {
+            return m_wgmxcc;
+        }
+
+        void setWGMXCCG(int16_t wgmxccg)
+        {
+            m_wgmxccg = wgmxccg;
+        }
+
+        int16_t wgmxccg() const
+        {
+            return m_wgmxccg;
+        }
+
+        void setBiasEnum(rocisa::DataType dataType)
         {
             m_biasType = dataType;
         }
 
-        DataType biasEnum() const
+        rocisa::DataType biasEnum() const
         {
             return m_biasType;
         }
@@ -123,14 +143,28 @@ namespace Tensile
             m_gsu = 0;
         }
 
+        // Fallback means if the problem is running with a CU-fallback sol.
+        void setFallbackStatus(bool cuFallback)
+        {
+            m_fallbackStatus = cuFallback;
+        }
+
+        bool fallbackStatus() const
+        {
+            return m_fallbackStatus;
+        }
+
     private:
-        uint16_t       m_gsu            = 0; // default value
-        bool           m_gsuc           = false; // default value
-        bool           m_gsuwgmrr       = false; // default value
-        int16_t        m_wgm            = 0; // default value
-        DataType       m_biasType       = DataType::None;
-        int            m_factorDim      = 0;
-        ActivationType m_activationType = ActivationType::None;
+        int16_t          m_gsu            = 0; // default value
+        bool             m_gsuc           = false; // default value
+        bool             m_gsuwgmrr       = false; // default value
+        int16_t          m_wgm            = 0; // default value
+        uint16_t         m_wgmxcc         = 0; // default value
+        int16_t          m_wgmxccg        = 0; // default value
+        rocisa::DataType m_biasType       = rocisa::DataType::None;
+        int              m_factorDim      = 0;
+        ActivationType   m_activationType = ActivationType::None;
+        bool             m_fallbackStatus = false; // default value
     };
 
     /**
@@ -141,8 +175,8 @@ namespace Tensile
 
     struct ConstantDescriptor
     {
-        std::string name;
-        DataType    dataType;
+        std::string      name;
+        rocisa::DataType dataType;
     };
 
     class ContractionProblem : public Problem
@@ -167,7 +201,7 @@ namespace Tensile
         }
 
         virtual void resetTensor(int                           idx,
-                                 DataType                      type,
+                                 rocisa::DataType              type,
                                  std::initializer_list<size_t> sizes,
                                  std::initializer_list<size_t> strides)
         {
@@ -201,22 +235,22 @@ namespace Tensile
             return m_workspaceSizeGroupedGemm;
         }
 
-        void setF32XdlMathOp(DataType value)
+        void setF32XdlMathOp(rocisa::DataType value)
         {
             m_f32XdlMathOp = value;
         }
 
-        DataType f32XdlMathOp() const
+        rocisa::DataType f32XdlMathOp() const
         {
             return m_f32XdlMathOp;
         }
 
-        void setComputeInputType(DataType value)
+        void setComputeInputType(rocisa::DataType value)
         {
             m_computeInputType = value;
         }
 
-        DataType computeInputType() const
+        rocisa::DataType computeInputType() const
         {
             return m_computeInputType;
         }
@@ -239,8 +273,8 @@ namespace Tensile
         size_t m_workspaceSize            = std::numeric_limits<size_t>::max();
         size_t m_workspaceSizeGroupedGemm = std::numeric_limits<size_t>::max();
 
-        DataType m_f32XdlMathOp;
-        DataType m_computeInputType;
+        rocisa::DataType m_f32XdlMathOp;
+        rocisa::DataType m_computeInputType;
 
         bool m_useDeviceUserArguments = false;
     };
@@ -270,6 +304,7 @@ namespace Tensile
             METADATA      = 11,
             Synchronizer  = 12,
             AMAXD         = 13,
+            COMPRESSED    = 14,
             TENSOR_COUNT
         };
 
@@ -330,25 +365,25 @@ namespace Tensile
    * Create a ContractionProblemGemm representing a batched GEMM, specifying
    * strides between matrices.
    */
-        static ContractionProblemGemm GEMM_Strides(bool     transA,
-                                                   bool     transB,
-                                                   DataType aType,
-                                                   DataType bType,
-                                                   DataType cType,
-                                                   DataType dType,
-                                                   size_t   m,
-                                                   size_t   n,
-                                                   size_t   k,
-                                                   size_t   batchSize,
-                                                   size_t   lda,
-                                                   size_t   aStride,
-                                                   size_t   ldb,
-                                                   size_t   bStride,
-                                                   size_t   ldc,
-                                                   size_t   cStride,
-                                                   size_t   ldd,
-                                                   size_t   dStride,
-                                                   double   beta);
+        static ContractionProblemGemm GEMM_Strides(bool             transA,
+                                                   bool             transB,
+                                                   rocisa::DataType aType,
+                                                   rocisa::DataType bType,
+                                                   rocisa::DataType cType,
+                                                   rocisa::DataType dType,
+                                                   size_t           m,
+                                                   size_t           n,
+                                                   size_t           k,
+                                                   size_t           batchSize,
+                                                   size_t           lda,
+                                                   size_t           aStride,
+                                                   size_t           ldb,
+                                                   size_t           bStride,
+                                                   size_t           ldc,
+                                                   size_t           cStride,
+                                                   size_t           ldd,
+                                                   size_t           dStride,
+                                                   double           beta);
 
         /**
    * Create a ContractionProblemGemm representing a batched SGEMM, with
@@ -441,13 +476,13 @@ namespace Tensile
                                                      BatchIndices const&        batchIndices,
                                                      BoundIndices const&        boundIndices,
                                                      std::vector<size_t> const& indexSizes,
-                                                     DataType                   aType,
+                                                     rocisa::DataType           aType,
                                                      std::vector<size_t> const& aStrides,
-                                                     DataType                   bType,
+                                                     rocisa::DataType           bType,
                                                      std::vector<size_t> const& bStrides,
-                                                     DataType                   cType,
+                                                     rocisa::DataType           cType,
                                                      std::vector<size_t> const& cStrides,
-                                                     DataType                   dType,
+                                                     rocisa::DataType           dType,
                                                      std::vector<size_t> const& dStrides,
                                                      double                     beta);
 
@@ -479,13 +514,13 @@ namespace Tensile
    */
         static ContractionProblemGemm FromIndexSizes(std::string const&         operationIdentifier,
                                                      std::vector<size_t> const& indexSizes,
-                                                     DataType                   aType,
+                                                     rocisa::DataType           aType,
                                                      std::vector<size_t> const& aStrides,
-                                                     DataType                   bType,
+                                                     rocisa::DataType           bType,
                                                      std::vector<size_t> const& bStrides,
-                                                     DataType                   cType,
+                                                     rocisa::DataType           cType,
                                                      std::vector<size_t> const& cStrides,
-                                                     DataType                   dType,
+                                                     rocisa::DataType           dType,
                                                      std::vector<size_t> const& dStrides,
                                                      double                     beta);
 
@@ -565,12 +600,12 @@ namespace Tensile
             return m_cEqualsD;
         }
 
-        void setAlphaType(DataType type)
+        void setAlphaType(rocisa::DataType type)
         {
             m_alphaType = type;
         }
 
-        DataType alphaType() const
+        rocisa::DataType alphaType() const
         {
             return m_alphaType;
         }
@@ -585,17 +620,17 @@ namespace Tensile
             return m_alphaRestriction;
         }
 
-        void setBetaType(DataType type)
+        void setBetaType(rocisa::DataType type)
         {
             m_betaType = type;
         }
 
-        DataType betaType() const
+        rocisa::DataType betaType() const
         {
             return m_betaType;
         }
 
-        DataType computeType() const
+        rocisa::DataType computeType() const
         {
             return m_betaType;
         }
@@ -665,12 +700,13 @@ namespace Tensile
             return m_useScaleAlphaVec;
         }
 
-        void setE(DataType                   type,
+        void setE(rocisa::DataType           type,
                   std::vector<size_t> const& sizes,
                   std::vector<size_t> const& strides,
                   bool                       isOutput = false)
         {
-            if(type != DataType::None && m_useE)
+            m_auxType = type;
+            if(type != rocisa::DataType::None && m_useE)
             {
                 // Currently only supports offset = 0
                 m_tensors[ContractionProblemGemm::TENSOR::E]
@@ -679,7 +715,7 @@ namespace Tensile
             }
         }
 
-        void setBias(DataType                       type,
+        void setBias(rocisa::DataType               type,
                      size_t                         length,
                      size_t                         stride,
                      bool                           isOutput  = false,
@@ -689,7 +725,7 @@ namespace Tensile
             setParams().setBiasEnum(type);
             setParams().setFactorDim(factorDim);
             m_biasSrc = src;
-            if(type != DataType::None && m_useBias)
+            if(type != rocisa::DataType::None && m_useBias)
             {
                 size_t batchIdx = 2;
                 for(size_t j = 0; j < m_batchIndices.size(); j++)
@@ -726,50 +762,50 @@ namespace Tensile
             return m_biasSrc;
         }
 
-        void setScaleA(DataType type, size_t length)
+        void setScaleA(rocisa::DataType type, size_t length)
         {
             m_scaleAType = type;
-            if(type != DataType::None && !m_useScaleAB.empty())
+            if(type != rocisa::DataType::None && !m_useScaleAB.empty())
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEA]
                     = {"scaleA", m_scaleAType, {length}, {1, length}};
             }
         }
 
-        void setScaleB(DataType type, size_t length)
+        void setScaleB(rocisa::DataType type, size_t length)
         {
             m_scaleBType = type;
-            if(type != DataType::None && !m_useScaleAB.empty())
+            if(type != rocisa::DataType::None && !m_useScaleAB.empty())
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEB]
                     = {"scaleB", m_scaleBType, {length}, {1, length}};
             }
         }
 
-        void setScaleC(DataType type)
+        void setScaleC(rocisa::DataType type)
         {
             m_scaleCType = type;
-            if(type != DataType::None && m_useScaleCD)
+            if(type != rocisa::DataType::None && m_useScaleCD)
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEC]
                     = {"scaleC", m_scaleCType, {1}, {1, 1}};
             }
         }
 
-        void setScaleD(DataType type)
+        void setScaleD(rocisa::DataType type)
         {
             m_scaleDType = type;
-            if(type != DataType::None && m_useScaleCD)
+            if(type != rocisa::DataType::None && m_useScaleCD)
             {
                 m_tensors[ContractionProblemGemm::TENSOR::SCALED]
                     = {"scaleD", m_scaleDType, {1}, {1, 1}};
             }
         }
 
-        void setScaleAlphaVec(DataType type, size_t length, int factorDim = 0)
+        void setScaleAlphaVec(rocisa::DataType type, size_t length, int factorDim = 0)
         {
             m_scaleAlphaVecType = type;
-            if(type != DataType::None && m_useScaleAlphaVec)
+            if(type != rocisa::DataType::None && m_useScaleAlphaVec)
             {
                 setParams().setFactorDim(factorDim);
                 m_tensors[ContractionProblemGemm::TENSOR::SCALEALPHAVEC]
@@ -777,18 +813,18 @@ namespace Tensile
             }
         }
 
-        void setAmaxD(DataType type, bool isOutput = false)
+        void setAmaxD(rocisa::DataType type, bool isOutput = false)
         {
-            if(type != DataType::None && m_outputAmaxD)
+            if(type != rocisa::DataType::None && m_outputAmaxD)
             {
                 m_tensors[ContractionProblemGemm::TENSOR::AMAXD] = {"amaxD", type, {1}, {1, 1}};
                 m_tensors[ContractionProblemGemm::TENSOR::AMAXD].setAsOutput(isOutput);
             }
         }
 
-        void setSynchronizer(DataType type, size_t length)
+        void setSynchronizer(rocisa::DataType type, size_t length)
         {
-            if(type != DataType::None)
+            if(type != rocisa::DataType::None)
             {
                 m_tensors[ContractionProblemGemm::TENSOR::Synchronizer]
                     = {"Synchronizer", type, {length}, {1, length}};
@@ -872,8 +908,24 @@ namespace Tensile
 
         PerformanceMetric performanceMetric() const
         {
-            const bool experimental = Debug::Instance().useExperimentalSelection();
-            return experimental ? PerformanceMetric::Experimental : m_performanceMetric;
+            const int experimental = Debug::Instance().useExperimentalSelection();
+            auto      option       = static_cast<ExperimentalOption>(experimental);
+
+            switch(option)
+            {
+            case ExperimentalOption::None:
+                return m_performanceMetric;
+
+            case ExperimentalOption::MLP:
+                return PerformanceMetric::ExperimentalMLP;
+
+            case ExperimentalOption::StreamK:
+                return PerformanceMetric::ExperimentalStreamK;
+
+            default:
+                // warning?
+                return m_performanceMetric;
+            }
         }
 
         void setDeterministicMode(bool value)
@@ -905,12 +957,12 @@ namespace Tensile
             return m_activationType;
         }
 
-        void setActivationComputeType(DataType value)
+        void setActivationComputeType(rocisa::DataType value)
         {
             m_activationComputeType = value;
         }
 
-        DataType activationComputeType() const
+        rocisa::DataType activationComputeType() const
         {
             return m_activationComputeType;
         }
@@ -942,6 +994,26 @@ namespace Tensile
             return m_maxProblemSize;
         }
 
+        bool swizzleTensorA() const
+        {
+            return m_swizzleTensorA;
+        }
+
+        bool swizzleTensorB() const
+        {
+            return m_swizzleTensorB;
+        }
+
+        void setSwizzleTensorA(bool swizzle)
+        {
+            m_swizzleTensorA = swizzle;
+        }
+
+        void setSwizzleTensorB(bool swizzle)
+        {
+            m_swizzleTensorB = swizzle;
+        }
+
         /// Allocated elements excluding batch dimensions
         /// Used in assembly kernels to determine buffer limits, if batch dimes not
         /// packed
@@ -953,9 +1025,9 @@ namespace Tensile
         {
             return m_allocatedElementsNonBatchB;
         }
-        size_t allocatedElementsNonBatchCompressedA() const
+        size_t allocatedElementsNonBatchCompressed() const
         {
-            return m_allocatedElementsNonBatchCompressedA;
+            return m_allocatedElementsNonBatchCompressed;
         }
 
         size_t flopsPerMac() const;
@@ -977,9 +1049,13 @@ namespace Tensile
         {
             return m_tensors[ContractionProblemGemm::TENSOR::D];
         }
+        TensorDescriptor const& e() const
+        {
+            return m_tensors[ContractionProblemGemm::TENSOR::E];
+        }
         TensorDescriptor const& compressed() const
         {
-            return m_tensor_compressed;
+            return m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED];
         }
         TensorDescriptor const& metadata() const
         {
@@ -989,6 +1065,10 @@ namespace Tensile
         {
             return m_tensors[ContractionProblemGemm::TENSOR::BIAS];
         }
+        TensorDescriptor const& scaleAlphaVec() const
+        {
+            return m_tensors[ContractionProblemGemm::TENSOR::SCALEALPHAVEC];
+        }
         TensorDescriptor const& amaxd() const
         {
             return m_tensors[ContractionProblemGemm::TENSOR::AMAXD];
@@ -1064,6 +1144,9 @@ namespace Tensile
             return getOperationDescription();
         }
 
+        size_t getNumTiles(SizeMapping const& sizeMapping, size_t gsu) const;
+        size_t getItersPerTile(SizeMapping const& sizeMapping) const;
+
         void checkPersistentKernelEligibility(ContractionSolution const& solution,
                                               Hardware const&            hardware);
 
@@ -1072,6 +1155,11 @@ namespace Tensile
             return m_eligibleForPK;
         }
 
+        double arithmeticIntensity() const
+        {
+            return m_arithmeticIntensity;
+        }
+
         virtual std::vector<ConstantDescriptor> const constants() const
         {
             std::vector<ConstantDescriptor> c = {{"alpha", m_alphaType}, {"beta", m_betaType}};
@@ -1092,7 +1180,7 @@ namespace Tensile
             }
             // Push the rest of the args even unused.
             for(; i < numAll; i++)
-                c.push_back({s[i], DataType::None});
+                c.push_back({s[i], rocisa::DataType::None});
 
             return c;
         }
@@ -1104,24 +1192,24 @@ namespace Tensile
                            size_t              workspaceSize);
 
         static ContractionProblemGemm
-            createDefaultProblem(bool                   transA,
-                                 bool                   transB,
-                                 DataType               typeA,
-                                 DataType               typeB,
-                                 DataType               typeC,
-                                 DataType               typeD,
-                                 DataType               typeAlpha,
-                                 DataType               typeBeta,
-                                 DataType               typeComputeInput,
-                                 DataType               typeCompute,
-                                 double                 alpha,
-                                 double                 beta,
-                                 bool                   useBias,
-                                 bool                   useGradient,
-                                 std::vector<DataType>& biasDataTypeWhiteList,
-                                 std::vector<int>&      biasSrcWhiteList,
-                                 bool                   isGroupedGemm,
-                                 size_t                 maxWorkspaceBytes);
+            createDefaultProblem(bool                           transA,
+                                 bool                           transB,
+                                 rocisa::DataType               typeA,
+                                 rocisa::DataType               typeB,
+                                 rocisa::DataType               typeC,
+                                 rocisa::DataType               typeD,
+                                 rocisa::DataType               typeAlpha,
+                                 rocisa::DataType               typeBeta,
+                                 rocisa::DataType               typeComputeInput,
+                                 rocisa::DataType               typeCompute,
+                                 double                         alpha,
+                                 double                         beta,
+                                 bool                           useBias,
+                                 bool                           useGradient,
+                                 std::vector<rocisa::DataType>& biasDataTypeWhiteList,
+                                 std::vector<int>&              biasSrcWhiteList,
+                                 bool                           isGroupedGemm,
+                                 size_t                         maxWorkspaceBytes);
 
     private:
         std::string m_sumNames;
@@ -1129,35 +1217,44 @@ namespace Tensile
 
         ContractionProblemParameters m_params;
 
-        bool           m_cEqualsD                = false;
-        bool           m_stridedBatched          = true;
-        bool           m_groupedGemm             = false;
-        int            m_groupedGemmCount        = std::numeric_limits<int>::max();
-        bool           m_highPrecisionAccumulate = false;
-        bool           m_deterministicMode       = false;
-        bool           m_eligibleForPK           = true;
-        bool           m_useGradient             = false;
-        bool           m_useE                    = false;
-        bool           m_outputAmaxD             = false;
-        int            m_useBias                 = 0;
-        std::string    m_useScaleAB              = "";
-        bool           m_useScaleCD              = false;
-        int            m_useScaleAlphaVec        = 0;
-        ActivationType m_activationType          = ActivationType::None;
-        bool           m_activationNoGuard       = false;
-        int            m_sparse                  = 0;
+        bool             m_cEqualsD                = false;
+        bool             m_stridedBatched          = true;
+        bool             m_groupedGemm             = false;
+        int              m_groupedGemmCount        = std::numeric_limits<int>::max();
+        bool             m_highPrecisionAccumulate = false;
+        bool             m_deterministicMode       = false;
+        bool             m_eligibleForPK           = true;
+        bool             m_useGradient             = false;
+        bool             m_useE                    = false;
+        rocisa::DataType m_auxType                 = rocisa::DataType::None;
+        bool             m_outputAmaxD             = false;
+        bool             m_swizzleTensorA          = false;
+        bool             m_swizzleTensorB          = false;
+        int              m_useBias                 = 0;
+        std::string      m_useScaleAB              = "";
+        bool             m_useScaleCD              = false;
+        int              m_useScaleAlphaVec        = 0;
+        ActivationType   m_activationType          = ActivationType::None;
+        bool             m_activationNoGuard       = false;
+        int              m_sparse                  = 0;
 
         KernelLanguage    m_kernelLanguage    = KernelLanguage::Any;
         PerformanceMetric m_performanceMetric = PerformanceMetric::DeviceEfficiency;
-
-        DataType m_alphaType         = DataType::None; // if not assigned, will follow d-type
-        DataType m_betaType          = DataType::None; // for bwd-compatible
-        DataType m_scaleAType        = DataType::None; // if not assigned, will follow alpha-type
-        DataType m_scaleBType        = DataType::None; // if not assigned, will follow alpha-type
-        DataType m_scaleCType        = DataType::None; // if not assigned, will follow beta-type
-        DataType m_scaleDType        = DataType::None; // if not assigned, will follow beta-type
-        DataType m_scaleAlphaVecType = DataType::None; // if not assigned, will follow alpha-type
-        DataType m_activationComputeType = DataType::None;
+        double            m_arithmeticIntensity;
+        rocisa::DataType  m_alphaType
+            = rocisa::DataType::None; // if not assigned, will follow d-type
+        rocisa::DataType m_betaType = rocisa::DataType::None; // for bwd-compatible
+        rocisa::DataType m_scaleAType
+            = rocisa::DataType::None; // if not assigned, will follow alpha-type
+        rocisa::DataType m_scaleBType
+            = rocisa::DataType::None; // if not assigned, will follow alpha-type
+        rocisa::DataType m_scaleCType
+            = rocisa::DataType::None; // if not assigned, will follow beta-type
+        rocisa::DataType m_scaleDType
+            = rocisa::DataType::None; // if not assigned, will follow beta-type
+        rocisa::DataType m_scaleAlphaVecType
+            = rocisa::DataType::None; // if not assigned, will follow alpha-type
+        rocisa::DataType m_activationComputeType = rocisa::DataType::None;
 
         ContractionProblemGemm::TENSOR m_biasSrc = ContractionProblemGemm::TENSOR::D;
 
@@ -1185,12 +1282,11 @@ namespace Tensile
 
         size_t m_allocatedElementsNonBatchA;
         size_t m_allocatedElementsNonBatchB;
-        size_t m_allocatedElementsNonBatchCompressedA;
-
-        TensorDescriptor m_tensor_compressed;
+        size_t m_allocatedElementsNonBatchCompressed;
 
         void normalize();
         void normalizeSparse();
+        void calcArithmeticIntensity();
 
         void consistencyCheck() const;
 
@@ -1229,6 +1325,7 @@ namespace Tensile
                           void const*          _b,
                           void const*          _c,
                           void*                _d,
+                          void*                _e,
                           void const* const*   _batchA,
                           void const* const*   _batchB,
                           void const* const*   _batchC,
@@ -1242,7 +1339,15 @@ namespace Tensile
                           void const*          _scaleAlphaVec,
                           void*                _ws,
                           void*                _Synchronizer,
-                          unsigned char const* _metadata);
+                          unsigned char const* _metadata,
+                          void const*          _compressed);
+
+        ContractionInputs(void const*     _a,
+                          void const*     _b,
+                          void const*     _c,
+                          void*           _d,
+                          ConstantVariant _alpha,
+                          ConstantVariant _beta);
 
         // TODO: Remove this
         void const* a     = nullptr;
@@ -1265,6 +1370,9 @@ namespace Tensile
         void const* scaleD        = nullptr;
         void const* scaleAlphaVec = nullptr;
 
+        unsigned char const* metadata = nullptr;
+        void const* compressed        = nullptr;
+
         // Constants
         ConstantVariant              alpha = static_cast<float>(0);
         ConstantVariant              beta  = static_cast<float>(0);
@@ -1273,7 +1381,6 @@ namespace Tensile
         // Workspace
         void*                ws           = nullptr;
         void*                Synchronizer = nullptr;
-        unsigned char const* metadata     = nullptr;
 
         std::vector<size_t> maxElements;
         size_t              workspaceSize;
@@ -1317,4 +1424,4 @@ namespace Tensile
     /**
  * @}
  */
-} // namespace Tensile
+} // namespace TensileLite
