@@ -28,10 +28,13 @@
 
 #include <cstdlib>
 #include <string>
+#ifdef Tensile_ENABLE_MARKER
+#include <roctracer/roctx.h>
+#endif
 
 #include <Tensile/Singleton.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     /**
  * @brief Common place for defining flags which enable debug behaviour.
@@ -70,7 +73,9 @@ namespace Tensile
 
         bool enableDebugSelection() const;
 
-        bool useExperimentalSelection() const;
+        bool useStreamKDataParrallel() const;
+
+        int useExperimentalSelection() const;
 
         std::string getMetric() const;
 
@@ -82,10 +87,44 @@ namespace Tensile
 
         int getGridbasedTopSols() const;
 
+        bool printStreamKGridInfo() const;
+
         bool gridBasedKDTree() const;
 
         bool gridBasedBatchExp() const;
 
+        __attribute__((always_inline)) inline void markerStart(const char* name) const
+        {
+#ifdef Tensile_ENABLE_MARKER
+            if(m_printMarker)
+            {
+                roctxRangePush(name);
+            }
+#endif
+        }
+
+        __attribute__((always_inline)) inline void markerStart(const char*        name,
+                                                               const std::string& objPath) const
+        {
+#ifdef Tensile_ENABLE_MARKER
+            if(m_printMarker)
+            {
+                std::string s = name + std::string(": ") + objPath;
+                roctxRangePush(s.c_str());
+            }
+#endif
+        }
+
+        __attribute__((always_inline)) inline void markerStop() const
+        {
+#ifdef Tensile_ENABLE_MARKER
+            if(m_printMarker)
+            {
+                roctxRangePop();
+            }
+#endif
+        }
+
     private:
         friend LazySingleton<Debug>;
 
@@ -93,7 +132,8 @@ namespace Tensile
         int         m_value2;
         bool        m_naivePropertySearch = false;
         bool        m_debugSelection      = false;
-        bool        m_experimentSelection = false;
+        bool        m_dataParallel      = false;
+        int         m_experimentSelection = 0;
         int         m_solution_index      = -1;
         bool        m_solselTrace         = false;
         std::string m_metric              = "";
@@ -101,7 +141,8 @@ namespace Tensile
         bool        m_benchmark           = false;
         bool        m_gridbasedKdTree     = false;
         bool        m_gridbasedBatchExp   = false;
+        bool        m_printMarker         = false;
 
         Debug();
     };
-} // namespace Tensile
+} // namespace TensileLite
