@@ -31,7 +31,7 @@
 #include <Tensile/ContractionSolution.hpp>
 #include <Tensile/Serialization/Base.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Serialization
     {
@@ -58,6 +58,7 @@ namespace Tensile
 
                 iot::mapRequired(io, "hardwarePredicate", s.hardwarePredicate);
                 iot::mapRequired(io, "problemPredicate", s.problemPredicate);
+                iot::mapRequired(io, "taskPredicate", s.taskPredicate);
 
                 iot::mapRequired(io, "debugKernel", s.debugKernel);
                 iot::mapOptional(io, "libraryLogicIndex", s.libraryLogicIndex);
@@ -73,10 +74,10 @@ namespace Tensile
         };
 
         template <typename IO>
-        struct MappingTraits<ContractionSolution::SizeMapping, IO>
+        struct MappingTraits<SizeMapping, IO>
         {
             using iot = IOTraits<IO>;
-            static void mapping(IO& io, ContractionSolution::SizeMapping& s)
+            static void mapping(IO& io, SizeMapping& s)
             {
                 iot::mapRequired(io, "waveNum", s.waveNum);
 
@@ -100,6 +101,8 @@ namespace Tensile
                 iot::mapOptional(io, "packBatchDims", s.packBatchDims);
                 iot::mapOptional(io, "packSummationDims", s.packSummationDims);
                 iot::mapOptional(io, "magicDivAlg", s.magicDivAlg);
+                iot::mapOptional(io, "streamK", s.streamK);
+                iot::mapOptional(io, "streamKAtomic", s.streamKAtomic);
                 iot::mapOptional(io, "persistentKernel", s.persistentKernel);
                 iot::mapOptional(io, "persistentKernelAlongBatch", s.persistentKernelAlongBatch);
                 iot::mapRequired(io, "sourceKernel", s.sourceKernel);
@@ -113,8 +116,16 @@ namespace Tensile
                 iot::mapOptional(io, "CustomKernelName", s.customKernelName);
 
                 iot::mapRequired(io, "workGroupMappingXCC", s.workGroupMappingXCC);
+                iot::mapRequired(io, "workGroupMappingXCCGroup", s.workGroupMappingXCCGroup);
+
                 iot::mapRequired(io, "globalSplitUCoalesced", s.globalSplitUCoalesced);
-                iot::mapRequired(io, "globalSplitUWorkGroupMappingRoundRobin", s.globalSplitUWorkGroupMappingRoundRobin);
+                iot::mapRequired(io,
+                                 "globalSplitUWorkGroupMappingRoundRobin",
+                                 s.globalSplitUWorkGroupMappingRoundRobin);
+                iot::mapRequired(io, "CUOccupancy", s.CUOccupancy);
+                iot::mapRequired(io, "PrefetchGlobalRead", s.PrefetchGlobalRead);
+                iot::mapRequired(io, "MathClocksUnrolledLoop", s.MathClocksUnrolledLoop);
+                iot::mapRequired(io, "synchronizerSizePerWG", s.synchronizerSizePerWG);
             }
 
             const static bool flow = false;
@@ -175,6 +186,8 @@ namespace Tensile
                 iot::mapOptional(io, "sparse", s.sparse);
                 iot::mapOptional(io, "f32XdlMathOp", s.f32XdlMathOp);
                 iot::mapOptional(io, "supportDeviceUserArguments", s.supportDeviceUserArguments);
+                iot::mapOptional(io, "swizzleTensorA", s.swizzleTensorA);
+                iot::mapOptional(io, "swizzleTensorB", s.swizzleTensorB);
             }
 
             const static bool flow = false;
@@ -209,4 +222,4 @@ namespace Tensile
             const static bool flow = false;
         };
     } // namespace Serialization
-} // namespace Tensile
+} // namespace TensileLite
