@@ -32,7 +32,7 @@
 #include <Tensile/AMDGPUPredicates.hpp>
 #include <Tensile/Predicates.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Serialization
     {
@@ -135,7 +135,7 @@ namespace Tensile
         using HardwarePredicateSMT = SubclassMappingTraits<Predicates::Predicate<Hardware>, IO>;
 
         template <typename IO>
-        const typename HardwarePredicateSMT<IO>::SubclassMap HardwarePredicateSMT<IO>::subclasses
+        const typename HardwarePredicateSMT<IO>::SubclassMap SubclassMappingTraits<Predicates::Predicate<Hardware>, IO>::subclasses
             = HardwarePredicateSMT<IO>::GetSubclasses();
 
         template <typename IO>
@@ -178,7 +178,7 @@ namespace Tensile
         using AMDGPUPredicateSMT = SubclassMappingTraits<Predicates::Predicate<AMDGPU>, IO>;
 
         template <typename IO>
-        const typename AMDGPUPredicateSMT<IO>::SubclassMap AMDGPUPredicateSMT<IO>::subclasses
+        const typename AMDGPUPredicateSMT<IO>::SubclassMap SubclassMappingTraits<Predicates::Predicate<AMDGPU>, IO>::subclasses
             = AMDGPUPredicateSMT<IO>::GetSubclasses();
 
         template <typename IO>
@@ -214,6 +214,7 @@ namespace Tensile
                 iot::enumCase(io, value, "gfx940", AMDGPU::Processor::gfx940);
                 iot::enumCase(io, value, "gfx941", AMDGPU::Processor::gfx941);
                 iot::enumCase(io, value, "gfx942", AMDGPU::Processor::gfx942);
+                iot::enumCase(io, value, "gfx950", AMDGPU::Processor::gfx950);
                 iot::enumCase(io, value, "gfx1010", AMDGPU::Processor::gfx1010);
                 iot::enumCase(io, value, "gfx1011", AMDGPU::Processor::gfx1011);
                 iot::enumCase(io, value, "gfx1012", AMDGPU::Processor::gfx1012);
@@ -221,9 +222,54 @@ namespace Tensile
                 iot::enumCase(io, value, "gfx1100", AMDGPU::Processor::gfx1100);
                 iot::enumCase(io, value, "gfx1101", AMDGPU::Processor::gfx1101);
                 iot::enumCase(io, value, "gfx1102", AMDGPU::Processor::gfx1102);
+                iot::enumCase(io, value, "gfx1103", AMDGPU::Processor::gfx1103);
+                iot::enumCase(io, value, "gfx1150", AMDGPU::Processor::gfx1150);
+                iot::enumCase(io, value, "gfx1151", AMDGPU::Processor::gfx1151);
                 iot::enumCase(io, value, "gfx1200", AMDGPU::Processor::gfx1200);
                 iot::enumCase(io, value, "gfx1201", AMDGPU::Processor::gfx1201);
             }
         };
+
+        template <typename IO>
+        struct SubclassMappingTraits<Predicates::Predicate<Task>, IO>
+            : public DefaultSubclassMappingTraits<
+                  SubclassMappingTraits<Predicates::Predicate<Task>, IO>,
+                  Predicates::Predicate<Task>,
+                  IO>
+        {
+            using Self = SubclassMappingTraits<Predicates::Predicate<Task>, IO>;
+            using Base = DefaultSubclassMappingTraits<
+                SubclassMappingTraits<Predicates::Predicate<Task>, IO>,
+                Predicates::Predicate<Task>,
+                IO>;
+            using SubclassMap = typename Base::SubclassMap;
+            const static SubclassMap subclasses;
+
+            using Generic = PredicateMappingTraits<Task, IO>;
+
+            static SubclassMap GetSubclasses()
+            {
+                SubclassMap rv({Base::template Pair<Predicates::Contraction::WorkspaceCheck>()});
+
+                auto gmap = Generic::GetSubclasses();
+                rv.insert(gmap.begin(), gmap.end());
+
+                return rv;
+            }
+        };
+
+        template <typename IO>
+        using TaskPredicateSMT = SubclassMappingTraits<Predicates::Predicate<Task>, IO>;
+
+        template <typename IO>
+        const typename TaskPredicateSMT<IO>::SubclassMap SubclassMappingTraits<Predicates::Predicate<Task>,IO>::subclasses
+            = TaskPredicateSMT<IO>::GetSubclasses();     
+
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::WorkspaceCheck, IO>
+            : public AutoMappingTraits<Predicates::Contraction::WorkspaceCheck, IO>
+        {
+        };
+
     } // namespace Serialization
-} // namespace Tensile
+} // namespace TensileLite
