@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,12 +39,13 @@
 #include <Tensile/Serialization/Base.hpp>
 #include <Tensile/Serialization/Predicates.hpp>
 
-#include <Tensile/Serialization/DecisionTreeLibrary.hpp>
 #include <Tensile/Serialization/ExactLogicLibrary.hpp>
 #include <Tensile/Serialization/GranularitySelectionLibrary.hpp>
 #include <Tensile/Serialization/MapLibrary.hpp>
 #include <Tensile/Serialization/MatchingLibrary.hpp>
 #include <Tensile/Serialization/PlaceholderLibrary.hpp>
+#include <Tensile/Serialization/MLPClassificationLibrary.hpp>
+
 
 namespace TensileLite
 {
@@ -77,10 +78,11 @@ namespace TensileLite
                      Base::template Pair<ProblemSelectionLibrary<MyProblem, MySolution>>(),
                      Base::template Pair<ProblemMapLibrary<MyProblem, MySolution>>(),
                      Base::template Pair<ProblemFreeSizeLibrary<MyProblem, MySolution>>(),
+                     Base::template Pair<ProblemPredictionLibrary<MyProblem, MySolution>>(),
                      Base::template Pair<ProblemMatchingLibrary<MyProblem, MySolution>>(),
                      Base::template Pair<GranularitySelectionLibrary<MyProblem, MySolution>>(),
                      Base::template Pair<PlaceholderLibrary<MyProblem, MySolution>>(),
-                     Base::template Pair<DecisionTreeLibrary<MyProblem, MySolution>>()});
+                     Base::template Pair<MLPClassificationLibrary<MyProblem, MySolution>>()});
             }
         };
 
@@ -89,7 +91,7 @@ namespace TensileLite
 
         template <typename MyProblem, typename MySolution, typename IO>
         const typename dsmt<MyProblem, MySolution, IO>::SubclassMap
-            dsmt<MyProblem, MySolution, IO>::subclasses
+            SubclassMappingTraits<SolutionLibrary<MyProblem, MySolution>, IO>::subclasses
             = dsmt<MyProblem, MySolution, IO>::GetSubclasses();
 
         template <typename MyProblem, typename MySolution, typename IO>
