@@ -28,6 +28,7 @@
 
 #include <Tensile/AMDGPU.hpp>
 #include <Tensile/Tensile.hpp>
+#include <Tensile/analytical/Utils.hpp>
 
 #include <hip/hip_runtime.h>
 
@@ -42,6 +43,8 @@ namespace TensileLite
 
             hipDeviceProp_t properties;
 
+            std::shared_ptr<analytical::Hardware> analyticalHardware;
+
             virtual std::string archName() const override;
         };
 
