@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,18 @@
 
 namespace TensileLite
 {
+    // TODD- Currently, this map is for detecting CU-Fallback + runtime modification for XCC.
+    //       XCC is introduced since 942, so this is only targeting at gfx942 for now.
+    //       (i.e. Detect this on 942 only. For other devices, we don't have problems for CU-FB + mismatching XCC,
+    //        so all the other devices will be seen as standardCU)
+    //       THIS IS A TEMPORARILY WORKAROUND. WE MAY NEED TO COME UP WITH A BETTER WAY TO DETECT CU-FALLBACK
+    static const std::map<AMDGPU::Processor, int>& StandardCUMap()
+    {
+        static const std::map<AMDGPU::Processor, int> StandardCU_XCC
+            = {{AMDGPU::Processor::gfx942, 304}};
+        return StandardCU_XCC;
+    }
+
     TENSILE_API std::string AMDGPU::type() const
     {
         return Type();
@@ -40,6 +52,7 @@ namespace TensileLite
         , computeUnitCount(cus)
         , deviceName(name)
         , skDynamicGrid(getSKDynamicGrid())
+        , skDynamicWGM(getSKDynamicWGM())
         , skMaxCUs(getSKMaxCUs())
         , skGridMultiplier(getSKGridMultiplier())
         , skFixedGrid(getSKFixedGrid())
@@ -49,6 +62,27 @@ namespace TensileLite
 
     TENSILE_API AMDGPU::~AMDGPU() = default;
 
+    TENSILE_API bool AMDGPU::isStandardCU() const
+    {
+        // return the result if we already tested it.
+        if(isStandardCUs != -1)
+            return (isStandardCUs == 1);
+
+        // assume current device is a standard cu device.
+        isStandardCUs = 1;
+
+        auto mapIter = StandardCUMap().find(processor);
+        // NB: For any other devices not included in the map, we always see them as true (they have no XCC predicate issue.)
+        if(mapIter != StandardCUMap().end())
+        {
+            // check if current device is a standard cu devcie.
+            if(computeUnitCount != mapIter->second)
+                isStandardCUs = 0;
+        }
+
+        return (isStandardCUs == 1);
+    }
+
     TENSILE_API bool AMDGPU::runsKernelTargeting(AMDGPU::Processor other) const
     {
         if(other > this->processor)
