@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,39 +34,39 @@
 #include <cstddef>
 #include <set>
 
-namespace Tensile
+namespace TensileLite
 {
     ContractionProblem::ContractionProblem(size_t size, size_t workspaceSize)
         : m_workspaceSize(workspaceSize)
-        , m_f32XdlMathOp(DataType::Float)
+        , m_f32XdlMathOp(rocisa::DataType::Float)
     {
         m_tensors.resize(size);
         m_names.resize(size);
     }
 
-    ContractionProblemGemm ContractionProblemGemm::GEMM_Strides(bool     transA,
-                                                                bool     transB,
-                                                                DataType aType,
-                                                                DataType bType,
-                                                                DataType cType,
-                                                                DataType dType,
-                                                                size_t   m,
-                                                                size_t   n,
-                                                                size_t   k,
-                                                                size_t   batchSize,
-                                                                size_t   lda,
-                                                                size_t   aStride,
-                                                                size_t   ldb,
-                                                                size_t   bStride,
-                                                                size_t   ldc,
-                                                                size_t   cStride,
-                                                                size_t   ldd,
-                                                                size_t   dStride,
-                                                                double   beta)
+    ContractionProblemGemm ContractionProblemGemm::GEMM_Strides(bool             transA,
+                                                                bool             transB,
+                                                                rocisa::DataType aType,
+                                                                rocisa::DataType bType,
+                                                                rocisa::DataType cType,
+                                                                rocisa::DataType dType,
+                                                                size_t           m,
+                                                                size_t           n,
+                                                                size_t           k,
+                                                                size_t           batchSize,
+                                                                size_t           lda,
+                                                                size_t           aStride,
+                                                                size_t           ldb,
+                                                                size_t           bStride,
+                                                                size_t           ldc,
+                                                                size_t           cStride,
+                                                                size_t           ldd,
+                                                                size_t           dStride,
+                                                                double           beta)
     {
-        Tensile::ContractionProblemGemm::FreeIndices  free(2);
-        Tensile::ContractionProblemGemm::BoundIndices bound(1);
-        Tensile::ContractionProblemGemm::BatchIndices batch(1);
+        TensileLite::ContractionProblemGemm::FreeIndices  free(2);
+        TensileLite::ContractionProblemGemm::BoundIndices bound(1);
+        TensileLite::ContractionProblemGemm::BatchIndices batch(1);
 
         free[0].isA = true;
         free[0].i = free[0].c = free[0].d = 0;
@@ -150,10 +150,10 @@ namespace Tensile
 
         return GEMM_Strides(transA,
                             transB,
-                            DataType::Float,
-                            DataType::Float,
-                            DataType::Float,
-                            DataType::Float,
+                            rocisa::DataType::Float,
+                            rocisa::DataType::Float,
+                            rocisa::DataType::Float,
+                            rocisa::DataType::Float,
                             m,
                             n,
                             k,
@@ -177,8 +177,8 @@ namespace Tensile
                                                         TensorDescriptor const& d,
                                                         double                  beta)
     {
-        Tensile::ContractionProblemGemm::FreeIndices free(2);
-        BoundIndex                                   bound;
+        TensileLite::ContractionProblemGemm::FreeIndices free(2);
+        BoundIndex                                       bound;
 
         free[0].isA = true;
         free[0].i = free[0].c = free[0].d = 0;
@@ -393,13 +393,13 @@ namespace Tensile
     ContractionProblemGemm
         ContractionProblemGemm::FromIndexSizes(std::string const&         operationIdentifier,
                                                std::vector<size_t> const& indexSizes,
-                                               DataType                   aType,
+                                               rocisa::DataType           aType,
                                                std::vector<size_t> const& aStrides,
-                                               DataType                   bType,
+                                               rocisa::DataType           bType,
                                                std::vector<size_t> const& bStrides,
-                                               DataType                   cType,
+                                               rocisa::DataType           cType,
                                                std::vector<size_t> const& cStrides,
-                                               DataType                   dType,
+                                               rocisa::DataType           dType,
                                                std::vector<size_t> const& dStrides,
                                                double                     beta)
     {
@@ -439,13 +439,13 @@ namespace Tensile
                                                BatchIndices const&        batchIndices,
                                                BoundIndices const&        boundIndices,
                                                std::vector<size_t> const& indexSizes,
-                                               DataType                   aType,
+                                               rocisa::DataType           aType,
                                                std::vector<size_t> const& aStrides,
-                                               DataType                   bType,
+                                               rocisa::DataType           bType,
                                                std::vector<size_t> const& bStrides,
-                                               DataType                   cType,
+                                               rocisa::DataType           cType,
                                                std::vector<size_t> const& cStrides,
-                                               DataType                   dType,
+                                               rocisa::DataType           dType,
                                                std::vector<size_t> const& dStrides,
                                                double                     beta)
     {
@@ -564,9 +564,9 @@ namespace Tensile
         gemm.m_tensors[ContractionProblemGemm::TENSOR::SCALED] = TensorDescriptor("scaleD");
         gemm.m_tensors[ContractionProblemGemm::TENSOR::SCALEALPHAVEC]
             = TensorDescriptor("scaleAlphaVec");
-        gemm.m_tensors[ContractionProblemGemm::TENSOR::METADATA] = TensorDescriptor("metadata");
-        gemm.m_tensors[ContractionProblemGemm::TENSOR::AMAXD]    = TensorDescriptor("amaxD");
-        gemm.m_tensor_compressed                                 = TensorDescriptor("compressed");
+        gemm.m_tensors[ContractionProblemGemm::TENSOR::METADATA]   = TensorDescriptor("metadata");
+        gemm.m_tensors[ContractionProblemGemm::TENSOR::AMAXD]      = TensorDescriptor("amaxD");
+        gemm.m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED] = TensorDescriptor("compressed");
         return gemm;
     }
 
@@ -609,6 +609,7 @@ namespace Tensile
             m_beta); // Set enum using beta to potentially allow for faster solutions
         consistencyCheck();
         normalize();
+        calcArithmeticIntensity();
     }
 
     size_t ContractionProblemGemm::toAPos(size_t idx) const
@@ -641,6 +642,53 @@ namespace Tensile
         return found->i;
     }
 
+    size_t ContractionProblemGemm::getNumTiles(SizeMapping const& sizeMapping, size_t gsu) const
+    {
+        // Get the normal WorkGroup numbers by sizeMapping MacroTile
+        dim3 numWG(1, 1, 1);
+        for(size_t i = 0; i < m_freeIndicesA.size(); i++)
+        {
+            numWG.x *= m_freeSizesA.at(i);
+        }
+        for(size_t i = 0; i < m_freeIndicesB.size(); i++)
+        {
+            numWG.y *= m_freeSizesB.at(i);
+        }
+        for(size_t i = 0; i < m_batchIndices.size(); i++)
+        {
+            if(sizeMapping.packBatchDims & 0x1)
+                numWG.x *= m_batchSizes[i];
+            if(sizeMapping.packBatchDims & 0x2)
+                numWG.y *= m_batchSizes[i];
+            if(!sizeMapping.packBatchDims)
+                numWG.z *= m_batchSizes[i];
+        }
+
+        numWG.x = CeilDivide(numWG.x, sizeMapping.macroTile.x);
+        numWG.y = CeilDivide(numWG.y, sizeMapping.macroTile.y);
+        if(sizeMapping.streamK == 0)
+            numWG.y *= gsu;
+
+        size_t problemTiles = numWG.x * numWG.y;
+        if(sizeMapping.persistentKernelAlongBatch || sizeMapping.streamK != 0)
+            problemTiles *= numWG.z;
+
+        return problemTiles;
+    }
+
+    size_t ContractionProblemGemm::getItersPerTile(SizeMapping const& sizeMapping) const
+    {
+        size_t boundSize = 1;
+        for(size_t i = 0; i < m_boundIndices.size(); ++i)
+        {
+            boundSize *= m_boundSizes[i];
+        }
+
+        size_t itersPerTile = CeilDivide(boundSize, sizeMapping.depthU);
+
+        return itersPerTile;
+    }
+
     void ContractionProblemGemm::checkPersistentKernelEligibility(
         ContractionSolution const& solution, Hardware const& hardware)
     {
@@ -887,22 +935,23 @@ namespace Tensile
                                             ? 0
                                             : metadata_strides[i - 1] * metadata_sizes[i - 1];
             }
-            m_tensor_compressed = TensorDescriptor("compressed",
-                                                   aTensor.dataType(),
-                                                   compressed_sizes.begin(),
-                                                   compressed_sizes.end(),
-                                                   compressed_strides.begin(),
-                                                   compressed_strides.end());
+            m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED]
+                = TensorDescriptor("compressed",
+                                    m_sparse != 2 ? aTensor.dataType() : bTensor.dataType(),
+                                    compressed_sizes.begin(),
+                                    compressed_sizes.end(),
+                                    compressed_strides.begin(),
+                                    compressed_strides.end());
 
             m_tensors[ContractionProblemGemm::TENSOR::METADATA]
                 = TensorDescriptor("metadata",
-                                   DataType::Int8,
+                                   rocisa::DataType::Int8,
                                    metadata_sizes.begin(),
                                    metadata_sizes.end(),
                                    metadata_strides.begin(),
                                    metadata_strides.end());
 
-            m_allocatedElementsNonBatchCompressedA = 1;
+            m_allocatedElementsNonBatchCompressed = 1;
             for(int idx = 0; idx < compressed().dimensions(); idx++)
             {
                 bool isBatch = m_batchIndices.end()
@@ -912,16 +961,16 @@ namespace Tensile
                                                    return bi.a == idx;
                                                });
                 if(!isBatch)
-                    m_allocatedElementsNonBatchCompressedA
-                        += m_tensor_compressed.strides()[idx]
-                           * (m_tensor_compressed.sizes()[idx] - 1);
+                    m_allocatedElementsNonBatchCompressed
+                        += m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED].strides()[idx]
+                           * (m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED].sizes()[idx] - 1);
             }
         }
         else
         {
-            m_tensor_compressed                                 = TensorDescriptor("compressed");
-            m_tensors[ContractionProblemGemm::TENSOR::METADATA] = TensorDescriptor("metadata");
-            m_allocatedElementsNonBatchCompressedA              = 0;
+            m_tensors[ContractionProblemGemm::TENSOR::COMPRESSED] = TensorDescriptor("compressed");
+            m_tensors[ContractionProblemGemm::TENSOR::METADATA]   = TensorDescriptor("metadata");
+            m_allocatedElementsNonBatchCompressed                 = 0;
         }
     }
 
@@ -1020,6 +1069,42 @@ namespace Tensile
             TENSILE_ASSERT_EXC(dUse == 1);
     }
 
+    void ContractionProblemGemm::calcArithmeticIntensity()
+    {
+        size_t problemSize = 1;
+        for(size_t i = 0; i < m_problemSizes.size(); ++i)
+        {
+            problemSize *= m_problemSizes[i];
+        }
+        double gflop = 2 * problemSize * 1e-9;
+
+        size_t aSize = 1;
+        for(size_t i = 0; i < a().dimensions(); ++i)
+        {
+            aSize *= a().sizes()[i];
+        }
+        size_t bSize = 1;
+        for(size_t i = 0; i < b().dimensions(); ++i)
+        {
+            bSize *= b().sizes()[i];
+        }
+        size_t cSize = 1;
+        for(size_t i = 0; i < c().dimensions(); ++i)
+        {
+            cSize *= c().sizes()[i];
+        }
+        if(m_beta != 0) // If problem includes beta, update gflops and gbytes
+        {
+            gflop += 2 * cSize * 1e-9; // Include (+ beta * C) in gflops
+            cSize *= 2; // Include read C and write D in gbytes
+        }
+        double gbyte
+            = (aSize * a().elementBytes() + bSize * b().elementBytes() + cSize * c().elementBytes())
+              * 1e-9;
+
+        m_arithmeticIntensity = gflop / gbyte;
+    }
+
     size_t ContractionProblemGemm::freeSizeA(size_t idx) const
     {
         return m_freeSizesA.at(idx);
@@ -1224,25 +1309,25 @@ namespace Tensile
         return rv.str();
     }
 
-    ContractionProblemGemm
-        ContractionProblemGemm::createDefaultProblem(bool                   transA,
-                                                     bool                   transB,
-                                                     DataType               typeA,
-                                                     DataType               typeB,
-                                                     DataType               typeC,
-                                                     DataType               typeD,
-                                                     DataType               typeAlpha,
-                                                     DataType               typeBeta,
-                                                     DataType               typeComputeInput,
-                                                     DataType               typeCompute,
-                                                     double                 alpha,
-                                                     double                 beta,
-                                                     bool                   useBias,
-                                                     bool                   useGradient,
-                                                     std::vector<DataType>& biasDataTypeWhiteList,
-                                                     std::vector<int>&      biasSrcWhiteList,
-                                                     bool                   isGroupedGemm,
-                                                     size_t                 maxWorkspaceBytes)
+    ContractionProblemGemm ContractionProblemGemm::createDefaultProblem(
+        bool                           transA,
+        bool                           transB,
+        rocisa::DataType               typeA,
+        rocisa::DataType               typeB,
+        rocisa::DataType               typeC,
+        rocisa::DataType               typeD,
+        rocisa::DataType               typeAlpha,
+        rocisa::DataType               typeBeta,
+        rocisa::DataType               typeComputeInput,
+        rocisa::DataType               typeCompute,
+        double                         alpha,
+        double                         beta,
+        bool                           useBias,
+        bool                           useGradient,
+        std::vector<rocisa::DataType>& biasDataTypeWhiteList,
+        std::vector<int>&              biasSrcWhiteList,
+        bool                           isGroupedGemm,
+        size_t                         maxWorkspaceBytes)
     {
         assert(typeBeta == typeCompute);
         // Tensor descriptors for a, b
@@ -1315,43 +1400,44 @@ namespace Tensile
         // clang-format on
 
         // Descriptor for input matrix C
-        Tensile::TensorDescriptor c{"c", typeC, {m, n, batch_count}, {1, m, m * n}};
+        TensileLite::TensorDescriptor c{"c", typeC, {m, n, batch_count}, {1, m, m * n}};
 
         // Descriptor for output matrix D
-        Tensile::TensorDescriptor d{"d", typeD, {m, n, batch_count}, {1, m, m * n}};
+        TensileLite::TensorDescriptor d{"d", typeD, {m, n, batch_count}, {1, m, m * n}};
 
-        Tensile::TensorDescriptor e{"e"};
-        Tensile::TensorDescriptor bias{"bias"};
-        Tensile::TensorDescriptor scaleA("scaleA");
-        Tensile::TensorDescriptor scaleB("scaleB");
-        Tensile::TensorDescriptor scaleC("scaleC");
-        Tensile::TensorDescriptor scaleD("scaleD");
-        Tensile::TensorDescriptor scaleAlpha{"scaleAlpha"};
+        TensileLite::TensorDescriptor e{"e"};
+        TensileLite::TensorDescriptor bias{"bias"};
+        TensileLite::TensorDescriptor scaleA("scaleA");
+        TensileLite::TensorDescriptor scaleB("scaleB");
+        TensileLite::TensorDescriptor scaleC("scaleC");
+        TensileLite::TensorDescriptor scaleD("scaleD");
+        TensileLite::TensorDescriptor scaleAlpha{"scaleAlpha"};
 
         // The ContractionProblemGemm
-        Tensile::ContractionProblemGemm problem{a,
-                                                b,
-                                                c,
-                                                d,
-                                                e,
-                                                bias,
-                                                scaleA,
-                                                scaleB,
-                                                scaleC,
-                                                scaleD,
-                                                scaleAlpha,
-                                                freeIndex,
-                                                batchIndex,
-                                                boundIndex,
-                                                beta,
-                                                maxWorkspaceBytes};
+        TensileLite::ContractionProblemGemm problem{a,
+                                                    b,
+                                                    c,
+                                                    d,
+                                                    e,
+                                                    bias,
+                                                    scaleA,
+                                                    scaleB,
+                                                    scaleC,
+                                                    scaleD,
+                                                    scaleAlpha,
+                                                    freeIndex,
+                                                    batchIndex,
+                                                    boundIndex,
+                                                    beta,
+                                                    maxWorkspaceBytes};
 
         problem.setComputeInputType(typeComputeInput);
         problem.setAlphaType(typeAlpha);
         problem.setBetaType(typeBeta);
 
         // HPA is active iff sizeof(compute type) > sizeof(input type)
-        problem.setHighPrecisionAccumulate(GetElementSize(typeCompute) > GetElementSize(typeA));
+        problem.setHighPrecisionAccumulate(rocisa::GetElementSize(typeCompute)
+                                           > rocisa::GetElementSize(typeA));
 
         // set batch mode
         problem.setStridedBatched(true);
@@ -1364,11 +1450,11 @@ namespace Tensile
         // set bias mode
         if(useBias)
         {
-            DataType                                biasType = biasDataTypeWhiteList[0];
-            Tensile::ContractionProblemGemm::TENSOR biasSrc
-                = static_cast<Tensile::ContractionProblemGemm::TENSOR>(biasSrcWhiteList[0]);
+            rocisa::DataType                            biasType = biasDataTypeWhiteList[0];
+            TensileLite::ContractionProblemGemm::TENSOR biasSrc
+                = static_cast<TensileLite::ContractionProblemGemm::TENSOR>(biasSrcWhiteList[0]);
             problem.setBias(biasType, 1, 0, useGradient, biasSrc);
-            problem.setParams().setBiasEnum(Tensile::DataType::None);
+            problem.setParams().setBiasEnum(rocisa::DataType::None);
         }
 
         // Add problem predicates for CEqualsD
@@ -1427,6 +1513,7 @@ namespace Tensile
                                          void const*          _b,
                                          void const*          _c,
                                          void*                _d,
+                                         void*                _e,
                                          void const* const*   _batchA,
                                          void const* const*   _batchB,
                                          void const* const*   _batchC,
@@ -1440,11 +1527,13 @@ namespace Tensile
                                          void const*          _scaleAlphaVec,
                                          void*                _ws,
                                          void*                _Synchronizer,
-                                         unsigned char const* _metadata)
+                                         unsigned char const* _metadata,
+                                         void const*          _compressed)
         : a(_a)
         , b(_b)
         , c(_c)
         , d(_d)
+        , e(_e)
         , batchA(_batchA)
         , batchB(_batchB)
         , batchC(_batchC)
@@ -1459,6 +1548,22 @@ namespace Tensile
         , ws(_ws)
         , Synchronizer(_Synchronizer)
         , metadata(_metadata)
+        , compressed(_compressed)
     {
     }
-} // namespace Tensile
+
+    ContractionInputs::ContractionInputs(void const*     _a,
+                                         void const*     _b,
+                                         void const*     _c,
+                                         void*           _d,
+                                         ConstantVariant _alpha,
+                                         ConstantVariant _beta)
+        : a(_a)
+        , b(_b)
+        , c(_c)
+        , d(_d)
+        , alpha(_alpha)
+        , beta(_beta)
+    {
+    }
+} // namespace TensileLite
