@@ -29,7 +29,7 @@
 
 #include <algorithm>
 
-namespace Tensile
+namespace TensileLite
 {
     std::string ToString(KernelLanguage d)
     {
@@ -168,4 +168,4 @@ namespace Tensile
 
         return stream;
     }
-} // namespace Tensile
+} // namespace TensileLite
