@@ -35,7 +35,7 @@
 #include <Tensile/hip/HipUtils.hpp>
 
 //@TODO add alternative for windows
-#ifndef WIN32
+#ifndef _WIN32
 #include <glob.h>
 #endif
 #include <regex>
@@ -256,6 +256,10 @@ namespace TensileLite
                 {
                     return err;
                 }
+                else
+                {
+                    (void)hipGetLastError(); // clear hipErrorNotFound
+                }
             }
 
             return err;
@@ -294,6 +298,8 @@ namespace TensileLite
 
                     if(err == hipSuccess)
                         return err;
+                    else if(err == hipErrorFileNotFound)
+                        (void)hipGetLastError(); // clear hipErrorFileNotFound
                 }
 
                 return err;
