@@ -160,6 +160,10 @@ namespace TensileLite
             {
                 return AMDGPU::Processor::gfx942;
             }
+            else if(deviceString.find("gfx950") != std::string::npos)
+            {
+                return AMDGPU::Processor::gfx950;
+            }
             else if(deviceString.find("gfx1010") != std::string::npos)
             {
                 return AMDGPU::Processor::gfx1010;
