@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@ import subprocess
 import yaml
 
 from Tensile import Tensile
-from Tensile.TensileInstructions import DataType
+from Tensile.Common.DataType import DataType
 
 ################################################################################
 # Locate Executables
