@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2024 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2024-2025 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,8 @@ import os
 import subprocess
 import math
 import numpy as np
+import concurrent.futures
+
 
 # Paths to the input and output files
 parser = argparse.ArgumentParser(description="""Generate Tensile config file""")
@@ -88,26 +90,44 @@ LibraryType = "GridBased"
 
 CU_RE = r"Compute Unit:(?P<COMPUTE_UNIT>[\w ]+)"
 
-res = subprocess.run("/opt/rocm/llvm/bin/offload-arch", stdout=subprocess.PIPE)
-ArchitectureName = res.stdout.decode("utf-8").strip()
-res = subprocess.run("rocminfo | grep Compute", stdout=subprocess.PIPE, shell=True, env={"ROCR_VISIBLE_DEVICES":"0"})
-match = re.search(CU_RE, res.stdout.decode("utf-8").split('\n')[-2])
 NUM_STAGES = args.num_stages
 DIV_MI = 3 # 33.3%
 MIN_MI = 5 # min 5 solutions
 NONTEMPORALRATIO = 8
-CU = 0
-if match:
-    CU = int(match.group('COMPUTE_UNIT').strip())
-else:
-    raise RuntimeError("Failed to get compute unit from rocminfo")
+MAX_MT = int(os.environ.get("MAX_MT", 256))
 
-if ArchitectureName == 'gfx942':
-    res = subprocess.run(["cat", "/sys/class/drm/card1/device/current_compute_partition"], stdout=subprocess.PIPE)
-    if res.stdout.decode("utf-8").strip() == "CPX":
-        XCC = 1
+OFFLOAD_ARCH = "/opt/rocm/llvm/bin/offload-arch"
+NUM_INST = "/sys/class/drm/card1/device/compute_partition_config/xcc/num_inst"
+
+ArchitectureName = os.environ.get("GPU_TARGET", None)
+if ArchitectureName is None:
+    if os.path.exists(OFFLOAD_ARCH):
+        res = subprocess.run(OFFLOAD_ARCH, stdout=subprocess.PIPE)
+        ArchitectureName = res.stdout.decode("utf-8").strip()
     else:
-        XCC = 4
+        raise FileNotFoundError(f"{OFFLOAD_ARCH} not found, please specific GPU_TARGET environment variable.")
+
+CU = os.environ.get("CU", None)
+if CU is None:
+    res = subprocess.run("rocminfo | grep Compute", stdout=subprocess.PIPE, shell=True, env={"ROCR_VISIBLE_DEVICES":"0"})
+    match = re.search(CU_RE, res.stdout.decode("utf-8").split('\n')[-2])
+    if match:
+        CU = int(match.group('COMPUTE_UNIT').strip())
+    else:
+        raise RuntimeError("Failed to get compute unit from rocminfo, please specific CU environment variable.")
+else:
+    CU = int(CU)
+
+XCC = os.environ.get("XCC", None)
+if ArchitectureName == 'gfx942':
+    if XCC is None:
+        if os.path.exists(NUM_INST):
+            res = subprocess.run(["cat", NUM_INST], stdout=subprocess.PIPE)
+            XCC = int(res.stdout.decode("utf-8").strip())
+        else:
+            raise FileNotFoundError(f"{NUM_INST} not found, please specific XCC environment variable.")
+    else:
+        XCC = int(XCC)
     DeviceNames = ["Device 0049", "Device 0050"]
     ScheduleName = "aquavanjaram"
 elif ArchitectureName == 'gfx90a':
@@ -122,15 +142,15 @@ if args.full_mfma:
     fp32_instructions = [[32,32,1,2], [32,32,2,1], [16,16,1,4], [16,16,4,1], [4,4,1,16]]
     fp8_instructions = [[32,32,16,1], [16,16,32,1]]
 else:
-    fp16_instructions = [[16,16,16,1]]
-    bf16_instructions = [[16,16,16,1],[32,32,8,1]]
-    tf32_instructions = [[16,16,8,1]]
-    fp32_instructions = [[16,16,4,1]]
+    fp16_instructions = [[16,16,16,1], [32,32,8,1]]
+    bf16_instructions = [[16,16,16,1], [32,32,8,1]]
+    tf32_instructions = [[16,16,8,1], [32,32,4,1]]
+    fp32_instructions = [[16,16,4,1], [32,32,2,1]]
     fp8_instructions = [[16,16,32,1]]
 
 
 HIPBLASLT_BENCH_BASE = (
-    r"(?P<CMD>\w+) --api_method c "
+    r"hipblaslt-bench --api_method (?P<API_METHOD>\w+) "
     r"-m (?P<M>[\d ]+)"
     r"-n (?P<N>[\d ]+)"
     r"-k (?P<K>[\d ]+)"
@@ -180,6 +200,11 @@ def build_pattern(has_bias=False):
 HIPBLASLT_BENCH_RE = build_pattern()
 HIPBLASLT_BENCH_RE_BIAS = build_pattern(has_bias=True)
 
+FP8_DTYPES = ["F8","F8N","B8","B8N","F8B8N","B8F8N","F8B8","B8F8"]
+
+def is_fp8(dtype):
+    return dtype in FP8_DTYPES
+
 # Function to extract problem sizes from a line
 def extract_problem_size(match):
     return [int(match.group('M').strip()), int(match.group('N').strip()), int(match.group('BATCH_COUNT').strip()), int(match.group('K').strip())]
@@ -193,7 +218,7 @@ def instruction_map(dtype_dict):
         return fp16_instructions
     elif dtype_dict["DataType"] == 'B':
         return bf16_instructions
-    elif dtype_dict["DataType"] == 'F8':
+    elif is_fp8(dtype_dict["DataType"]):
         return fp8_instructions
     else:
         return None
@@ -209,6 +234,8 @@ def datatype_map(dtype):
         return "B"
     elif dtype == "f8_r":
         return "F8"
+    elif dtype == "bf8_r":
+        return "B8"
     else:
         return None
 
@@ -235,7 +262,7 @@ def bias_datatype_map(dtype):
         return []
 
 def get_high_precision_accumulate(DataType):
-    if DataType in ["H", "B", "F8"]:
+    if DataType in ["H", "B"] + FP8_DTYPES:
         return True
     else:
         return False
@@ -248,7 +275,21 @@ def adapt_xf32(ComputeDataType):
 
 def extract_dtype(match):
     gdict = match.groupdict()
-    DataType = datatype_map(gdict.get('A_TYPE', '').strip())
+    DataTypeA = datatype_map(gdict.get('A_TYPE', '').strip())
+    DataTypeB = datatype_map(gdict.get('B_TYPE', '').strip())
+
+    if is_fp8(DataTypeA) and is_fp8(DataTypeB):
+        if DataTypeA != DataTypeB:
+            DataType = DataTypeA + DataTypeB
+        else:
+            DataType = DataTypeA
+
+        if ArchitectureName == "gfx942":
+            # Nanno FP8
+            DataType += "N"
+    else:
+        DataType = DataTypeA
+    
     DestDataType = datatype_map(gdict.get('C_TYPE', '').strip())
     ComputeDataType = datatype_map(gdict.get('COMPUTE_TYPE', '').strip())
     TransposeA = trans_map(gdict.get('TRANS_A', '').strip())
@@ -283,14 +324,14 @@ def find_matmul_instruction(mfma_instruction, size):
         for m_tiles in reversed(range(1, CU+1)):
             m_tile_size = size[0] // m_tiles
             # TODO:fp8 384x384
-            if m_tile_size > 256:
+            if m_tile_size > MAX_MT:
                 continue
             wave_tile_m = math.ceil(m_tile_size / mfma_instruction[0])
             if wave_tile_m <= 0:
                 continue
             for n_tiles in reversed(range(1, CU+1)):
                 n_tile_size = size[1] // n_tiles
-                if n_tile_size > 256:
+                if n_tile_size > MAX_MT:
                     continue
                 wave_tile_n = math.ceil(n_tile_size / mfma_instruction[1])
                 if wave_tile_n <= 0:
@@ -325,15 +366,20 @@ def get_groups(matmul_instruction_gen):
     return mi_groups0, mi_groups1, mi_left
 
 def match_pattern(line):
-    if 'bias_vector' in line:
-        match = re.search(
-            HIPBLASLT_BENCH_RE_BIAS, line
-        )
+    if line.startswith("hipblaslt-bench"):
+        if 'bias_vector' in line:
+            match = re.search(
+                HIPBLASLT_BENCH_RE_BIAS, line
+            )
+        else:
+            match = re.search(
+                HIPBLASLT_BENCH_RE, line
+            )
+        if match is None:
+            print("WARNING: can't find match for", line)
+        return match
     else:
-        match = re.search(
-            HIPBLASLT_BENCH_RE, line
-        )
-    return match
+        return None
 
 def extract_range(data):
     shapes = []
@@ -354,11 +400,11 @@ def split_gemms_by_gpus(unique_gemms, gpus):
             unique_gemms_subgroups[i%gpus] = [(k, v)]
     return unique_gemms_subgroups
 
-def calculate_min_flops(m_sum, n_sum, batch_sum, k_sum, iters):
-    m_avg = m_sum / len(unique_gemms_subgroup)
-    n_avg = n_sum / len(unique_gemms_subgroup)
-    batch_avg = batch_sum / len(unique_gemms_subgroup)
-    k_avg = k_sum / len(unique_gemms_subgroup)
+def calculate_min_flops(m_sum, n_sum, batch_sum, k_sum, samples_num, iters):
+    m_avg = m_sum / samples_num
+    n_avg = n_sum / samples_num
+    batch_avg = batch_sum / samples_num
+    k_avg = k_sum / samples_num
 
     return (ENQUEUES_PER_SYNC + iters) * m_avg * n_avg * batch_avg * k_avg / 2
 
@@ -367,11 +413,12 @@ def calculate_gsu(matmul_instruction, size):
     mt1 = matmul_instruction[1] * matmul_instruction[6] * matmul_instruction[8]
     return max(1, CU // (math.ceil(size[0] / mt0) * math.ceil(size[1] / mt1)))
 
-def dump_yaml(gpu_idx, gemm_group, yaml_file, m_sum, n_sum, batch_sum, k_sum, iters, groups, gsu_group):
-    MinFlopsPerSync = calculate_min_flops(m_sum, n_sum, batch_sum, k_sum, iters)
+def dump_yaml(gpu_idx, gemm_group, yaml_file, m_sum, n_sum, batch_sum, k_sum, samples_num, iters, groups, gsu_group, matmul_instructions):
+    MinFlopsPerSync = calculate_min_flops(m_sum, n_sum, batch_sum, k_sum, samples_num, iters)
     # Read the YAML file
     with open(yaml_file, 'r') as f:
         data = yaml.safe_load(f)
+
     data["GlobalParameters"]["EnqueuesPerSync"] = ENQUEUES_PER_SYNC
     data["GlobalParameters"]["MaxEnqueuesPerSync"] = iters
     data["GlobalParameters"]["NumWarmups"] = NUM_WARM_UP
@@ -440,8 +487,10 @@ def dump_yaml(gpu_idx, gemm_group, yaml_file, m_sum, n_sum, batch_sum, k_sum, it
     # Write the updated YAML file
     yaml_file = os.path.basename(yaml_file)
     slices = yaml_file.split('.')
-    with open(slices[0]+'.'+str(gpu_idx)+'.'+slices[1], 'w') as f:
+    fname = slices[0]+'.'+str(gpu_idx)+'.'+slices[1]
+    with open(fname, 'w') as f:
         yaml.dump(data, f, default_flow_style=None)
+    print(f"Dumped yaml to {fname}")
 
 
 if args.hipblaslt_log and args.gridbase_config is None:
@@ -449,16 +498,25 @@ if args.hipblaslt_log and args.gridbase_config is None:
     unique_gemms = {}
     # Read problem sizes from the input file
     with open(args.hipblaslt_log, 'r') as f:
-        for line in f:
+        lines = f.readlines()
+        def _extract_gemms(line):
             match = match_pattern(line)
             if match:
                 size = extract_problem_size(match)
                 dtype = extract_dtype(match)
                 if dtype is None:
-                    print(f"Can't find dtype for {line}, please contact hipblaslt expert")
-                    continue
+                    print(f"WARNING: Can't find dtype for {line}, please contact hipblaslt expert")
+                    return None
                 size_str = json.dumps(size)
                 dtype_str = json.dumps(dtype)
+                return (size_str, dtype_str)
+            return None
+
+        with concurrent.futures.ProcessPoolExecutor(8) as executor:
+            results = executor.map(_extract_gemms, list(lines))
+        for res in results:
+            if res is not None:
+                (size_str, dtype_str) = res
                 if (size_str, dtype_str) in unique_gemms:
                     unique_gemms[(size_str, dtype_str)] += 1
                 else:
@@ -470,19 +528,19 @@ if args.hipblaslt_log and args.gridbase_config is None:
 
     unique_gemms_subgroups = split_gemms_by_gpus(unique_gemms, args.gpus)
 
-    for gpu_idx, unique_gemms_subgroup in enumerate(unique_gemms_subgroups):
+    def _process_gemms(item):
+        gpu_idx, unique_gemms_subgroup = item
         gemm_group = {}
         gsu_group = {}
         matmul_instructions = {}
         groups = {}
         if unique_gemms_subgroup is None:
-            continue
+            return None
 
         m_sum = 0
         n_sum = 0
         batch_sum = 0
         k_sum = 0
-
         for k, v in unique_gemms_subgroup:
             size_str, dtype_str = k
             original_size = json.loads(size_str)
@@ -546,7 +604,7 @@ if args.hipblaslt_log and args.gridbase_config is None:
                     size[max_dim] = size[max_dim] // 2
 
             if not matmul_instruction_found:
-                print(f"Can't find mfma instructions for {original_size}, please contact hipblaslt expert")
+                print(f"WARNING: Can't find mfma instructions for {original_size}, please contact hipblaslt expert")
             else:
                 if dtype_str in gemm_group:
                     gemm_group[dtype_str].append({'Exact': list(original_size)})
@@ -556,8 +614,9 @@ if args.hipblaslt_log and args.gridbase_config is None:
                 n_sum += original_size[1]
                 batch_sum += original_size[2]
                 k_sum += original_size[3]
+        samples_num = len(unique_gemms_subgroup)
+        return dump_yaml(gpu_idx, gemm_group, args.tensile_config, m_sum, n_sum, batch_sum, k_sum, samples_num, args.iters, groups, gsu_group, matmul_instructions)
 
-        dump_yaml(gpu_idx, gemm_group, args.tensile_config, m_sum, n_sum, batch_sum, k_sum, args.iters, groups, gsu_group)
 
 elif args.gridbase_config and args.hipblaslt_log is None:
     LibraryType = "GridBased"
@@ -588,7 +647,8 @@ elif args.gridbase_config and args.hipblaslt_log is None:
 
     unique_gemms_subgroups = split_gemms_by_gpus(unique_gemms, args.gpus)
 
-    for gpu_idx, unique_gemms_subgroup in enumerate(unique_gemms_subgroups):
+    def _process_gemms(item):
+        gpu_idx, unique_gemms_subgroup = item
         gemm_group = {}
         matmul_instructions = {}
         gsu_group = {}
@@ -630,7 +690,7 @@ elif args.gridbase_config and args.hipblaslt_log is None:
                     size[max_dim] = size[max_dim] // 2
 
             if not matmul_instruction_found:
-                print(f"Can't find mfma instructions for {original_size}, please contact hipblaslt expert")
+                print(f"WARNING: Can't find mfma instructions for {original_size}, please contact hipblaslt expert")
             else:
                 if dtype_str in gemm_group:
                     gemm_group[dtype_str].append({'Exact': list(original_size)})
@@ -641,5 +701,11 @@ elif args.gridbase_config and args.hipblaslt_log is None:
                 n_sum += original_size[1]
                 batch_sum += original_size[2]
                 k_sum += original_size[3]
+        samples_num = len(unique_gemms_subgroup)
+        return dump_yaml(gpu_idx, gemm_group, args.tensile_config, m_sum, n_sum, batch_sum, k_sum, samples_num, args.iters, {}, gsu_group, matmul_instructions)
 
-        dump_yaml(gpu_idx, gemm_group, args.tensile_config, m_sum, n_sum, batch_sum, k_sum, args.iters, {}, gsu_group)
+with concurrent.futures.ProcessPoolExecutor(args.gpus) as executor:
+    results = executor.map(_process_gemms, list(enumerate(unique_gemms_subgroups)))
+    for result in results:
+        if result is not None:
+            print(result)
